//  $Id: fconfig.C,v 1.3 1996/10/26 19:13:11 shaggy Exp $
//      file configuration utility
//      
//      relative path: ./fconfig
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       File configuration utility
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

#define fconfig_VER_MAJOR	0
#define fconfig_VER_MINOR	32
#define fconfig_VER_PATCH	1
#define fconfig_VER_STR	"0.32.1"
#define fconfig_VER_DATE	"October 06, 1996"
#define fconfig_VER_NDATE	"10/06/96"

//  $Log: fconfig.C,v $
//  Revision 1.3  1996/10/26  19:13:11  shaggy
//  header update
//
//  Revision 1.2  1996/10/06  13:42:53  shaggy
//  fconfig 0.32.1 (October 06, 1996: 09:42:50)
//  	fixed up for RCS version control
//
//  Revision 1.1  1996/10/06  13:41:56  shaggy
//  Initial revision
//
//
//	fconfig v0.32.0 (August 25, 1996: 13:18:17)
//	First version with new version control
//
//	Pre-version-control  History:
//	v0.31		first release
//	v0.02-0.30	internal development versions
//	v0.01		orginally created in January, 1995


#include <sys/types.h>
#include <sys/wait.h>
#include <stdio.h>
#include <string.h>
#include <dirent.h>
#include <unistd.h>
#include <stdlib.h>
#include <limits.h>


#ifndef PATH_MAX
#define PATH_MAX	1024	// pathname max size
#endif
#ifndef NAME_MAX
#define NAME_MAX	256	// filename max size
#endif
#define VNAME_MAX	128	// variable name max size
#ifndef LINE_MAX
#define LINE_MAX	256	// input line max size
#endif
#define CMD_MAX		8192	// command buffer max size
#define BUF_MAX		65536	// temporary buffer max size
#define MAX_FLAGS	128	// max # of flags


// notes:
//	allow files names "file.C.head" that will
//	be included prior to all "file.C" files
//	give option to join files physically together
//	or use cpp to join them at compile time

// called with:
//	fconfig (machtypes...) (src...) spath files ...
//		machtypes: a single argument that is a list of
//			   of machine type specifications
//		src:	   list of paths from the current
//			   directory to each source tree
//		spath:	   path from the start of the source tree
//			   to the current source directory
//		files:	   all remaining arguments are the names
//			   of the source files to configure
// uses environment variables:
//	Autoconfig: set this flag for auto-configuration (0 or false disables)
//	Command_c: (gcc) command for compiling .c files	(gcc)
//	Command_C: (gxx) command for compiling .C files	(g++)
//	Command_S: (as)	command for compiling .S files 	(as)
//	etc...
//	Flags_Needed:	flags needed for any file type
//	Flags_Needed_c:	defaults flags needed for .c files
//	Flags_Wanted_c: desired flags for .c files
//	Flags_Banned_c: flags banned from .c files
//	Flags_Needed_gxx:	flags needed for g++
//	Makedepend_c:	command for making .c dependencies (gcc -M)
//	Cpp_c:		command for running cpp on .c files (gcc -E)
//
// variable substitutions:
//	$@	target name
//	$<	source name
//	$^	names of all dependencies? (not supported)
//	$*	the stem of the filename
//
// job:
//	searches source area and all appropriate subdirectories
//	for files with the same name
//	i.e.:	machtype = i486 pc i386
//	search includes: . m-i386 m-i486 m-pc m-vax-i386 m-68k-i486 etc
//	builds a local "file.C" that contains #include references
//	to all the other parts
//	run through "gcc -M" to get dependency listing
//	run through "gcc -E | grep "#pragma Make"" to get all commands
//	define env vars for each pragma that starts with Make
//	check for conflicts, and generate command and flags
//	for building each target
//
//	Generate "file.C.o.mak" that consists of:
//		file.o file.o.mak: file.C ../../src/blah/file.C etc...
//		file.o: 
//			gcc -c -O2 -fforce-mem -funroll-loops file.C
//		file.o.mak:
//			fconfig "m-i386 m-pc" "../../src" blah file.C
//
//	pragma directives:
//		#pragma Make Exclusive
//		#pragma	Make Command = gcc -c
//		#pragma	Make Flags_Needed = -fforce-mem
//		#pragma Make Flags_Wanted = -funroll-loops
//		#pragma	Make Flags_Banned = -pedantic
//	future:
//		#pragma Main #pragma implementation
//		#pragma Implementation
//		#pragma Interface
// errors:
//	o	should only be one "Command_ext_etc.." var for each
//			file extension (ext)
//	o	should never have conflict of flags needed and
//			flags banned


const char *const*machtypes;
int nmach;
const char *const*src;
int nsrc;
const char *spath;
const char *const*argvp;

const char *target_name;	// target filename
const char *source_name;	// source filename
const char *base_name;		// current basename (stem) of source
const char *extensions;		// string of possible extensions
bool dest_makefile;		// true if writing flags to makefile

FILE *logfile;

bool exclusive;		// signal for #pragma Make exclusive

bool autoconfig = false;


class DirQ {
	DirQ	*next;
	DirQ	*prev;
	char	*path;
	char	*name;
	char	*ref;	// reference extension (i.e. m-i386-d2)
	char	*ext;	// source extension expected
public:
	DirQ()
		{ name = 0; next = this; prev = this; }
	DirQ(char *newpath, char *newname, char *newref)
		{ path = newpath; name = newname; ref = newref;
		  next = this; prev = this; ext = ""; }
	DirQ(char *newpath, char *newname, char *newref, char *newext)
		{ path = newpath; name = newname; ref = newref;
		  next = this; prev = this; ext = newext; }
	void	add_head(DirQ *d)
		{ d->next = next; d->prev = this; next->prev = d; next = d; }
	void	add_tail(DirQ *d)
		{ d->prev = prev; d->next = this; prev->next = d; prev = d; }
	DirQ*	_next() const
		{ if (next->name) return next;
		  else return 0; }
	DirQ*	_prev() const
		{ if (prev->name) return prev;
		  else return 0; }
	DirQ*	remove() 
		{ next->prev = prev; prev->next = next;
		  prev = 0; next = 0; return this; }
	char*	_name()	const { return name; }
	char*	_path() const { return path; }
	char*	_ref() const	{ return ref; }
	char*	_ext() const	{ return ext; }
};


DirQ	dirs;


const char* getnext(const char **ap, char *buf)
{
char *bp = buf;
	if(!*ap)
		return 0;
	while(**ap && **ap <= ' ')
		(*ap)++;
	if(!**ap)
		return 0; 
	while(**ap && **ap > ' ')
		*bp++ = *(*ap)++;
	*bp = '\0';
	return buf;
}

const char* getnext_ext(const char **ap, const char *name, char *buf)
{
char *bp = buf;
	while(*name)
		*bp++ = *name++;
	if(!*ap) 
		return 0;
	*bp = '_';
	if(!getnext(ap, bp+1)) {
		*ap = 0;	
		*bp = '\0';
	}
	return buf;
}


const char*const* parse_list(const char *str, int *size)
{
const char *cp;
char buf[VNAME_MAX];
int count;
char **list;
int i;
	cp = str;
	count = 0;
	while(getnext(&cp, buf)) 
		count++;
	list = new char*[count+1];
	cp = str;
	for(i=0; i<count; i++) 
		list[i] = strdup(getnext(&cp, buf));
	list[i] = 0;
	*size = count;
	return list;
}


bool mach_match(const char *type, const char *name)
{
const char *cp, *tp;
	if(!strcmp(type, name))
		return true;
	if(name[0] != 'm' || name[1] != '-')
		return false;
	cp = name;
	while(*cp) {
		while(*cp != '-')
			if(!*cp++)
				return false;
		for(tp = type; *(++cp) == *tp && *tp; tp++)
			continue;
		if(!*tp && (!*cp || *cp == '-')) 
			return true;
	}
	return false;
}


void find_dirs()
{
int srcdir;
DIR *dptr;
struct dirent *d;
DirQ dq;
DirQ *dqp, *dnext;
char path[PATH_MAX];
char dname[NAME_MAX];
char ref[NAME_MAX];
int mach;
	for(srcdir=nsrc-1; srcdir>=0; srcdir--) {
		strcpy(path, src[srcdir]);
		if(path[strlen(path)] != '/')
			strcat(path, spath);
		if(!(dptr = opendir(path))) {
			perror(path);
			continue;
		}
		if(srcdir)
			sprintf(dname, "-d%d", srcdir);
		else
			dname[0] = '\0';
		while((d = readdir(dptr))) {
			sprintf(ref, "-%s%s", d->d_name, dname);
			if(!strncmp(d->d_name, "m-", 2)) 
				dq.add_tail(new DirQ(path,
				    strdup(d->d_name), strdup(ref)));
		}
		closedir(dptr);
		sprintf(ref, "-mi%s", dname);
		dq.add_tail(new DirQ(path, ".", strdup(ref)));
	}
	for(mach=0; mach<nmach; mach++) {
		dqp = dq._next();
		if(!strcmp(machtypes[mach], "gen"))
			dirs.add_tail(new DirQ("./", ".", "-mg", ".mg"));
		while(dqp) {
			dnext = dqp->_next();
			if(mach_match(machtypes[mach], dqp->_name()))
				dirs.add_tail(dqp->remove());
			dqp = dnext;
		}
	}
}



void convert_fname(char *fname)
{
	while(*fname) {
		if((*fname < 'a' || *fname > 'z') 
				&& (*fname < 'A' || *fname > 'Z')
				&& (*fname < '0' || *fname > '9'))
			*fname = '_';
		fname++;
	}
}


char* get_var(const char *name, char *varbuf);


void strcat_subst(char *buf, const char *str)
{
const char *src;
char varname[VNAME_MAX];
char *bp;
	while(*buf)
		buf++;
	while(*str)	
		if(*str == '$') {
			switch(str[1]) {
			case '@':	src = target_name; str++; break;
			case '<':	src = source_name; str++; break;
			case '*':	src = base_name; str++; break;
			case '(':
				str++;
				bp = varname; 
				while(*(++str) != ')')
					*bp++ = *str;
				*bp = '\0';
				get_var(varname, buf);
				src = buf;
				break;
			default:	
				if(dest_makefile)	
					src = "$$";
				else
					src = "$";
			}
			str++;
			while(*src)
				*buf++ = *src++;
		} else
			*buf++ = *str++;
	*buf = '\0';
}



const int WANTED	= 1;
const int NEEDED	= 2;
const int BANNED	= 4;

const int NOTUSED	= 0;
const int USED		= 1;
const int CONFLICT	= 2;


const int flag_action[] =
	{NOTUSED, USED, USED, USED,
	NOTUSED, NOTUSED, CONFLICT, CONFLICT};


static struct {
	char	str[VNAME_MAX];
	int	type;
} flaglist[MAX_FLAGS];

static char	conflictbuf[CMD_MAX];
static char	nflags;


void add_flags(const char *fmt, const char *name, int type)
// search 'extensions' and match all combinations of
//	'name_ext1_ext2' and 'name_ext'
{
const char *ext = extensions;
const char *ext2;
const char *varname;
const char *varname2;
char buf[VNAME_MAX];
char buf2[VNAME_MAX];
char namestr[VNAME_MAX];
const char *val;
int i;
const char *flag;
	sprintf(namestr, fmt, name);
	while((varname2 = getnext_ext(&ext, namestr, buf))) {
	    varname = varname2;
	    ext2 = extensions;
	    do {
	        if((val = getenv(varname))) {
		    while((flag = getnext(&val, buf))) {
			for(i=0; i<nflags; i++) {
				if(!strcmp(flaglist[i].str, flag)) {
					flaglist[i].type |= type;
					break;
				}
			}
			if(i == nflags) {
				strcpy(flaglist[i].str, flag);
				flaglist[i].type = type;
				nflags++;
			}
		    }

	        }
	    } while((varname = getnext_ext(&ext2, varname2, buf2)));
	}
}


char* get_flags(const char *name, char *flagbuf)
{
int i;
	nflags = 0;
	add_flags("%s_Wanted", name, WANTED);
	add_flags("%s_Needed", name, NEEDED);
	add_flags("%s_Banned", name, BANNED);
	strcpy(flagbuf, "");
	strcpy(conflictbuf, "");
	for(i=0; i<nflags; i++) 
		switch(flag_action[flaglist[i].type]) {
		case USED:
			strcat(flagbuf, " ");
			strcat_subst(flagbuf, flaglist[i].str);
			break;
		case CONFLICT:
			strcat(conflictbuf, " ");
			strcat(conflictbuf, flaglist[i].str);
			break;
		default:
			break;
		}
	if(conflictbuf[0])
		printf("Warning: Conflicting %s flags: %s\n",
			name, conflictbuf);
	return flagbuf;
}



char* get_var(const char *name, char *varbuf)
// search 'extensions' and first try to match
//	'name_ext1_ext2'
//	and next just 'name_ext'
//	trying all possible extensions in order
//	if that fails, then it calls get_flags instead
{
char buf[VNAME_MAX];
char buf2[VNAME_MAX];
const char *val;
const char *ext = extensions; 
const char *ext2;
const char *varname;
const char *varname2;
	while((varname2 = getnext_ext(&ext, name, buf))) {
		ext2 = extensions;
		while((varname = getnext_ext(&ext2, varname2, buf2))) {
			varbuf[0] = '\0';
			if((val=getenv(varname))) {
				strcat_subst(varbuf, val);
				return varbuf;
			}
		}
	}
	while((varname = getnext_ext(&ext, name, buf))) {
		varbuf[0] = '\0';
		if((val=getenv(varname))) {
			strcat_subst(varbuf, val);
			return varbuf;
		}
	}
	return get_flags(name, varbuf);
}


void fprint_var(FILE *f, const char *varname)
{
char *val;
	if(!(val = getenv(varname)))
		return;
	fprintf(f, "%s = ", varname);
	while(*val) {
		if(*val == '$')
			fputs("$$", f);
		else
			fputc(*val, f);
		val++;
	}
	fputc('\n', f);
}


void vconfig(const char *fname)
{
const char *allext;
const char *allvar;
const char *ext;
const char *var;
char varname[VNAME_MAX];
FILE *fcon;
char varstr[VNAME_MAX], extstr[VNAME_MAX], extstr2[VNAME_MAX];
	fcon = fopen(fname, "w");
	allvar = getenv("Variables");
	while((var = getnext(&allvar, varstr))) {
		fprint_var(fcon, var);
		allext = getenv("Extensions");
		while((ext = getnext(&allext, extstr))) {
			sprintf(varname, "%s_%s", var, ext);
			fprint_var(fcon, varname);
			const char *ext2;
			const char *allext2 = getenv("Extensions");
			while((ext2 = getnext(&allext2, extstr2))) {
				sprintf(varname, "%s_%s_%s", var, ext, ext2);
				fprint_var(fcon, varname);
			}
		}
	}
	fclose(fcon);
}



void mconfig(const char *destfname, const char *fname)
{
DirQ *dqp;
FILE *fmk;
FILE *f;
char allpath[CMD_MAX];
char cmd[CMD_MAX];
char path[PATH_MAX];
bool missing;
char buf[BUF_MAX];
int n;
	strcpy(allpath, "");
	dqp = &dirs;
	missing = true;
	fmk = fopen(destfname, "a");
	while(dqp) {
		sprintf(path, "%s%s/%s%s", 
			dqp->_path(), dqp->_name(), fname, dqp->_ext());
		if((f = fopen(path, "r"))) {
			char linkpath[PATH_MAX];
			sprintf(linkpath, "%s%s", fname, dqp->_ref());
			unlink(linkpath);
			if(symlink(path, linkpath) < 0) {
				char err[PATH_MAX*2+16];
				sprintf(err, "symlink(%s, %s)",
					path, linkpath);
				perror(err);
			}
			strcat(allpath, " ");
			fprintf(fmk, "%s: %s\n", destfname, linkpath);
			while((n = fread(buf, 1, sizeof(buf), f)) > 0)
				fwrite(buf, 1, n, fmk); 
			fclose(f);
			fprintf(fmk, "\n\n");
			strcat(allpath, " ");
			strcat(allpath, path);
			missing = false;
		}
		dqp = dqp->_next();
	}
	if(missing)
		printf("Warning: missing %s\n", fname);
	fclose(fmk);
	printf("Configured %s:\n", destfname);
	fflush(stdout);
	if(allpath[0]) {
		sprintf(cmd, "wc %s", allpath);
		system(cmd);
	}
}



void config_from_source(FILE *f, bool prescan)
{
char buf[LINE_MAX];
const char *cp;
char argbuf[LINE_MAX];
bool add;
char varname[VNAME_MAX];
const char *val;
	exclusive = false;
	while(fgets(buf, sizeof(buf), f)) {
		if(buf[0] != '#') 
			continue;
		cp = &buf[1];
		if(!getnext(&cp, argbuf) || strcmp(argbuf, "pragma")) {
			if(prescan && !strcmp(argbuf, "include")) 
				return;
			else
				continue;
		}
		if(!getnext(&cp, argbuf) || strcmp(argbuf, "Make"))
			continue;
		if(!getnext(&cp, varname))
			goto bad;
		if(!strcmp(varname, "Exclusive")) {
			exclusive = true;
			continue;
		}
		if(!getnext(&cp, argbuf))
			goto bad;
		if(!strcmp(argbuf, "+="))
			add = true;
		else if(!strcmp(argbuf, "="))
			add = false;
		else
			goto bad;
		while(*cp <= ' ' && *cp != '\n' && *cp)
			cp++;
		val = cp;
		while(*cp && *cp != '\n')
			cp++;
		*(char*)cp = '\0';
		if(add) {	
			argbuf[0] = '\0';
			cp = getenv(varname);
			if(cp) {
				strcpy(argbuf, cp);
				if(argbuf[0])
					strcat(argbuf, " ");
			}
			strcat(argbuf, val);
			val = argbuf;
		}
		setenv(varname, val, true);
	}
	return;
bad:
	printf("Bad pragma: %s\n", buf);
	exit(1);
}



bool make_rules(const char *fname, const char *basename, const char *dest_ext)
// use Rules_xx_xx and Command_xx to create .mak files
{
char destname[NAME_MAX];
char tmpname[NAME_MAX];
char makename[NAME_MAX];
char oldname[NAME_MAX];
char cmd[CMD_MAX];
char str[CMD_MAX];
FILE *fsrc;
FILE *fmak;
const char *cp;
	dest_makefile = false;
	sprintf(destname, "%s.%s", basename, dest_ext);
	sprintf(tmpname, "%s.%s.mak.tmp", fname, dest_ext);
	sprintf(makename, "%s.%s.mak", fname, dest_ext);
	sprintf(oldname, "%s.%s.mak.old", fname, dest_ext);
	target_name = destname;
	source_name = fname;
	base_name = basename;
	sprintf(cmd, "%s -D__FCONFIG__ > %s",
		get_var("Cpp", str), tmpname);
	fprintf(logfile, "Cpp %s: %s\n", fname, cmd);
	fflush(stdout);
	if(system(cmd) != 0) {
		printf("Command: %s\n", cmd);
		exit(1);
	}
	fsrc = fopen(tmpname, "r");
	config_from_source(fsrc, false);
	fclose(fsrc);
	sprintf(cmd, "%s -D__FCONFIG__ -D__MAKEDEPEND__ > %s",
		get_var("Makedepend", str), tmpname);
	fprintf(logfile, "Makedepend %s: %s\n", fname, cmd);
	fflush(stdout);
	if(system(cmd) != 0) {
		printf("Command: %s\n", cmd);
		exit(1);
	}
	fsrc = fopen(tmpname, "r");
	fmak = fopen(makename, "w");
	while(fgets(str, sizeof(str), fsrc)) {
		if(str[0] > ' ') {
			cp = strchr(str, ':');
			if(autoconfig)
				fprintf(fmak, "%s %s%s",
					destname, makename, cp);
			else
				fprintf(fmak, "%s%s", destname, cp);
		} else
			fputs(str, fmak);
	}
	dest_makefile = true;
	fprintf(fmak, "%s: %s\n\t%s\n", destname, 
		makename, get_var("Command", str));
	fprintf(fmak, "%s:\n\t%s \"%s\" \"%s\" %s %s\n\n\n",
		makename, argvp[0], argvp[1],
		argvp[2], argvp[3], fname);
	fclose(fmak);
	fclose(fsrc);
	unlink(tmpname);
	sprintf(cmd, "cmp -s %s %s", makename, oldname);
	fflush(stdout);
	dest_makefile = false;
	switch(system(cmd)) {
	case 1:
	case 256:
		return true;
	default:
		return false;
	} 
}



void fconfig(const char *fname)
{
DirQ *dqp;
FILE *finc;
FILE *fsrc;
char allpath[CMD_MAX];
char tmpname[NAME_MAX];
char path[PATH_MAX];
char namestr[NAME_MAX];
char basename[NAME_MAX];
char ext[NAME_MAX];
char cmd[CMD_MAX];
bool missing;
char rule[CMD_MAX];
const char *dest_ext;
const char *cp;
char str[CMD_MAX];
const char *allext;
bool has_rules = false;
char backspace[PATH_MAX*3];
	dest_makefile = false;

	printf("%s", fname);
	fflush(stdout);
	cp = fname;
	for(char *bsp = backspace; *cp; cp++) {
		*bsp++ = '\b'; *bsp++ = ' '; *bsp++ = '\b'; *bsp = '\0';
	}

	if((cp = strrchr(fname, '.'))) {
		strncpy(basename, fname, cp-fname);
		basename[cp-fname] = '\0';
		strcpy(ext, cp+1);
	} else {
		strcpy(basename, fname);
		strcpy(ext, "");
	}

	allext = getenv("Extensions");
	while((dest_ext = getnext(&allext, str))) {
		sprintf(rule, "Rule_%s_%s", ext, dest_ext);
		if(getenv(rule)) {
			has_rules = true;
			break;
		}
	}

	extensions = ext;
	sprintf(tmpname, "%s.tmp", fname);
	strcpy(allpath, "");
	finc = fopen(tmpname, "w");
	sprintf(namestr, "%s%s", spath, fname);
	fprintf(finc, "/* %s */\n", namestr);
	convert_fname(namestr);
	fprintf(finc, "#ifndef __%s__\n#define __%s__\n", namestr, namestr);
	fprintf(finc, "#include \"config.h\"\n");
	dqp = dirs._next();
	missing = true;
	while(dqp) {
		if(ext[0]) 
			sprintf(path, "%s%s/%s%s.%s", 
				dqp->_path(), dqp->_name(),
				basename, dqp->_ext(), ext);
		else 
			sprintf(path, "%s%s/%s%s", 
				dqp->_path(), dqp->_name(),
				basename, dqp->_ext());
		if((fsrc = fopen(path, "r"))) {
			char linkpath[PATH_MAX];
			if(ext[0])
				sprintf(linkpath, "%s%s.%s",
					basename, dqp->_ref(), ext);
			else
				sprintf(linkpath, "%s%s",
					basename, dqp->_ref());
			if(has_rules)
				config_from_source(fsrc, true);
			fclose(fsrc);
			unlink(linkpath);
			if(symlink(path, linkpath) < 0) {
				char err[PATH_MAX*2+16];
				sprintf(err, "symlink(%s, %s)",
					path, linkpath);
				perror(err);
			}
			fprintf(finc, "#include \"%s\"\n", linkpath);
			strcat(allpath, " ");
			strcat(allpath, path);
			missing = false;
			if(exclusive)
				break;
		}
		dqp = dqp->_next();
	}
	fprintf(finc, "#endif /* ifndef __%s__ */\n", namestr);
	fclose(finc);
	sprintf(cmd, "cmp -s %s %s", fname, tmpname);

	if(system(cmd) != 0) {
		if(missing) {
			if((fsrc = fopen(fname, "r"))) {
				fclose(fsrc);
				printf("%sFound %s:\n", backspace, fname);
			} else {
				printf("%sWarning: missing %s\n", 
					backspace, fname);
				unlink(tmpname);
				return;
			}
		} else {
			unlink(fname);
			rename(tmpname, fname);
			printf("%sConfigured %s:\n", backspace, fname);
			fflush(stdout);
			sprintf(cmd, "wc %s", allpath);
			system(cmd);
		}
	} else {
		unlink(tmpname);
		printf("%s", backspace);
	}
	if(!has_rules)
		return;
	
	allext = getenv("Extensions");
	while((dest_ext = getnext(&allext, str))) {
		sprintf(rule, "Rule_%s_%s", ext, dest_ext);
		if((cp=getenv(rule))) {
			char makefilename[NAME_MAX];

			sprintf(makefilename, "%s.%s.mak", fname, dest_ext);	
	
			printf("%s: ", makefilename);
			fflush(stdout);

			rule[0] = '\0';
			strcat_subst(rule, cp);
			extensions = rule;
			bool did_cnf = make_rules(fname, basename, dest_ext);

			cp = makefilename;
			for(char *bsp = backspace; *cp; cp++) {
				*bsp++ = '\b'; *bsp++ = ' '; *bsp++ = '\b'; *bsp = '\0';
			}

			printf("\b \b\b \b%s", backspace);
			if(did_cnf)
				printf("Configured %s\n", makefilename);
		}
	}
}


main(int argc, const char *const *argv)
{
const char *makefile = 0;

	if(!strcmp(argv[1], "-v")) {
		printf("fconfig v" fconfig_VER_STR " - " fconfig_VER_DATE "\n");
		exit(1);
	}
	
	if(argc < 4) {
		fprintf(stderr, "usage: [-v] %s machtypes src spath [-m makefile] [file...]\n",
			argv[0]);
		exit(1);
	}
	const char *ap = getenv("Autoconfig");
	if(ap) 
		autoconfig = ap[strlen(ap)-1] != '0'
		 && !(strlen(ap) >= 5 && !strcmp(&ap[strlen(ap)-5], "false"));
	else
		autoconfig = false;
	argvp = argv;
	machtypes = parse_list(argv[1], &nmach);
	src = parse_list(argv[2], &nsrc);
	spath = argv[3];
	find_dirs();
	logfile = fopen("config.log", "a");
	for(int i=4; i<argc; i++) {
		if(!strcmp(argv[i], "-m"))
			makefile = argv[++i];
		else if(!strcmp(argv[i], "config.var"))
			vconfig(argv[i]);
		else if(makefile)
			mconfig(makefile, argv[i]);
		else {
			int status;

			if(fork() == 0) {
				fconfig(argv[i]);
				exit(0);
			} else {
				wait(&status);
				if(!WIFEXITED(status) 
				   || WEXITSTATUS(status))
					exit(WEXITSTATUS(status));
			} 
		}
	}
	fclose(logfile);
}

