/*  $Id: bootparam.h,v 1.3 1996/11/08 14:43:34 shaggy Exp $
 *      parameters passed from boot program to kernel 
 *      
 *      relative path: ./lib/boot/m-pc
 *  
 *      Shag/OS: an Object-Oriented MicroKernel Operating System
 *       Libraries
 *        (boot)
 *         (m-pc)
 *  
 *  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution.
 *  
 *  Shag/OS is free software for non-profit uses; it may be freely distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'COPYING'.  If this license was not received,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For bug reports, please read the file 'BUGS' first, then include the 
 *      product name and version, and the word 'bug' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

/*  $Log: bootparam.h,v $
 *  Revision 1.3  1996/11/08  14:43:34  shaggy
 *  updated to version 0.96
 *
 *  Revision 1.2  1996/11/08  13:13:25  shaggy
 *  changed comments, typedefs, etc, to work with C
 *
 *  Revision 1.1  1996/11/08  12:44:28  shaggy
 *  Initial revision
 *
 *  Created in December, 1994
 */

#include <types.h>


#define BootParam_ver_major	0
#define BootParam_ver_minor	96

#define MAX_PSEG	4	/* maximum number of program segments */



/* types of panics: (ix86 PC specific) */

#define PANIC_TYPE_IRQ		1	/* Interrupt Request 	*/
#define PANIC_TYPE_EXC		2	/* Exception		*/
#define PANIC_TYPE_RESET	3	/* trapped CPU reset	*/
#define PANIC_TYPE_OTHER	4	/* anything else	*/

/* XXX - these are temporary, due to a genofs bug that causes them to not
 *       get generated automatically:
 */
#define PSeg_paddr	0
#define PSeg_psize	4
#define PSeg_vaddr	8
#define PSeg_vsize	12
#define PSeg_sizeof	16


#ifndef ASM

struct RegBlock;
struct TssBlock;
struct FileInfo;
struct CpuInfo;
struct MapFile;

#ifndef __cplusplus
	typedef struct RegBlock RegBlock;
	typedef struct TssBlock TssBlock;
	typedef struct FileInfo FileInfo;
	typedef struct CpuInfo CpuInfo;
	typedef struct MapFile MapFile;
	typedef struct PSeg PSeg;
	typedef struct BootParam BootParam;
#    ifndef bool
#	define bool	char
#    endif
#    ifndef true
#	define true	1
#    endif
#    ifndef false
#	define false	0	
#    endif
#endif

/* important note:
 *	If any changes are made to BootParam that 
 *	alter the offsets of existing attributes,
 *	then backwards compatability with older
 *	kernels or older boot loaders is lost.
 *	(for existing compiled binaries only)
 *	Therefore, unless absolutely necessary, all changes
 *	should be made in a manner that does not rearrange any
 *	attributes or change their sizes.
 *
 *	Especially don't change anything up to, and including the boot loader
 *	version and name, because it may be critical to obtain this
 *	information, to determinate the compatability of the interface.
 *
 *	Reserved spaces allow for future expansion without
 *	changing the layout extensively.
 */


struct PSeg {		// program segment
	uint32	paddr;		// physical address (or file offset)
	uint32	psize;		// physical size (or file seg size)
	uint32	vaddr;		// virtual address
	uint32	vsize;		// virtual size of segment
};


/* If we're using C, we can't scope these inside the structure... */
#ifndef __cplusplus
	typedef uint16 index_t;
	typedef uint16 fd_t;
#endif


/* BootParam interface format v0.96: */
struct BootParam {
#ifdef __cplusplus
	typedef	uint16 index_t;	/* file index type */
	typedef uint16 fd_t;	/* file descriptor type */
#endif

	/* Generic, portable interface: */

	uint8	bootver_major;	/* major number of boot loader version */
	uint8	bootver_minor;	/* minor number of boot loader version */
	uint16	reserved2;
	char	*bootname;	/* name of boot loader (without the version) */
	int32	argc;		/* argc for main() */
	char	**argv;		/* *argv[] for main() */
	char	**envp;		/* *envp[] for main() (currently unused) */
	uint32	reserved1;
	uint32	reserved3;
	uint32	entry;		/* starting execution address */
	uint32	lomem;		/* lowest virtual address available */
	uint32	himem;		/* highest virtual address available + 1 */
	PSeg	text;		/* text segment */
	PSeg	data;		/* data segment */
	PSeg	spare;		/* spare segment */
	PSeg	bss;		/* bss segment */
	PSeg	reserved;	/* future extra spare segment */
	uint32	reserved15;
	uint32	reserved16;

	uint32	msgbuf;		/* kernel message buffer */
	uint16	msgsize;	/* size of message buffer */
	uint16	msgptr;		/* current pointer into message */

	fd_t	console_fd;	/* file descriptor for console */
	fd_t	self_fd;	/* file descriptor for current executable */
	uint32	reserved21;
	void	(*exit)(int32 code);		/* exit from kernel */
	void	(*reboot)(bool cold);		/* warm/cold reboot */
	fd_t	(*open)				/* open file and */
		(const char *name);		/* return 0 or fd */
	void	(*close)			/* close a file */
		(fd_t);
	const MapFile* (*readdir)		/* open directory entry and */
		(index_t index);		/* return 0 or map address */
	int32	(*read)				/* read from a file/device */
		(fd_t, void *adr, uint32 len);
	int32	(*write)			/* write to a file/device */
		(fd_t, const void *adr, uint32 len);
	void	(*fstat)
		(fd_t, FileInfo *stbuf);
	void	(*seek)				/* set file position */
#ifdef __cplusplus
		(fd_t, uint32 ofs=0);
#else
		(fd_t, uint32);
#endif
	uint32	(*tell)				/* get file position */
		(fd_t);
	void	(*reserved25[2])();


	/* ix86 PC and PC BIOS Interface: */

	CpuInfo* cpuinfo;	/* CPU information */
	uint32	buffer;		/* buffer for calling BIOS */
	uint16	bufsize;	/* size of buffer */
	uint8	bootdev;	/* BIOS boot device */
	uint8	reserved4;
	uint32	base_addr;	/* minimum base address */
	uint32	base_size;	/* size of base memory */
	uint32	base_lomem;	/* lowest available base address */
	uint32	base_himem;	/* highest available base address + 1 */
	uint32	ext_addr;	/* minimum extended memory address */
	uint32	ext_size;	/* size of available extended memory */

	uint16	timer0_count;	/* timer0 value */
	uint8	timer0_mode;	/* timer0 mode */
	uint8	reserved20;
	uint32	reserved13;
	uint32	reserved14;

	void	(*panic)	/* panic and exit */
		(uint8 type, uint32 num, uint32 code,
		 TssBlock *tss);
		/* Note: panic() should exist on all platforms, but the
		 * parameters to it will vary depending on the nature of
		 * the information to be reported
		 */

	void	(*callbios)			/* call a BIOS interrupt */
		(uint8 vec, RegBlock *reg);
	void	(*reserved26[2])();
};


/*

Kernel booting notes:

	After an executable is loaded from SOLO,
	virtual memory is initialized so that the executable
	is already mapped to where it wants to be loaded,
	and such that rest of that segment (up to 4MB) is
	also mapped in, giving some temporarily useable memory
	from 'lomem' to 'himem'.   This needs to be taken into
	account by any virtual memory management system that
	takes over afterwards.
	Also, physical memory from 00000000-000FFFFF is mapped
	into the same corresponding virtual addresses, allowing
	direct access to physical low memory with the same address.
	Therefore, the executable should *not* try to map
	itself into the low 1Mb of memory, or it will conflict
	with the mappings for physical memory.
	So, if addresses in this range are to be used, the old
	pagetable for low memory should be preserved, and must
	be used every time a call to any of the BootParam
	functions is made.  Or, a new mapping of the low 1Mb of memory
	must be set up.  Also, the first 32K of memory must not
	be altered, except in the regions from 
	(buffer to buffer+bufsize-1) and (msgbuf to msgbuf+msgsize-1)
	
*/



#else

#include <boot/bootparam.ofs>

#endif

