/*  $Id: cpuinfo.h,v 1.2 1996/11/08 14:42:13 shaggy Exp $
 *      relative path: ./lib/boot/m-pc
 *  
 *      Shag/OS: an Object-Oriented MicroKernel Operating System
 *       Libraries
 *        (boot)
 *         (m-pc)
 *  
 *  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution.
 *  
 *  Shag/OS is free software for non-profit uses; it may be freely distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'COPYING'.  If this license was not received,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For bug reports, please read the file 'BUGS' first, then include the 
 *      product name and version, and the word 'bug' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

/*  $Log: cpuinfo.h,v $
 *  Revision 1.2  1996/11/08  14:42:13  shaggy
 *  updated to include magic number
 *
 *  Revision 1.1  1996/11/08  14:19:38  shaggy
 *  Initial revision
 *
 *  creation date: January 1995
 */

#include <types.h>


/* CPU family: */
#define CPU_8086	0
#define CPU_80186	1
#define CPU_80286	2
#define CPU_80386	3
#define CPU_80486	4
#define CPU_80586	5
#define CPU_PENTIUM	5
#define CPU_80686	6
#define CPU_PENTIUM_PRO	6
#define CPU_80786	7
#define CPU_SEPTIUM	7
#define	CPU_UNKNOWN	0xff

#define CPU_MODEL_UNKNOWN	0xff
#define CPU_REVISION_UNKNOWN	0xff
#define CPU_TYPE_UNKNOWN	0xff

/* CPU flags: */
#define CPU_HAS_FPU		0x0001
#define CPU_HAS_MCHECK		0x0080
#define CPU_HAS_CMPXCHG8B	0x0100


#ifndef ASM

struct CpuInfo {
	uint32	magic;	/* "In86" */
	uint8	family;
	uint8	model;
	uint8	revision;
	uint8	type;
	uint32	flags;
	char	vendor[12];
	uint8	null;	/* to terminate 'vendor' */
	uint8	resv1;
	uint8	resv2;
	uint8	resv3;
	const char *name;
};

#endif


/* Notes:
 *	The "magic" number is used to identify the type of CPU structure
 *	being used, and thus also the general type of the CPU.
 *	This is 4 byte value, composed of printable ascii characters.
 *	In86 is being used for Intel 80x86 processors, and the following
 *	names could potentially be used for other processors:
 *		In86		Intel 80x86 series
 *		InI8		Intel i860 
 *		RS6K		R6000 series
 *		Mo68		Motorola 680x0 series
 *		Mo68		Motorola 88000 series
 *		NS32		National Semiconductor 32000 series
 *		PWPC		Power PC
 *		Alph		DEC Alpha
 *		MIPS		MIPS processor
 *		Clip		Clipper processor
 *		SPRC		SPARC series
 *		6502		:)
 */

