/*  fileinfo.h                                                  (CRC:a66957c4)
 *      File Information structure
 *      
 *      relative path: ./lib/boot/m-pc
 *      creation date: December 1994
 *      last modified: Sat Oct 26 15:09:56 1996
 *      header update: Sat Oct 26 15:09:57 1996
 *  
 *      Shag/OS: an Object-Oriented MicroKernel Operating System
 *       Libraries
 *        (boot)
 *         (m-pc)
 *  
 *  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution.
 *  
 *  Shag/OS is free software for non-profit uses; it may be freely distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'COPYING'.  If this license was not received,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For bug reports, please read the file 'BUGS' first, then include the 
 *      product name and version, and the word 'bug' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

#include <types.h>


#ifndef ASM

struct FileInfo {
	uint64	atime;		// last access time
	uint64	ctime;		// creation time (or last change of state)
	uint64	mtime;		// last modification time
	uint32	size;		// size of file in bytes
	uint16	blocks;		// size of file in blocks
	uint16	blksize;	// size of one file block
	uint32	rdev;		// raw device ID
	// These should probably be part of a different structure,
	// with specific calls for disk information...
	// but for now, they're here for simplicity
	// (don't expect them to stay here, though)
	uint16	cylinders;	// total cylinders on raw device
	uint8	heads;		// total heads on raw device
	uint8	sectors;	// total sectors on raw device
};


#define RDEV_MAJOR(d)	((d)>>16)&0xff)
#define RDEV_MINOR(d)	((d)&0xffff)
#define RDEV_UNIT(d)	((d)&0xff)
#define RDEV_SUBINIT(d)	((d>>8)&0xff)
#define RDEV_FLAGS(d)	RDEV_SUBUNIT(d)


#else
#include <boot/fileinfo.ofs>
#endif

