//  $Id$
//      
//      I/O library functions for interfacing with SOLO
//      
//      relative path: ./lib/boot/m-pc
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       Libraries
//        (boot)
//         (m-pc)
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

#include <defines>
#include <boot/bootparam.h>
#include <boot/fileinfo.h>
#include <posix/string.h>
#include <posix/stdarg.h>
#include <posix/stdio.h>

#define SOLO_NOSTDIO
#include "libsolo.h"


#define MAXBUF 512	// max buffer size for I/O


extern BootParam *bootparam;
solo_fd_t stdin_fileno;
solo_fd_t stdout_fileno;
solo_fd_t stderr_fileno;


int solo_puts(const char *s)
{
	return bootparam->write(stdout_fileno, s, strlen(s));
}


char* solo_gets(char *s, size_t /*len*/)
{
char *sp = s;		
	while(bootparam->read(stdin_fileno, sp, 1) == 1) {
		if(*sp == '\n') {
			*sp = 0;	
			return s;
		}
		sp++;
	}
	*sp = 0;
	if(sp == s)
		return 0;
	else	
		return s;
}


void solo_exit(int val)
{
        bootparam->exit(val);
	while(true);
}


int solo_vprintf(const char *fmt, va_list args) {
char buf[MAXBUF];
int len;
	len = vsprintf(buf, fmt, args);
	solo_puts(buf);
	return len;
}


int solo_printf(const char *fmt, ...) {
char buf[MAXBUF];
int len;
va_list args;
        va_start(args, fmt);
        len = vsprintf(buf, fmt, args);
        va_end(args);
        solo_puts(buf);
        return len;
}


solo_fd_t  solo_open(const char *path, int /*flags*/) {
solo_fd_t fd;
	if(!(fd = bootparam->open(path)))
		return (solo_fd_t)-1;
	else
		return fd;
}


int solo_close(solo_fd_t fd) {
	bootparam->close(fd);
	return 0;
}


int solo_read(solo_fd_t fd, void *buf, size_t size)
{
	return bootparam->read(fd, buf, size);
}


int solo_write(solo_fd_t fd, const void *buf, size_t size)
{
	return bootparam->write(fd, buf, size);
}


off_t solo_lseek(solo_fd_t fd, off_t ofs, int whence)
{
FileInfo info;
	switch(whence) {
	case SEEK_CUR:
		ofs += bootparam->tell(fd);
		break;
	case SEEK_END:
		bootparam->fstat((BootParam::fd_t)fd, &info);
		ofs += info.size;
		break;
	case SEEK_SET:
	default:
		break;
	}
	bootparam->seek(fd, ofs);
	return bootparam->tell(fd);
}


void solo_fstat(solo_fd_t fd, FileInfo *info)
{
	bootparam->fstat(fd, info);
}



extern "C" bool _bootparam_init();

extern "C" bool _libsolo_init()
{
	static bool initialized = false;
	static bool initializing = false;

	if(initialized)
		return true;
	if(initializing)
		return false;
	initializing = true;
	_bootparam_init();
	stdin_fileno = bootparam->console_fd;
	stdout_fileno = bootparam->console_fd;
	stderr_fileno = bootparam->console_fd;

	initialized = true;
	initializing = false;
	return true;
}

// Global constructor:
bool _libsolo_initialized = _libsolo_init();

