//  Queue.H                                                     (CRC:9046430e)
//      relative path: ./lib/c++
//      creation date: Fri May  3 18:05:54 1996
//      last modified: Sat Oct 26 15:08:15 1996
//      header update: Sat Oct 26 15:08:16 1996
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       Libraries
//        C++ specific code
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

	
class QueueNode {
	friend class Queue;
	QueueNode *prev;
	QueueNode *next;
public:
	void clear() { prev = next = this; }
	bool empty() const { return next == this; }
	QueueNode& insert(QueueNode *node) {
			node->prev = prev; node->next = this;
			prev->next = node; prev = node; return *node; }
	QueueNode& append(QueueNode *node) {
			node->next = next; node->prev = this;
			next->prev = node; next = node; return *node; }
	QueueNode& get_next() const { return *next; }
	QueueNode& get_prev() const { return *prev; }
	QueueNode& remove() {
			next->prev = prev;
			prev->next = next; return *this; }
	QueueNode& rotate_next() {
			QueueNode *node = next;
			next = node->next;
			node->next->prev = this;
			prev->next = node;
			node->prev = prev;
			prev = node;
			node->next = this; return *node; }
	QueueNode& rotate_prev() {
			QueueNode *node = prev;
			prev = node->prev;
			node->prev->next = this;
			next->prev = node;
			node->next = next;
			next = node;
			node->prev = this; return *node; }
};


template<class T> class QueueIterator {
	struct Node : QueueNode {
		T item;
		Node(const T &newitem) { item = newitem; }
	} *current;
public:
	QueueIterator(const QueueNode *c) { current = (Node*)c; }
	T& operator* () { return current->item; }
	operator QueueNode() { return *current; }
	QueueIterator& operator++ () { current = (Node*)&current->get_next(); return *this; }
	QueueIterator& operator-- () { current = (Node*)&current->get_prev(); return *this; }
	QueueIterator operator++ (int) { Node *old = current;
		current = (Node*)&current->get_next(); return old; }
	QueueIterator operator-- (int) { Node *old = current;
		current = (Node*)&current->get_prev(); return old; }
	QueueIterator insert(const T &item) { return (Node*)&current->insert(new Node(item)); }
	QueueIterator append(const T &item) { return (Node*)&current->append(new Node(item)); }
	QueueIterator& remove() { Node *old = current; operator++();
			delete (Node*)(&old->remove()); return *this; }
	bool operator== (const QueueIterator &q) const
		{ return current == q.current; }
	bool operator!= (const QueueIterator &q) const
		{ return current != q.current; }
};



template<class T> class Queue {
	friend class QueueIterator<T>;
	QueueNode queue;
public:
	Queue() { queue.clear(); }
	bool empty() const { return queue.empty(); }
	QueueIterator<T> head() const { return QueueIterator<T>(&queue.get_next()); }
	QueueIterator<T> tail() const { return QueueIterator<T>(&queue.get_prev()); }
	QueueIterator<T> begin() const { return QueueIterator<T>(&queue.get_next()); }
	QueueIterator<T> end() const { return QueueIterator<T>(&queue); }
	QueueIterator<T> rotate_next() { return QueueIterator<T>(&queue.rotate_next()); }
	QueueIterator<T> rotate_prev() { return QueueIterator<T>(&queue.rotate_prev()); }
	~Queue() { while(!queue.empty()) head().remove(); }
};

