//  vstream.C                                                   (CRC:d3931b2e)
//      relative path: ./lib/c++
//      creation date: Tue May 21 20:08:16 1996
//      last modified: Sat Oct 26 15:08:28 1996
//      header update: Sat Oct 26 15:08:29 1996
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       Libraries
//        C++ specific code
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

#include "vstream.H"

ssize_t
vstream::read(VecAny)
{ return -1; }

ssize_t
vstream::write(CVecAny)
{ return -1; }

ofs_t
vstream::seek(ofs_t, int) 
{ return -1; }

ofs_t
vstream::tell() const
{ return -1; }

vstream*
vstream::dup() const
{ return 0; }


ssize_t
vstream::read_at(ofs_t p, VecAny buf)
{
	ofs_t cur = tell();
	seek(p);
	ssize_t n = read(buf);
	seek(cur);
	return n;
}

ssize_t
vstream::write_at(ofs_t p, CVecAny buf)
{
	ofs_t cur = tell();
	seek(p);
	ssize_t n = write(buf);
	seek(cur);
	return n;
}

ssize_t
vstream::read_through(Str buf, char ch)
{
	for(size_t n = 0; n < buf.count(); n++) {
		if(!(read(Str(buf[n])))) return n;
		if(buf[n] == ch) return n+1;
	}
	return buf.count();
}

ssize_t
vstream::read_through(WStr buf, wchar_t ch)
{
	for(size_t n = 0; n < buf.count(); n++) {
		if(!(read(WStr(buf[n])))) return n;
		if(buf[n] == ch) return n+1;
	}
	return buf.count();
}


// returns number of bytes flushed/purged
ssize_t
vstream::flush(int)
{
	return 0;
}

