//  vstream.H                                                   (CRC:e599d905)
//      relative path: ./lib/c++
//      creation date: Sat Apr 20 20:55:49 1996
//      last modified: Sat Oct 26 15:08:09 1996
//      header update: Sat Oct 26 15:08:09 1996
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       Libraries
//        C++ specific code
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

#include <types.h>
#include <Vec>



#ifndef SEEK_SET
#define SEEK_SET	0
#define SEEK_CUR	1
#define SEEK_END	2
#endif


class vstream {
public:
	enum io_mode {
		READ = 0x0001,
		WRITE = 0x0002,
		APPEND = 0x0004,
		CREATE = 0x0008,
		TRUNC = 0x0010,
		NOBLOCK = 0x0020,
		SYNC = 0x0040
	};	
	virtual ~vstream() {}
	virtual ssize_t read(VecAny buf);
	virtual ssize_t read_through(Str buf, char ch);
	virtual ssize_t read_through(WStr buf, wchar_t ch);
	virtual ssize_t write(CVecAny buf);
	virtual ofs_t seek(ofs_t p, int whence = SEEK_SET);
	virtual ofs_t tell() const;
	virtual ssize_t read_at(ofs_t p, VecAny buf);
	virtual ssize_t write_at(ofs_t p, CVecAny buf);
	virtual ssize_t flush(int flags = WRITE);
	virtual vstream* dup() const;
	ssize_t purge() { return flush(READ|WRITE|TRUNC|NOBLOCK); }
	ssize_t purge_input() { return flush(READ|TRUNC|NOBLOCK); }
	ssize_t purge_output() { return flush(WRITE|TRUNC|NOBLOCK); }
	ssize_t read(void *adr, size_t size) 
		{ return read(Vec_void(adr, size)); }
	ssize_t write(const void *adr, size_t size) 
		{ return write(CVec_void(adr, size)); }
	ssize_t read_at(ofs_t p, void *adr, size_t size) 
		{ return read_at(p, Vec_void(adr, size)); }
	ssize_t write_at(ofs_t p, const void *adr, size_t size) 
		{ return write_at(p, CVec_void(adr, size)); }
	void rewind() { seek(0, SEEK_SET); }
};


