//  Debug.C                                                     (CRC:8aa8624a)
//      relative path: ./lib/elf
//      creation date: Sat Jan 20 21:50:47 1996
//      last modified: Sat Oct 26 15:08:56 1996
//      header update: Sat Oct 26 15:08:57 1996
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       Libraries
//        Elf-format binary control
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

#include "Debug.h"

vaddr_t Debug::pc_traceback(int level)
{
vaddr_t pc;
ulong *bp,*sp;
	pc = 0;
	__asm__("movl %%ebp,%0":"g="(bp));
	while(level-- >= 0) {
		if(!bp)
			break;
		sp = bp;		/* simulate 'leave' */
		bp = (ulong*)(*sp++);	/* simulate 'leave' */
		pc = (vaddr_t)(*sp++);	/* simulate 'ret' */
		if(!pc)
			break;
	}
	return(pc);
}


void Debug::stack_traceback(/* Context *context, boolean sym */ )
{
ulong *bp;
ulong *sp;
vaddr_t pc,newpc;
char buf[80];
int nparam;
	if(context) {
		pc = context->pc;
		sp = (ulong*)(context->reg[REG_ESP]);
		bp = (ulong*)(context->reg[REG_EBP]);
	} else {
		__asm__("movl %%esp,%0":"g="(sp));
		__asm__("movl %%ebp,%0":"g="(bp));
		pc = 0;
	}
	while(1) {
		if(pc) {
/*
			if(sym && pc > KERNBASE && kinfo.Root) {
				kprintf("pc=%8x  ", pc); 
				kernsym_getbyvalue(pc, buf, sizeof(buf));
				kprintf("%s ", buf);
			} else
*/
				kprintf("pc=%8x sp=%8x bp=%8x  ", pc, sp, bp); 
		}
		if(!bp) 
			break;
		sp = bp; 		 /* simulate 'leave' */
		bp = (ulong*)(*sp++);	 /* simulate 'leave' */
		newpc = (vaddr_t)(*sp++); /* simulate 'ret' */
		if(pc && newpc) {
			nparam = bp-sp;
			if(!bp)
				nparam = 1;
			/* check for 'addl $xx,%esp' */
			if(*(ushort*)(newpc) == 0xc483) 
				nparam = (*(uchar*)(newpc+2))/4;
			else if (*(uchar*)(newpc) != 0xc9)
				kprintf("?"); 
			kprintf("(");
			while(nparam--) {
				kprintf("%x",*sp++);	
				if(nparam)
					kprintf(",");
			}
			kprintf(")\n");	
		}
		if(!newpc || newpc == pc)
			break;
		pc = newpc;
	} 
	kprintf("\n");
}

