//  Elf32.C                                                     (CRC:4e0bbdc6)
//      relative path: ./lib/elf
//      creation date: Sun Jan 28 17:15:26 1996
//      last modified: Sat Oct 26 15:08:49 1996
//      header update: Sat Oct 26 15:08:50 1996
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       Libraries
//        Elf-format binary control
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

#pragma Make Includes_Banned += -nostdinc -nostdinc++ 
#pragma Make Flags_Banned += -static 

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <vstream>
#include <vdir>
#include <vthread>
#include "Elf32.H"
#include "Error.h"




const char* Elf32::ei_class_str(int n)
{
	switch(n) {
	case ELFCLASS32: return "32-bit";
	case ELFCLASS64: return "64-bit";
	default: return "???";
	}
}

const char* Elf32::ei_data_str(int n)
{
	switch(n) {
	case ELFDATA2LSB: return "LSB";
	case ELFDATA2MSB: return "MSB";
	default: return "???";
	}
}


const char* Elf32::e_type_str(int n)
{
	switch(n) {
	case ET_NONE: return "NONE";
	case ET_REL: return "REL";
	case ET_EXEC: return "EXEC";
	case ET_DYN: return "DYN";
	case ET_CORE: return "CORE";
	default: return "???";
	}
}


const char* Elf32::e_machine_str(int n)
{
	switch(n) {
	case EM_NONE: return "NONE";
	case EM_M32: return "M32";
	case EM_SPARC: return "SPARC";
	case EM_386: return "386";
	case EM_68K: return "68K";
	case EM_88K: return "88K";
	case EM_486: return "486";
	case EM_860: return "860";
	case EM_MIPS: return "MIPS";
	default: return "???";
	}
}


const char* Elf32::sh_type_str(int n)
{
	switch(n) {
	case SHT_NULL: return "NULL";
	case SHT_PROGBITS: return "PROGBITS";
	case SHT_SYMTAB: return "SYMTAB";
	case SHT_STRTAB: return "STRTAB";
	case SHT_RELA: return "RELA";
	case SHT_HASH: return "HASH";
	case SHT_DYNAMIC: return "DYNAMIC";
	case SHT_NOTE: return "NOTE";
	case SHT_NOBITS: return "NOBITS";
	case SHT_REL: return "REL";
	case SHT_SHLIB: return "SHLIB";
	case SHT_DYNSYM: return "DYNSYM";
	case SHT_NUM: return "NUM";
	default: return "???";
	}
}


// XXX/DEP -- I386 (feb)
const char* Elf32::r_type_str(int n)
{
	switch(n) {
	case R_386_NONE: return "NONE";
	case R_386_32: return "32 (S+A)";
	case R_386_PC32: return "PC32 (S+A-P)";
	case R_386_GOT32: return "GOT32 (G+A-P)";
	case R_386_PLT32: return "PLT32 (L+A-P)";
	case R_386_COPY: return "COPY"; 
	case R_386_GLOB_DAT: return "GLOB_DAT (S)";
	case R_386_JMP_SLOT: return "JMP_SLOT (S)";
	case R_386_RELATIVE: return "RELATIVE (B+A)";
	case R_386_GOTOFF: return "GOTOFF (S+A-GOT)";
	case R_386_GOTPC: return "GOTPC (GOT+A-P)";
	default: return "???";
	}
}

const char* Elf32::p_type_str(int n)
{
	switch(n) {
	case PT_NULL: return "NULL";
	case PT_LOAD: return "LOAD";
	case PT_DYNAMIC: return "DYNAMIC";
	case PT_INTERP: return "INTERP";
	case PT_NOTE: return "NOTE";
	case PT_SHLIB: return "SHLIB";
	case PT_PHDR: return "PHDR";
	default: return "???";
	}
}


const char* Elf32::d_tag_str(int n)
{
	switch(n) {
	case DT_NULL: return "NULL";
	case DT_NEEDED: return "NEEDED";
	case DT_PLTRELSZ: return "PLTRELSZ"; 
	case DT_PLTGOT: return "PLTGOT";
	case DT_HASH: return "HASH";
	case DT_STRTAB: return "STRTAB";
	case DT_SYMTAB: return "SYMTAB";
	case DT_RELA: return "RELA";
	case DT_RELASZ: return "RELASZ";
	case DT_RELAENT: return "RELAENT";
	case DT_STRSZ: return "STRSZ";
	case DT_SYMENT: return "SYMENT";
	case DT_INIT: return "INIT";
	case DT_FINI: return "FINI";
	case DT_SONAME: return "SONAME";
	case DT_RPATH: return "RPATH";
	case DT_SYMBOLIC: return "SYMBOLIC";
	case DT_REL: return "REL";
	case DT_RELSZ: return "RELSZ";
	case DT_RELENT: return "RELENT";
	case DT_PLTREL: return "PLTREL";
	case DT_DEBUG: return "DEBUG";
	case DT_TEXTREL: return "TEXTREL";
	case DT_JMPREL: return "JMPREL";
	default: return "???";
	}
}


void Elf32::read(Phdr &phdr, int i) const
{
	stream->read_at(ehdr.e_phoff+i*ehdr.e_phentsize, ADR_LEN(phdr));
}

void Elf32::read(Shdr &shdr, int i) const
{
	stream->read_at(ehdr.e_shoff+i*ehdr.e_shentsize, ADR_LEN(shdr));
}


Elf32::Elf32(vstream *newstream) 
{
	stream = newstream;
	stream->read_at(0, ADR_LEN(ehdr));
	if(    ehdr.e_ident[EI_MAG0] != MAG0
	    || ehdr.e_ident[EI_MAG1] != MAG1
	    || ehdr.e_ident[EI_MAG2] != MAG2
	    || ehdr.e_ident[EI_MAG3] != MAG3)
		THROW(Error("Not an ELF file"));
	link = 0;
	shstrtab = 0;
}

Elf32::~Elf32()
{
	delete stream;
}


ulong Elf32::hash(const uchar *name) 
{
	ulong h = 0, g;
	while(*name) {
		h = (h << 4) + *name++;
		if ((g = h & 0xf0000000))
			h ^= g >> 24;
		h &= ~g;
	}
	return h;
}



const char* Elf32::shstr(Word index) const
{
	if(!shstrtab && ehdr.e_shstrndx) {
		Shdr shdr;
		read(shdr, ehdr.e_shstrndx);
		const_cast<char*&>(shstrtab)
			= new char[shdr.sh_size];
		stream->read_at(shdr.sh_offset, shstrtab, shdr.sh_size);
		return &shstrtab[index];
	} 
	if(shstrtab)
		return &shstrtab[index];
	else
		return "?";
}


const char* Elf32::section(int index) const
{
Shdr shdr;
	switch(index) {
	case SHN_UNDEF:
		return "*UND*"; 
	case SHN_ABS:
		return "*ABS*";
	case SHN_COMMON:
		return "*COM*";
	default:
		if(index >= SHN_LORESERVE)
			return "*RSV*";
		read(shdr, index);
		return shstr(shdr.sh_name);
	}
}



void Elf32::dump(const Ehdr &e)
{
	printf("ELF %s %s v%d  p:%lx,%x  s:%lx,%x f:%lx\n",
		ei_class_str(e.e_ident[EI_CLASS]),
		ei_data_str(e.e_ident[EI_DATA]),
		e.e_ident[EI_VERSION],
		e.e_phoff, e.e_phentsize,
		e.e_shoff, e.e_shentsize, e.e_flags);
	printf("ET:%s EM:%s v:%ld entry:%08lx p:%d s:%d/%d\n",
	        e_type_str(e.e_type), e_machine_str(e.e_machine),
		e.e_version, e.e_entry, e.e_phnum, e.e_shnum,
		e.e_shstrndx);
}

void Elf32::dump(const Phdr &p)
{
	printf("PT:%s v:%08lx p:%08lx s:%lx o:%lx s:%lx f:%lx a:%ld\n",
		p_type_str(p.p_type), p.p_vaddr, p.p_paddr,
		p.p_memsz, p.p_offset, p.p_filesz,
		p.p_flags, p.p_align);
}

void Elf32::dump(const Shdr &s) const
{
	printf("[%s] SHT:%s %c%c%c a:%08lx o:%lx s:%lx l:%ld a:%ld e:%ld\n",
		shstr(s.sh_name), sh_type_str(s.sh_type),
		(s.sh_flags&SHF_ALLOC)?'a':'-',
		(s.sh_flags&SHF_WRITE)?'w':'-',
		(s.sh_flags&SHF_EXECINSTR)?'x':'-',
		s.sh_addr, s.sh_offset, s.sh_size,
		s.sh_link, s.sh_addralign, s.sh_entsize);
}


void Elf32::dump(const Sym &s, const char *strtab) const
{
	printf("%08lx/%lx %c %c %s %s\n",
		s.st_value, s.st_size,
		"LGW             "[s.st_type()],
		"?OCSF           "[s.st_bind()],
		section(s.st_shndx),
		s.st_name?&strtab[s.st_name]:"?");
}


void Elf32::dump(const Rel &r, const Sym *sym,
			const char *strtab) const
{
	printf("%08lx %s : ",
		r.r_offset, r_type_str(r.r_type()));
	dump(sym[r.r_sym()], strtab);
}

void Elf32::dump(const Rela &r, const Sym *sym,
			const char *strtab) const
{
	printf("%08lx %s %lx: ",
		r.r_offset, r_type_str(r.r_type()),
		r.r_addend);
	dump(sym[r.r_sym()], strtab);
}


void Elf32::dump() const
{
	dump(ehdr);
	for(int i=0; i<ehdr.e_phnum; i++) {
		Phdr phdr;
		read(phdr, i);
		printf("ph %d: ", i);
		dump(phdr);
	}
	for(int i=0; i<ehdr.e_shnum; i++) {
		Shdr shdr;
		read(shdr, i);
		printf("sh %d: ", i);
		dump(shdr);
	}
}


Elf32::Word Elf32::link_info::symhash(const char *name, Word x)
{
Word nbucket = hash[0];
Word *bucket = &hash[2];
Word *chain = &hash[2+nbucket];
Word y = bucket[x%nbucket];
	while(y != STN_UNDEF) {
		if(!strcmp(&strtab[symtab[y].st_name], name))
			return y;
		y = chain[y];
	}
	return y;
}


void Elf32::dyn_chain::dump(int indent)
{
	for(int i=0; i<indent*4; i++)
		printf(" ");
	printf("%s (link=%p,load=%p,dyn=%p) %d\n", link->libname,
			link, (void*)(link->loadaddr), link->dynamic_addr,
			link->initialized);
	if(next_level)
		next_level->dump(indent+1);
	if(next)
		next->dump(indent);
}


Elf32::Sym* Elf32::dyn_chain::lookup_level(int level,
			  const char *name, Word hnum, link_info **linkp)
{
Sym* sym = 0;
dyn_chain *chain = this;
bool hit_bottom = true;
	printf("level=%d\n", level);
	while(chain && !sym) {
		if(level > 0) {
			if(chain->next_level) {
				sym = chain->next_level->lookup_level(
					level-1, name, hnum, linkp);
				hit_bottom = false;
			}
		} else {
			link_info *link = chain->link;
			if(!link->initialized && !link->initializing)	
				link->initialize(); 
			Word snum = link->symhash(name, hnum);
			printf("snum=%ld\n", snum);
			if(snum != STN_UNDEF) {
				// link->elf32->dump(link->symtab[snum], link->strtab);
				if(link->symtab[snum].st_shndx != SHN_UNDEF) {
					sym = &link->symtab[snum];
					*linkp = link;
				}	
			}
		}
		chain = chain->next; 
	}
	if(level && hit_bottom)
		return (Sym*)-1;
	return sym;
}


Elf32::Word Elf32::dyn_chain::lookup(const char *name)
{
// XXX -- need way of dealing with weak symbols
Word hnum = Elf32::hash(name);
Sym *sym;
int level = 0;
link_info *link;
	while(!(sym = lookup_level(level, name, hnum, &link)))
		level++;
	printf("got sym=%p\n", sym);
	if(sym == (Sym*)-1) {
		// XXX -- these should be probably be set to something useful
		if(!strcmp(name, "___brk_addr"))
			return (Word)sbrk(0);
		if(!strcmp(name, "__environ"))
			return 0;
		printf("sym not found. chain=\n");
		dump();
		THROW(Error(name));
		return 0;
	}
	if(sym->st_shndx == SHN_ABS)  
		return sym->st_value;
	else
		return sym->st_value + link->loadaddr;
}


Elf32::Word Elf32::link_info::symval(Word snum)
{
Sym &s = symtab[snum];
	printf("sym=%lx\n", snum);
	// elf32->dump(s, strtab);
	if(s.st_shndx == SHN_UNDEF) {
		// XXX -- can this be optimized to save the
		//	  value after it's looked up,
		//	  or will that be a problem when there
		//	  are weak bindings, etc... ?
		const char *name = &strtab[s.st_name];
		printf("look up %s\n", name);
		return chain_top->lookup(name);
	}
	if(s.st_shndx == SHN_ABS) 
		return s.st_value;
	else
		return s.st_value + loadaddr;
}



// XXX/DEP -- I386 (feb)
Elf32::Addr Elf32::link_info::reloc(const Rel &r, Sword addend, bool bind_jmp)
{
	Word *ofs = (Word*)(r.r_offset + addend);
	switch(r.r_type()) {
	case R_386_NONE:
		break;
	case R_386_GLOB_DAT:	
		*ofs = symval(r.r_sym());
		break;
	case R_386_JMP_SLOT:
		if(bind_jmp)
			*ofs = symval(r.r_sym());
		else
			*ofs += addend;
		break;
	case R_386_32:
		*ofs += symval(r.r_sym());
		break;
	case R_386_PC32:	
		*ofs += symval(r.r_sym()) - (Addr)ofs;
		break;
	case R_386_RELATIVE:
		*ofs += addend;
		break;
	case R_386_COPY:
	case R_386_GOTOFF:
	case R_386_GOTPC:
	case R_386_GOT32:
	case R_386_PLT32:	
	default:
		printf("Warning: %s not supported\n",
				r_type_str(r.r_type()));
		break;
	}
	printf("*(%p) = %lx\n", ofs, *ofs);
	return (Addr)ofs;
}






Elf32::Addr Elf32::link_info::resolver(Word ofs)
{
	printf("resolving (%x) ofs:%ld\n", (uint)this, ofs);
	if(pltrel == DT_REL)
		return reloc(*(Rel*)(jmprel+ofs), loadaddr);
	else if(pltrel == DT_RELA)
		return reloc(*(Rel*)(jmprel+ofs),
			((Rela*)(jmprel+ofs))->r_addend);
	THROW(Error("unexpected symbol type"));
}


// XXX/DEP -- I386 (feb)
#define MANGLED_resolve	"resolve__Q25Elf329link_info"
#define MANGLED_resolver "resolver__Q25Elf329link_infoUl"
__asm__(".globl "MANGLED_resolve";" 
	MANGLED_resolve ": "
#ifdef __PIC__
		"call "MANGLED_resolver"@PLT;"
#else
		"call "MANGLED_resolver";"
#endif
          	"addl $8,%esp; "
         	"jmp *(%eax)"
);



Elf32::link_info::link_info(Word adjust, const Dyn *dynamic,
					bool preinitialized)
{
	memset(this, 0, sizeof(*this));
	loadaddr = adjust;
	dynamic_addr = dynamic;
	for(const Dyn *dp = dynamic; dp->d_tag != DT_NULL; dp++) {
		const Dyn &d = *dp;
		switch(d.d_tag) {
		case DT_PLTRELSZ:
			pltrelsz = d.d_val; break;
		case DT_PLTGOT:
			pltgot = (Addr*)(d.d_ptr + adjust); break;
		case DT_HASH:
			hash = (Word*)(d.d_ptr + adjust); break;
		case DT_STRTAB:
			strtab = (const char*)(d.d_ptr + adjust); break;
		case DT_SYMTAB:
			symtab = (Sym*)(d.d_ptr + adjust); break;
		case DT_RELA:
			rela = d.d_ptr + adjust; break;
		case DT_RELASZ:	  relasz = d.d_val; break;
		case DT_RELAENT:  relaent = d.d_val; break;
		case DT_STRSZ:	  strsz = d.d_val; break;
		case DT_SYMENT:	  syment = d.d_val; break;
		case DT_INIT:
			init = (void (*)())(d.d_ptr + adjust); break;
		case DT_FINI:
			fini = (void (*)())(d.d_ptr + adjust); break;
		case DT_SYMBOLIC: symbolic = true; break;
		case DT_REL:
			rel = d.d_ptr + adjust; break;
		case DT_RELSZ:	  relsz = d.d_val; break;
		case DT_RELENT:	  relent = d.d_val; break;
		case DT_PLTREL:	  pltrel = d.d_val; break;
		case DT_TEXTREL:  textrel = true; break;
		case DT_JMPREL:
			jmprel = d.d_ptr + adjust; break;
		case DT_DEBUG:
			debug = (Debug*)(d.d_ptr + adjust); break;
		}
	}
	if(!strtab)
		THROW(Error("No dynamic string table"));
	for(const Dyn *dp = dynamic; dp->d_tag != DT_NULL; dp++) {
		const Dyn &d = *dp;
		switch(d.d_tag) {
		case DT_SONAME:
			libname = &strtab[d.d_val];
			break;
		}
	}
	initialized = preinitialized;
}


void Elf32::link_info::dump()
{
	printf("link_info for %s (%p)\n", libname, this);
	for(const Dyn *dp = dynamic_addr; dp->d_tag != DT_NULL; dp++) {
		const Dyn &d = *dp;
		printf("%08lx %s: ",
			d.d_val, d_tag_str(d.d_tag));
		switch(d.d_tag) {
		case DT_NEEDED:
		case DT_SONAME:
		case DT_RPATH:
			printf("%s", &strtab[d.d_val]);
		}
		printf("\n");
	}
}


void Elf32::link_info::load_dependencies()
{
bool libc_needed = true;
	printf("** loading depend for %s\n", libname);
	printf("chain=%p\n", chain);
	for(const Dyn *dp = dynamic_addr; dp->d_tag != DT_NULL; dp++) {
		const Dyn &d = *dp;
		switch(d.d_tag) {
		case DT_RPATH:
			printf("Warning: RPATH not supported (%s)\n",
				&strtab[d.d_val]);
			break;
		case DT_NEEDED:
			printf("we(%s) need %s\n", libname, &strtab[d.d_val]);
			load_shared_lib(chain, &strtab[d.d_val]);
			if(!strncmp(&strtab[d.d_val], "libc.", 5))
				libc_needed = false;
			printf("got it\n");
			break;
		}
	}
	if(libc_needed) {
		printf("getting libc for %s\n", libname);
		load_shared_lib(chain, "libc.so");
		load_shared_lib(chain, "_System_");
	}
	printf("done depend for %s\n", libname);
}


void Elf32::link_info::initialize()
{
	if(initializing || initialized) 
		THROW(Error("reinitializing library"));
	initializing = true;
	printf("initializing %s\n", libname);
	printf("doing reloc: %lx, %lx\n", relsz, relent);
	for(uint i=0; i<relsz; i += relent) {
		Rel &r = *(Rel*)(rel+i);
		// elf32->dump(r, (Sym*)symtab, strtab);
		reloc(r, loadaddr, bind_now);
	}
	printf("doing reloc2: %lx, %lx\n", relasz, relaent);
	for(uint i=0; i<relasz; i += relaent) {
		Rela &r = *(Rela*)(rel+i);
		// elf32->dump(r, (Sym*)symtab, strtab);
		reloc((Rel)r, r.r_addend, bind_now);
	}
	printf("link table: %p\n", this);
	pltgot[1] = (Addr)this;
	pltgot[2] = (Addr)resolve;
	if(pltrel == DT_REL) 
		for(uint i=0; i<pltrelsz; i += relent) {
			Rel &r = *(Rel*)(jmprel+i);
			// elf32->dump(r, (Sym*)symtab, strtab);
			reloc(r, loadaddr, bind_now);
		}
	else if(pltrel == DT_RELA)  
		for(uint i=0; i<pltrelsz; i += relaent) {
			Rela &r = *(Rela*)(jmprel+i);
			// elf32->dump(r, (Sym*)symtab, strtab);
			reloc((Rel)r, r.r_addend, bind_now);
		}
	printf("calling init: %p\n", init);
	if(init)
		init();
	// XXX/??? -- should we look this up? (atexit) if we're a 
	//  dynamically-loaded dynamic linker, we should,
	//  otherwise, we should already know where it is... (feb)
	if(fini)
		atexit(fini);
	initializing = false;
	initialized = true;
	printf("done %s\n", libname);
}


void Elf32::load_shared_lib(dyn_chain *chain, const char *name)
{
link_info *link = chain->link;
dyn_chain **dynp = &chain->next_level;
	printf("loading %s\n", name);
	while(*dynp) 
		dynp = &(*dynp)->next;
	while(link->prev)
		link = link->prev;
	link->chain->dump();
	for(link_info *linkp = link; linkp; linkp = linkp->next) {
		printf("checking %s\n", linkp->libname);
		if(linkp->libname && !strcmp(linkp->libname, name)) {
			printf("already loaded at %p\n", linkp);
			*dynp = new dyn_chain(linkp);
			printf("chain:\n");
			chain->dump();
			return;
		}
	}

	// XXX - should check the cache (/etc/ld.so.cache?) first
	const vfile *newfile;
	vstream *newstream;
	if(!(newfile = thread->get_cwd()->get_file(name, ".:/usr:/lib")))
		THROW(Error("Cannot find library"));
	if(!(newstream = newfile->open())) 
		THROW(Error("Cannot open library"));
	delete newfile;
	Elf32 elf(newstream);
	elf.load();
	printf("loaded, checking name\n");
	for(link_info *linkp = link; linkp; linkp = linkp->next) {
		printf("checking %s\n", linkp->libname);
		if(linkp->libname
		   && !strcmp(linkp->libname, elf.link->libname)) {
			printf("already loaded at %p\n", linkp);
			*dynp = new dyn_chain(linkp);
			printf("chain:\n");
			chain->dump();
			return;
		}
	}
	printf("got new one, old chain:\n");
	chain->dump();
	printf("dump done\n");
	*dynp = elf.link->chain;
	printf("a\n");
	elf.link->chain_top = chain->link->chain_top;
	printf("b\n");
	link = chain->link;
	printf("c\n");
	printf("link=%p\n", link);
	while(link->next)
		link = link->next;
	printf("d\n");
	printf("link=%p\n", link);
	link->next = elf.link;
	printf("link=%p\n", link);
	printf("e\n");
	elf.link->prev = link;
	printf("f\n");
	elf.link->next = 0;
	printf("chain top:\n");
	link->chain_top->dump();
	printf("g\n");
	elf.link->load_dependencies();
}


void Elf32::attach(link_info *newlink)
{
	newlink->next = link;
	newlink->prev = 0;
	if(link)
		link->prev = newlink;
	link = newlink;
}



// returns address adjustment
Elf32::Addr Elf32::load() 
{
Addr min = ADDR_MAX;
Addr max = ADDR_MIN;
Word size, adjust;
void *region;
	for(int i=0; i<ehdr.e_phnum; i++) {
		Phdr phdr;
		read(phdr, i);
		if(phdr.p_vaddr < min)
			min = phdr.p_vaddr;
		if(phdr.p_vaddr + phdr.p_memsz > max)
			max = phdr.p_vaddr + phdr.p_memsz;
	}
	size = max-min;
	printf("va: %lx, sz:%lx\n", min, size);
	if(ehdr.e_type == ET_EXEC) 
		printf("WARNING: can't load at specified addresses\n");
	region = (void*)(new char[size]);
	adjust = (int)region - min;
	printf("r:%p adj:%lx\n", region, adjust);
	for(int i=0; i<ehdr.e_phnum; i++) {
		Phdr phdr;
		read(phdr, i);
		if(phdr.p_memsz && phdr.p_filesz) {
			printf("reading %lx at %lx\n",
				phdr.p_memsz, phdr.p_vaddr+adjust);
			stream->read_at(phdr.p_offset,
				(void*)(phdr.p_vaddr+adjust),
				phdr.p_memsz);
		}
		switch(phdr.p_type) {
		case PT_LOAD:
		case PT_PHDR:
			break;
		case PT_DYNAMIC:
			/*
			 if(!first_link) {
				first_link = new link_info(this, 0, _DYNAMIC);
				first_link->dump();
			}
			{
				link_info &link = *(new link_info(this, adjust,
					(Dyn*)(phdr.p_vaddr+adjust)));
				link.prev = first_link;
				link.next = first_link->next;
				first_link->next = &link;
				link.dump();
				link.initialize();
			}
			*/
			{
				link_info *newlink = new link_info(adjust,
					(Dyn*)(phdr.p_vaddr+adjust));
				newlink->chain = new dyn_chain(newlink);
				newlink->chain_top = newlink->chain;
				newlink->dump();
				attach(newlink);
			}
			break;
		case PT_INTERP:
			printf("interpreter: %s\n",
				(char*)(phdr.p_vaddr+adjust));
			printf("WARNING: Interpreter not supported\n");
			// THROW(Error("Interpreter not supported"));
			break;
		default:
			break;
		}
	}
	printf("loaded\n");
	return adjust;
}


void Elf32::exec()
{
Addr adjust;
	adjust = load();
	load_dependencies();
	initialize();
	link->chain->dump();
	printf("running at %lx/%lx\n", ehdr.e_entry, ehdr.e_entry+adjust);
	((void (*)())(ehdr.e_entry+adjust))();
	printf("done\n");
}


