//  Elf32.H                                                     (CRC: e6ae576)
//      relative path: ./lib/elf
//      creation date: December 1995
//      last modified: Sat Oct 26 15:08:46 1996
//      header update: Sat Oct 26 15:08:47 1996
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       Libraries
//        Elf-format binary control
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

#ifndef ASM

#include <vstream>
#include <vdir>



class Elf32 {
public:
	typedef unsigned long	Addr;
	typedef unsigned short	Half;
	typedef unsigned long	Off;
	typedef long		Sword;
	typedef unsigned long	Word;
	typedef unsigned char	uchar;
	typedef unsigned long	ulong;
	const Addr ADDR_MAX = -1UL;
	const Addr ADDR_MIN = 0;

// main file header:
	
	enum e_ident_indices {
		EI_MAG0 = 0, EI_MAG1, EI_MAG2, EI_MAG3,	// file id
		EI_CLASS,				// file class
		EI_DATA,				// data encoding
		EI_VERSION,				// file version
		EI_PAD,					// start of padding
		EI_NIDENT = 16				// size of e_ident
	};
	const char MAG0 = '\x7f';
	const char MAG1 = 'E';
	const char MAG2 = 'L';
	const char MAG3 = 'F';
	enum ei_class {
		ELFCLASSNONE = 0,	// invalid class
		ELFCLASS32 = 1,		// 32-bit objects
		ELFCLASS64 = 2		// 64-bit objects
	};
	enum ei_data {
		ELFDATANONE = 0,	// invalid encoding
		ELFDATA2LSB = 1,	// little endian
		ELFDATA2MSB = 2		// big endian
	};

	struct Ehdr {
		uchar	e_ident[EI_NIDENT];
		Half	e_type;		// object file type
		Half	e_machine;
		Word	e_version;
		Addr	e_entry;	// virtual execution address
		Off	e_phoff;	// program header offset
		Off	e_shoff;	// section header offset
		Word	e_flags;
		Half	e_ehsize;	// ELF header size
		Half	e_phentsize;	// program header entry size
		Half	e_phnum;	// number of ph entries
		Half	e_shentsize;	// section header entry size
		Half	e_shnum;	// number of sh entries
		Half	e_shstrndx;	// index to string table
	};

	enum e_type {
		ET_NONE = 0,		// no file type
		ET_REL = 1,		// relocatable file
		ET_EXEC = 2,		// executable file
		ET_DYN = 3,		// shared object file
		ET_CORE = 4,		// core file
		// ET_LOPROC = 0xff00,	// processor-specific
		// ET_HIPROC = 0xffff	// processor-specific
		ET_LOPROC = 5,		// processor-specific
		ET_HIPROC = 6		// processor-specific
	};
	enum e_machine {
		EM_NONE = 0,		// no machine
		EM_M32 = 1,		// AT&T WE 32100
		EM_SPARC = 2,		// SPARC
		EM_386 = 3,		// Intel 80386
		EM_68K = 4,		// Motorola 68000
		EM_88K = 5,		// Motorola 88000
		EM_486 = 6,   		// Intel 486 -- not used?
		EM_860 = 7,		// Intel 80860
		EM_MIPS = 8		// MIPS RS3000
	};
	enum e_version {
		EV_NONE = 0,		// invalid version
		EV_CURRENT = 1		// current version
	};

// section headers:

	struct Shdr {
		Word	sh_name;	// index to name of section
		Word	sh_type;	// section type
		Word	sh_flags;	 
		Addr	sh_addr;	// memory address to load at
		Off	sh_offset;	// data offset into file
		Word	sh_size;	// size of section
		Word	sh_link;	// index to another section
			// SHT_DYNAMIC:		string table
			// SHT_SYMTAB,DYNSYM:	string table
			// SHT_REL,RELA:	symbol table
			// SHT_HASH:		symbol table
		Word	sh_info;	// section-dependent
		Word	sh_addralign;	// 0,1,2,4,8 etc..
		Word	sh_entsize;	// item size for entries
	};
	
	enum sh_num {
		SHN_UNDEF = 0,
		SHN_LORESERVE = 0xff00,
		SHN_LOPROC = 0xff00,
		SHN_HIPROC = 0xff1f,
		SHN_ABS = 0xfff1,	// absolute values
		SHN_COMMON = 0xfff2,	// unallocated externals
		SHN_HIRESERVE = 0xffff
	};
	enum sh_type {
		SHT_NULL = 0,
		SHT_PROGBITS = 1,	// program-specific
		SHT_SYMTAB = 2,		// symbol table
		SHT_STRTAB = 3,		// string table
		SHT_RELA = 4,		// relocation w/ addends
		SHT_HASH = 5,		// symbol hash table
		SHT_DYNAMIC = 6,	// info for dynamic linking 
		SHT_NOTE = 7,		 
		SHT_NOBITS = 8,		// empty in file
		SHT_REL = 9,		// relocation w/o addends
		SHT_SHLIB = 10,		// reserved
		SHT_DYNSYM = 11,	// dynamic symbol table
		SHT_NUM = 12,		// ?
		SHT_LOPROC = 0x70000000,
		SHT_HIPROC = 0x7fffffff,
		SHT_LOUSER = 0x80000000,
		SHT_HIUSER = 0xffffffff
	};
	enum sh_flags {
		SHF_WRITE = 0x1,	// data must be writeable
		SHF_ALLOC = 0x2,	// must occupy memory
		SHF_EXECINSTR = 0x4,	// executable code
		SHF_MASKPROC = 0xf0000000
	};

	// Special sections:
	//	.bss, .comment, .data, .data1, .debug, .dynamic
	//	.dynstr, .dynsym, .fini, .got, .hash, .init, .interp
	//	.line, .note, .plt, .rel(name), .rela(name), .rodata
	//	.rodata1, .shrstrtab, .strtab, .symtab, .text

// symbol tables:

	enum st_num {
		STN_UNDEF = 0		// undefined symbol
	};
	enum st_bind {
		STB_LOCAL = 0,		// not visible outside obj file
		STB_GLOBAL = 1,		// visible to all object files
		STB_WEAK = 2,		// weak globals
		STB_LOPROC = 13,
		STB_HIPROC = 15
	};
	enum st_type {
		STT_NOTYPE = 0,		// unspecified
		STT_OBJECT = 1,		// data object
		STT_FUNC = 2,		// function or code
		STT_SECTION = 3,	// associated with a section
		STT_FILE = 4,		// source file name
		STT_LOPROC = 13,
		STT_HIPROC = 15
	};
	struct Sym {
		Word	st_name;	// index to sym name
		Addr	st_value;	// value/addr of symbol
		Word	st_size;	// # bytes if known
		uchar	st_info;	// type/binding
		uchar	st_other;	// reserved
		Half	st_shndx;	// section it relates to
		enum st_bind st_bind() const
			{ return (enum st_bind)(st_info >> 4); }
		enum st_type st_type() const
			{ return (enum st_type)(st_info & 0xf); }
		void set_st_info(enum st_bind b, enum st_type t)
			{ st_info = (b<<4)|(t&0xf); }
	};

// relocation tables:

	// DEP -- I386  
	enum r_type {
		R_386_NONE = 0,		// no calc
		R_386_32 = 1,		// S + A
		R_386_PC32 = 2,		// S + A - P
		R_386_GOT32 = 3,	// G + A - P
		R_386_PLT32 = 4,	// L + A - P
		R_386_COPY = 5,		// none
		R_386_GLOB_DAT = 6,	// S
		R_386_JMP_SLOT = 7,	// S
		R_386_RELATIVE = 8,	// B + A
		R_386_GOTOFF = 9,	// S + A - GOT
		R_386_GOTPC = 10	// GOT + A - P
		// A = addend
		// B = base address for shared object execution
		// G = offset into GOT
		// GOT = address of GOT
		// L = offset/address of PLT entry
		// P = offset/address of r_offset
		// S = value of symbol indexed by 'r_sym'
	};
	struct Rel {
		Addr	r_offset;
		Word	r_info;
		Word r_sym() const
			{ return r_info >> 8; }
		enum r_type r_type() const
			{ return (enum r_type)((uchar)r_info); }
		void set_r_info(Word s, enum r_type t)
			{ r_info = (s<<8)|(uchar)t; }
	};
	struct Rela : public Rel {
		Sword	r_addend;
	};

// program headers:

	struct Phdr {
		Word	p_type;		// type of segment
		Off	p_offset;	// offset into file
		Addr	p_vaddr;	// virtual address of segment
		Addr	p_paddr;	// physical address if needed
		Word	p_filesz;	// file image size of segment
		Word	p_memsz;	// memory image size of segment
		Word	p_flags;	 
		Word	p_align;	// 0,1,2,4,8,etc...
	};
	enum p_type {
		PT_NULL = 0,		// unused
		PT_LOAD = 1,		// loadable segment
		PT_DYNAMIC = 2,		// dynamic linking info
		PT_INTERP = 3,		// location/size of interpreter
		PT_NOTE = 4,
		PT_SHLIB = 5,		// reserved
		PT_PHDR = 6,		// location/size of header
		PT_LOPROC = 0x70000000,
		PT_HIPROC = 0x7fffffff
	};
	
// dynamic linking:

	struct Dyn {
		Sword	d_tag;
		union {
			Word	d_val;
			Addr	d_ptr;
		};
	};
	enum d_tag {			// ptr/val, exec, share
		DT_NULL = 0,		// ign, req, req
		DT_NEEDED = 1,		// val, opt, opt
		DT_PLTRELSZ = 2,	// val, opt, opt
		DT_PLTGOT = 3,		// ptr, opt, opt
		DT_HASH = 4,		// ptr, req, req
		DT_STRTAB = 5,		// ptr, req, req
		DT_SYMTAB = 6,		// ptr, req, req
		DT_RELA = 7,		// ptr, req, opt
		DT_RELASZ = 8,		// val, req, opt
		DT_RELAENT = 9,		// val, req, opt
		DT_STRSZ = 10,		// val, req, req
		DT_SYMENT = 11,		// val, req, req
		DT_INIT = 12,		// ptr, opt, opt
		DT_FINI = 13,		// ptr, opt, opt
		DT_SONAME = 14,		// val, ign, opt
		DT_RPATH = 15,		// val, opt, ign
		DT_SYMBOLIC = 16,	// ign, ign, opt
		DT_REL = 17,		// ptr, req, opt
		DT_RELSZ = 18,		// val, req, opt
		DT_RELENT = 19,		// val, req, opt
		DT_PLTREL = 20,		// val, opt, opt
		DT_DEBUG = 21,		// ptr, opt, ign
		DT_TEXTREL = 22,	// ign, opt, opt
		DT_JMPREL = 23,		// ptr, opt, opt
		DT_LOPROC = 0x70000000,
		DT_HIPROC = 0x7fffffff
	};

	class link_info;
	struct dyn_chain;
	
	struct Debug {
		int	r_version;	// debugging info version num
		link_info *r_map;	// list of maps
		ulong	r_brk;		// update routine
		enum {
			RT_CONSISTENT,
			RT_ADD,
			RT_DELETE
		} r_state;
		ulong	r_ldbase;	// base of ld.so
	};

	class link_info {
	public:
		friend dyn_chain;
		// needed for gdb compatability
		Addr loadaddr;
		const char *libname;
		const Dyn *dynamic_addr;
		link_info *next;
		link_info *prev;
		// not needed for gdb:
		dyn_chain *chain;
		dyn_chain *chain_top;
	private:
		Word pltrelsz;
		Addr *pltgot;
		Word *hash;
		const char *strtab;
		Sym *symtab;
		Addr rela;
		Word relasz;
		Word relaent;
		Word strsz;
		Word syment;
		void (*init)();
		void (*fini)();
		bool symbolic;
		Addr rel;
		Word relsz;
		Word relent;
		Word pltrel;
		bool textrel;
		Addr jmprel;
	
		bool bind_now;
		bool initializing;
		bool initialized;

		Addr resolver(Word ofs);
		void resolve();
	public:
		Debug *debug;
		link_info(Word adjust, const Dyn *dyn,
					bool preinitialized = false);
		Addr reloc(const Rel &r, Sword addend, bool bind_jmp = true);
		Word symval(Word snum);
		Word symhash(const char *name, Word hnum);
		Word symhash(const char *name)
			{ return symhash(name, Elf32::hash(name)); }
		void load_dependencies();
		void initialize();
		void dump();
	};

	struct dyn_chain {
	public:
		link_info *link;
		dyn_chain *next;
		dyn_chain *next_level;
		dyn_chain(link_info *newlink)
			{ link = newlink; next = 0; next_level = 0; }
		Word lookup(const char *name);
		void dump(int indent = 0);
	private:
		Sym* lookup_level(int level, const char *name,
					Word hnum, link_info **linkp);
	};

private:
	Ehdr ehdr;
	char *shstrtab;
	vstream *stream;	// current file
	link_info *link;
public:
	Elf32(vstream *stream);
	~Elf32();
	static const char* ei_class_str(int n);
	static const char* ei_data_str(int n);
	static const char* e_type_str(int n);
	static const char* e_machine_str(int n);
	static const char* sh_type_str(int n);
	static const char* r_type_str(int n);
	static const char* p_type_str(int n);
	static const char* d_tag_str(int n);
	const char* shstr(Word index) const;
	const char* section(int shnum) const;
	static ulong hash(const uchar *name);
	static ulong hash(const char *name)
		{ return hash((const uchar*)name); }
	void Elf32::read(Phdr &phdr, int i) const;
	void Elf32::read(Shdr &shdr, int i) const;
	static void dump(const Ehdr &e);
	static void dump(const Phdr &p);
	void dump(const Shdr &s) const;
	void dump(const Sym &s, const char *strtab) const;
	void dump(const Rel &r, const Sym *sym,
			const char *strtab) const;
	void dump(const Rela &r, const Sym *sym,
			const char *strtab) const;
	void dump() const;
	Addr load();
	void attach(link_info *newlink);
	void initialize() { if(link) link->initialize(); }
	void load_dependencies()
		{ if(link) link->load_dependencies(); }
	void exec();
	static void load_shared_lib(dyn_chain *chain, const char *name);
	void dynamic_link(Word adjust, const Dyn *dynamic);
};


extern Elf32::Dyn _DYNAMIC[];
extern Elf32::Addr _GLOBAL_OFFSET_TABLE_[];


#else

#include "Elf32.ofs"
#include <endian.h>

#define Elf32_Ehdr_magic	0x00
#define Elf32_Ehdr_class	0x04
#define Elf32_Ehdr_data		0x05
#define Elf32_Ehdr_version	0x06

#define ELFMAG0		0x7f
#define ELFMAG1		0x45
#define ELFMAG2		0x4c
#define ELFMAG3		0x46
#if BYTE_ORDER == BIG_ENDIAN
#define ELFMAG	0x7f454c46	// "0x7f""ELF"	
#else
#define ELFMAG	0x464c457f	// "0x7f""ELF"
#endif

// XXX - should be able to autogenerate this as well

#define ELFCLASSNONE		0
#define ELFCLASS32		1
#define ELFCLASS64		2

#define ELFDATANONE		0
#define ELFDATA2LSB		1
#define ELFDATA2MSB		2

#define ET_NONE		0
#define ET_REL		1
#define ET_EXEC		2
#define ET_DYN		3
#define ET_CORE		4

#define EM_NONE		0
#define EM_M32		1
#define EM_SPARC	2
#define EM_386		3
#define EM_68K		4
#define EM_88K		5
#define EM_486		6
#define EM_860		7
#define EM_MIPS		8

#define EV_NONE		0
#define EV_CURRENT	1


#endif

