/*  Error.h                                                     (CRC:a7320ec0)
 *      relative path: ./lib/elf
 *      creation date: Wed Jan 31 10:17:19 1996
 *      last modified: Sat Oct 26 15:08:51 1996
 *      header update: Sat Oct 26 15:08:52 1996
 *  
 *      Shag/OS: an Object-Oriented MicroKernel Operating System
 *       Libraries
 *        Elf-format binary control
 *  
 *  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution.
 *  
 *  Shag/OS is free software for non-profit uses; it may be freely distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'COPYING'.  If this license was not received,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For bug reports, please read the file 'BUGS' first, then include the 
 *      product name and version, and the word 'bug' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

class Error {
protected:
	const char *name;
	const char *func;
public:
	const char* get_name() const { return name; }
	const char* get_func() const { return func; }
	Error(const char *s) { name = s; }
	virtual char* msg(char *buf) const;
	Error& at(const char *f) { func = f; return *this; }
};


void unexpected(const Error &e) __attribute__((noreturn));


#ifdef HANDLE_EXCEPTIONS
#pragma Make Flags_Needed += -fhandle-exceptions
#define	THROW(e)	throw e
#define	TRY_BEGIN	try {
#define TRY_END		}
#define CATCH(o)	catch(o)
#define CATCH_ALL	catch(...)

#else 

#ifdef __dynamic__
// XXX -- must change THROW to actually return an error
#define THROW(e)	{ unexpected(e.at(__PRETTY_FUNCTION__)); }
#define TRY_BEGIN	
#define TRY_END
#define CATCH(o)	if(0)
#define CATCH_ALL

// #define TRY_BEGIN	{ obj _result_;
// #define TRY_END		} catch_block:
// #define CATCH(o)	if(_result_.istype(o)) 
// #define CATCH_ALL	
// #define OBJCALL(f)	if(Error((_result_ = f))) goto catch_block;

#else
#define THROW(e)	{ unexpected(e.at(__PRETTY_FUNCTION__)); }
#define TRY_BEGIN	
#define TRY_END
#define CATCH(o)	if(0)
#define CATCH_ALL
#endif
#endif

#ifdef ideas

TRY_BEGIN
	x
	OBJCALL(y)
	z
TRY_END
CATCH(a) {
	stuff
}
CATCH_ALL {
	morestuff
}

becomes:

try {
	x
	y (ocall)
	z
}
catch(a) {
	stuff
}
catch(...) {
	morestuff
}

or:

	obj result;
	x
	if(fail(result = y)) 
		goto catch_block
	z	
catch_block:
	if(result_matches(a)) {
		stuff
	}
	{
		morestuff
	}
#endif

