//  File.C                                                      (CRC:d13e3340)
//      relative path: ./lib/elf
//      creation date: Sun Jan 28 22:16:18 1996
//      last modified: Sat Oct 26 15:08:50 1996
//      header update: Sat Oct 26 15:08:51 1996
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       Libraries
//        Elf-format binary control
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

#pragma Make Includes_Banned += -nostdinc -nostdinc++
#pragma Make Flags_Banned += -static

#pragma Implementation
#define IMPLEMENTATION_File
 
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/param.h>

#include "Error.h"
#include "Perror.h"
#include "File.h"


extern "C" int printf(const char *fmt, ...);

File::File(const char *fname, const char *path)
{
	if(!fname)
		THROW(Error("null filename"));
	if(fname[0] == '/')
		path = "/";
	else if(!path)
		path = ".";
	while(*path) {
		char buf[PATH_MAX];
		char *cp;
		while(*path && (*path <= ' ' || *path == ':' || *path == ';'))
			path++;
		if(!*path)
			break;
		for(cp = buf; *path > ' ' && *path != ':'
				&& *path != ';'; cp++, path++) 
			*cp = *path;
		--cp;
		if(*cp++ != '/')
			*cp++ = '/';
		strcpy(cp, fname);
		if((_fd = open(buf, O_RDONLY)) >= 0) {
			printf("opened %s\n", buf);
			return;
		}
	}
	THROW(Perror(fname));
}

 
File::~File()
{
	if(close(_fd) < 0)
		THROW(Perror(_fd));
}

 
ssize_t File::read(DStr buf) const
{
	int n = ::read(_fd, buf.ptr, buf.len);
	if(n < 0)
		THROW(Perror(buf.len));
	return n;
}


ssize_t File::write(CStr buf) const
{
	int n = ::write(_fd, buf.ptr, buf.len);
	if(n < 0)
		THROW(Perror(buf.len));
	return n; 
}


off_t File::seek(off_t p) const
{
	off_t n = ::lseek(_fd, p, SEEK_SET);
	if(n < 0)
		THROW(Perror(p));
	return n; 
}


off_t File::tell() const
{
	off_t n = ::lseek(_fd, 0, SEEK_CUR);
	if(n < 0)
		THROW(Perror(_fd));
	return n;
}


ssize_t File::read_at(off_t p, DStr buf) const
{
	seek(p);
	return read(buf);
}


ssize_t File::write_at(off_t p, CStr buf) const
{
	seek(p);
	return write(buf); 
};


