/*  File.h                                                      (CRC:6139a9c1)
 *      relative path: ./lib/elf
 *      creation date: Wed Jan 31 10:34:04 1996
 *      last modified: Sat Oct 26 15:08:48 1996
 *      header update: Sat Oct 26 15:08:48 1996
 *  
 *      Shag/OS: an Object-Oriented MicroKernel Operating System
 *       Libraries
 *        Elf-format binary control
 *  
 *  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution.
 *  
 *  Shag/OS is free software for non-profit uses; it may be freely distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'COPYING'.  If this license was not received,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For bug reports, please read the file 'BUGS' first, then include the 
 *      product name and version, and the word 'bug' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

#pragma interface

#include <unistd.h>
#include "Error.h"
#include "vector.H"

class File {
	int _fd;
public:
	File(const char *fname, const char *path = 0);
	File(int fd) { _fd = fd; }
	~File();
	ssize_t read(DStr buf) const;
	ssize_t write(CStr buf) const;
	ssize_t read(void *adr, size_t size) const
		{ read(DStr((char*)adr, size)); }
	ssize_t write(const void *adr, size_t size) const
		{ write(CStr((const char*)adr, size)); }
	off_t seek(off_t p) const;
	off_t tell() const;
	ssize_t read_at(off_t p, DStr buf) const;
	ssize_t write_at(off_t p, CStr buf) const;
	ssize_t read_at(off_t p, void *adr, size_t size) const
		{ read_at(p, DStr((char*)adr, size)); }
	ssize_t write_at(off_t p, const void *adr, size_t size) const
		{ write_at(p, CStr((const char*)adr, size)); }
};



#ifdef __dynamic__

#ifndef __dynamic_interface__
#include "File.dh"
#else

interface "File.dh"
	Class File: File
		read buf
		write buf
end

#endif // __dynamic_interface__

#endif // __dynamic__

