/*  $Id: Port.h,v 1.3 1996/10/26 19:08:33 shaggy Exp $
 *      I/O Port access
 *      
 *      relative path: ./lib/hw/m-i386
 *  
 *      Shag/OS: an Object-Oriented MicroKernel Operating System
 *       Libraries
 *        Hardware-specific
 *         (m-i386)
 *  
 *  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution.
 *  
 *  Shag/OS is free software for non-profit uses; it may be freely distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'COPYING'.  If this license was not received,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For bug reports, please read the file 'BUGS' first, then include the 
 *      product name and version, and the word 'bug' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

/*  $Log: Port.h,v $
 *  Revision 1.3  1996/10/26  19:08:33  shaggy
 *  header update
 *
 *  Revision 1.2  1996/09/29  01:09:16  shaggy
 *  Faster, more efficient, general variable-like usage
 *
 *  Revision 1.1  1994/12/28  18:34:54  shaggy
 *  Initial revision
 *
 */


#include <types.h>


// notes:
//	indb = in double-byte (low byte first)
//	outdb = out double-byte (low byte first)
 

// a lot of funky macros could have been written to generate all
// the variations, but that would make this file harder to read,
// and make it take a lot longer to parse because all of the macros
// would have to be evaluated every time this header was used
// Besides, this way, any of the functions can be easily hand-tuned 
// for optimizations 

// Note that separate "constant" versions of these instructions
// are not needed-- they will be automatically generated when
// they apply

// XXX - oddly enough, the inXX() operations don't use the constants,
// even though the outXX() ones do... it appears that when there
// is an output contraint for an asm instruction, it never picks
// the 'N' input constraint...  if anyone knows how to fix this,
// please contact me


class Port {
protected:
	const uint16 base;
public:
	Port()	: base(0) {}
	Port(uint16 n) : base(n) {}

	void	io_delay() const volatile {
			asm("outb %al,$0x80"); }

	uint8	inb(uint16 p = 0) const volatile {
			register uint8 v;
		 	asm("inb %1,%0":"a="(v):"Nd"((uint16)(p+base)));
			return v; }
	uint16	indb(uint16 p = 0) const volatile {
			register uint16 v;
		 	asm("inb %1,%%al":"a="(v):"Nd"((uint16)(p+base)));
		 	asm("movb %%al,%%ah; inb %2,%%al; xchgb %%al,%%ah"
				:"a="(v):"0"(v),"Nd"((uint16)(p+base)));
			return v; }
	uint16	inw(uint16 p = 0) const volatile {
			register uint16 v;
		 	asm("inw %1,%0":"a="(v):"Nd"((uint16)(p+base)));
			return v; }
	uint32	inl(uint16 p = 0) const volatile {
			register uint32 v;
		 	asm("inw %1,%0":"a="(v):"Nd"((uint16)(p+base)));
			return v; }
	void	outb(uint8 v, uint16 p = 0) const volatile {
			asm("outb %0,%1": :"a"(v),"Nd"((uint16)(p+base))); }
	void	outdb(uint16 v, uint16 p = 0) const volatile {
			asm("outb %%al,%1":
				:"a"(v),"Nd"((uint16)(p+base))); 
			asm("xchgb %%ah,%%al; outb %%al,%1; xchgb %%ah,%%al":
				:"a"(v),"Nd"((uint16)(p+base))); }
	void	outw(uint16 v, uint16 p = 0) const volatile {
			asm("outw %0,%1": :"a"(v),"d"((uint16)(p+base))); }
	void	outl(uint32 v, uint16 p = 0) const volatile {
			asm("outw %0,%1": :"a"(v),"d"((uint16)(p+base))); }

	// pausing versions:
	uint8	inb_p(uint16 p = 0) const volatile {
			register uint8 v = inb(p);
			io_delay(); return v; }
	uint16	inw_p(uint16 p = 0) const volatile {
			register uint16 v = inw(p);
			io_delay(); return v; }
	uint32	inl_p(uint16 p = 0) const volatile {
			register uint32 v = inw(p);
			io_delay(); return v; }
	uint16	indb_p(uint16 p = 0) const volatile {
			register uint16 v;
		 	asm("inb %1,%%al":"a="(v):"Nd"((uint16)(p+base)));
			io_delay();
		 	asm("movb %%al,%%ah; inb %2,%%al; xchgb %%al,%%ah"
				:"a="(v):"0"(v),"Nd"((uint16)(p+base)));
			io_delay(); return v; }
	void	outb_p(uint8 v, uint16 p = 0) const volatile {
			outb(v, p); io_delay(); } 
	void	outw_p(uint16 v, uint16 p = 0) const volatile {
			outw(v, p); io_delay(); } 
	void	outl_p(uint32 v, uint16 p = 0) const volatile {
			outl(v, p); io_delay(); } 
	void	outdb_p(uint16 v, uint16 p = 0) const volatile {
			asm("outb %%al,%1":
				:"a"(v),"Nd"((uint16)(p+base))); 
			io_delay();
			asm("xchgb %%ah,%%al; outb %%al,%1; xchgb %%ah,%%al":
				:"a"(v),"Nd"((uint16)(p+base)) );
			io_delay(); } 
};


// Port8, Port16, and Port32 are "slow" ports that perform I/O waits
// this is to insure code safety, since fast ports won't work for
// many standard ports
// To use a fast port, use Port8F, Port16F, and Port32F

class Port8 : public Port {
public:
	Port8()	: Port(0) {}
	Port8(uint16 n) : Port(n) {}
	
	uint8 operator= (uint8 v) const { outb_p(v); return v; }
	operator uint8 () const { return inb_p(); }
	Port8 operator [] (uint16 index) const { return Port8(base+index); }
};


class Port16 : public Port {
public:
	Port16() : Port(0) {}
	Port16(uint16 n) : Port(n) {}
	
	uint16 operator= (uint16 v) const { outw_p(v); return v; }
	operator uint16 () const { return inw_p(); }
	Port16 operator [] (uint16 index) const { return Port16(base+index); }
};


class Port32 : public Port {
public:
	Port32() : Port(0) {}
	Port32(uint16 n) : Port(n) {}
	
	uint32 operator= (uint32 v) const { outl_p(v); return v; }
	operator uint32 () const { return inl_p(); }
	Port32 operator [] (uint16 index) const { return Port32(base+index); }
};


class FastPort8 : public Port {
public:
	FastPort8() : Port(0) {}
	FastPort8(uint16 n) : Port(n) {}
	
	uint8 operator= (uint8 v) const { outb(v); return v; }
	operator uint8 () const { return inb(); }
	FastPort8 operator [] (uint16 index) const
		{ return FastPort8(base+index); }
};



class FastPort16 : public Port {
public:
	FastPort16() : Port(0) {}
	FastPort16(uint16 n) : Port(n) {}
	
	uint16 operator= (uint16 v) const { outw(v); return v; }
	operator uint16 () const { return inw(); }
	FastPort16 operator [] (uint16 index) const
		{ return FastPort16(base+index); }
};


class FastPort32 : public Port {
public:
	FastPort32() : Port(0) {}
	FastPort32(uint16 n) : Port(n) {}
	
	uint32 operator= (uint32 v) const { outl(v); return v; }
	operator uint32 () const { return inl(); }
	FastPort32 operator [] (uint16 index) const
		{ return FastPort32(base+index); }
};

