//  Lexer.H                                                     (CRC:45bf84ca)
//      relative path: ./lib/parse
//      creation date: Mon Mar  4 00:21:39 1996
//      last modified: Sat Oct 26 15:10:05 1996
//      header update: Sat Oct 26 15:10:06 1996
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       Libraries
//        Generic parsing functions
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

#include <types.h>

class Lexer {
	const char *_str;	// source buffer
	size_t _len;		// remaining size of source buffer
	bool _sol;		// start of line
	int _line_num;		// current source line number
	enum ctypes {
		UPPER = 0x01,
		LOWER = 0x02,
		ALPHA = 0x03, 
		NUM = 0x04,
		ALPHANUM = 0x07,
		PUNC = 0x08,
		WHITESPACE = 0x20,
		HEXNUM = 0x40,
		OCTNUM = 0x80,
		BINNUM = 0x100,
		ID_START = 0x400,
		ID = 0x800
	}; 
	static ctypes ctype[256];
	static bool init_ctypes;
public:
	void set_src(const char *str, size_t len) 
		{ _str = str; _len = len; }
	Lexer();
	Lexer(const char *str, size_t len) { Lexer(); set_src(str, len); }
	bool sol() const { return _sol; }
	bool eol() const { return *_str == '\n'; }
	bool eof() const { return !_str; }
	void set_eof() { set_src(0, 0); }
	void set_empty() { set_src("", 0); }
	char get_char() const { return *_str; }
	int get_digit() const { return *_str-'0'; }
	int get_hex() const { if(is_num()) return get_digit();
			else return (*_str-'A'+10)&0xf; }
	bool match(char ch) const { return *_str == ch; }
	bool next_char() {
		if((_sol = (*_str == '\n'))) _line_num++;
		if(!--_len) return false;
		else { _str++; return true; }
	}
	int line_num() const { return _line_num; }
	void set_line_num(int num) { _line_num = num; }
	static bool is_upper(char ch) { return ctype[ch] & UPPER; }
	static bool is_lower(char ch) { return ctype[ch] & LOWER; }
	static bool is_alpha(char ch) { return ctype[ch] & ALPHA; }
	static bool is_num(char ch) { return ctype[ch] & NUM; }
	static bool is_bin(char ch) { return ctype[ch] & BINNUM; }
	static bool is_oct(char ch) { return ctype[ch] & OCTNUM; }
	static bool is_hex(char ch) { return ctype[ch] & HEXNUM; }
	static bool is_alphanum(char ch) 
			{ return ctype[ch] & ALPHANUM; }
	static bool is_punc(char ch) 
			{ return ctype[ch] & PUNC; }
	static bool is_whitespace(char ch) 
			{ return ctype[ch] & WHITESPACE; }
	static bool is_id_start(char ch) 
			{ return ctype[ch] & ID_START; }
	static bool is_id(char ch) 
			{ return ctype[ch] & ID; }
	bool is_empty() const { return !_len; }
	bool is_upper() const { return is_upper(*_str); }
	bool is_lower() const { return is_lower(*_str); }
	bool is_alpha() const { return is_alpha(*_str); }
	bool is_num() const { return is_num(*_str); }
	bool is_bin() const { return is_bin(*_str); }
	bool is_oct() const { return is_oct(*_str); }
	bool is_hex() const { return is_hex(*_str); }
	bool is_alphanum() const { return is_alphanum(*_str); }
	bool is_punc() const { return is_punc(*_str); }
	bool is_whitespace() const { return is_whitespace(*_str); }
	bool is_id_start() const { return is_id_start(*_str); }
	bool is_id() const { return is_id(*_str); }
};


#ifdef IMPLEMENTATION_Lexer
bool Lexer::init_ctypes = true;
enum Lexer::ctypes Lexer::ctype[256];
#endif


