//  ParsePath.C                                                 (CRC:ef3c4d9e)
//      relative path: ./lib/parse
//      creation date: Oct 1995
//      last modified: Sat Oct 26 15:10:03 1996
//      header update: Sat Oct 26 15:10:03 1996
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       Libraries
//        Generic parsing functions
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

#define IMPLEMENTATION_ParsePath
#include <ParsePath>



void ParsePath::parse(const char *path)
{
const char *basep;
const char *dirp;
char *dp = dir;
char *bp = base;
	if((absolute = (path[0] == '/')))
		++path;
	dirp = basep = path;
	while(*path) {
		if(*path == '/')
			basep = path+1;
		++path;
	}
	while(dirp != basep)
		*dp++ = *dirp++;
	while(dp != dir && *(dp-1) == '/')
		dp--;
	*dp = 0;
	while(*basep)
		*bp++ = *basep++;
	*bp = 0;
	end = bp;
	next_dir = dir;
}


bool ParsePath::get_next_dir(char *nd)
{
	if(!*next_dir)
		return false;
	while(*next_dir == '/')
		++next_dir;
	if(nd) {
		while(*next_dir && *next_dir != '/')
			*nd++ = *next_dir++;
		*nd = 0;
	} else
		while(*next_dir && *next_dir != '/')
			next_dir++;
	return true;
}


bool ParsePath::has_suffix(const char *suffix) const
{
const char *bp = end;
const char *sp = suffix;
	while(*sp) {
		if(bp-- == base)
			return false;
		++sp;
	}
	while(*suffix) {
		if(*suffix != *bp)
			return false;
		++suffix;
		++bp;
	}
	return true;
}


bool ParsePath::change_suffix
		(char *nb, const char *olds, const char *news) const
{
const char *sufstart;
const char *bp = end;
const char *sp = olds;
	while(*sp) {
		if(bp-- == base)
			return false;
		++sp;
	}
	sufstart = bp;
	while(*olds) {
		if(*olds != *bp)
			return false;
		++olds;
		++bp;
	}
	bp = base;
	while(bp != sufstart)
		*nb++ = *bp++;
	while(*news)
		*nb++ = *news++;
	*nb++ = 0;
	return true;
}


const char* ParsePath::get_suffix(const char *div) const
{
const char *bp = end;
	while(bp-- != base) {
		const char *dp = div;
		const char *mp = bp;
		while(*mp == *dp) {
			mp++;
			dp++;
			if(!*dp)
				return bp;
		}
	}
	return 0;
}


const char* ParsePath::get_ext(const char *div) const
{
const char *cp;
	if(!(cp = get_suffix(div)))
		return 0;
	while(*div) {
		cp++;
		div++;
	}
	return cp;
}


void ParsePath::get_dir_path(char *np) const
{
const char *dp;
	dp = dir;
	while(dp != next_dir) 
		*np++ = *dp++;
	*np = 0;
}

