//  ParsePath.H                                                 (CRC:a328d3a1)
//      relative path: ./lib/parse
//      creation date: Oct 1995
//      last modified: Sat Oct 26 15:10:06 1996
//      header update: Sat Oct 26 15:10:07 1996
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       Libraries
//        Generic parsing functions
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

#ifndef NAME_MAX
#define NAME_MAX	255	// max chars in a filename
#endif
#ifndef PATH_MAX
#define PATH_MAX	1024	// max chars in a pathname
#endif



// OPT: could compute each part when it is first accessed only
//	or could just make into separate classes, and inherit only
//	the parts needed


class ParsePath {
	bool absolute;
	char *end;
	char *next_dir;
	char base[NAME_MAX];
	char dir[PATH_MAX];
public:
	void parse(const char *path);
	ParsePath();
	ParsePath(const char *path)	{ parse(path); }
	const bool is_absolute() const	{ return absolute; }
	const bool has_dir() const	{ return dir[0] != 0; }
	const bool has_base() const	{ return base[0] != 0; }
	const bool is_root() const	{ return absolute && !has_dir() && !has_base(); }
	const char* get_dir() const	{ return dir; }
	const char* get_base() const	{ return base; }
	const char* get_ext(const char *div) const;
	const char* get_suffix(const char *div) const;
	bool has_suffix(const char *suffix) const;
	bool change_suffix(char *nb, const char *olds, const char *news) const;
	bool get_next_dir(char *nd = 0);
	void first_dir()
		{ next_dir = dir; }
	bool first_dir(char *nd)
		{ first_dir(); return get_next_dir(nd); }
	void get_dir_path(char *np) const;
};

