//  atoi.C                                                      (CRC: 80b9eb5)
//      relative path: ./lib/posix
//      creation date: 1992
//      last modified: Sat Oct  5 15:05:42 1996
//      header update: Sat Oct 26 15:09:29 1996
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       Libraries
//        Posix-complient code
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

//   convert an ascii string to a number


#include <types.h>
#include <posix/stddef.h>
#include <posix/stdlib.h>

#define DEF_NUMBASE 10
#define digit_value(ch)  (ch<='9'?ch-'0':10+(ch>='A'&&ch<='Z'?ch-'A':ch-'a'))


/* numbers are accepted in the following format:
 *   ddddd   default base
 *   0dddd   octal
 *   0xddd   hex
 *   0tddd   decimal
 *   0nddd   binary
 */

ulong strtoul(const char *str, char **adr, int base)
{
int x;
ulong num = 0;
	if(!base)
	base = DEF_NUMBASE;
	while(*str == ' ' || *str == '\f' || *str == '\n'
			|| *str == '\r' || *str == '\t' || *str == '\v')
		str++;
	if(*str == '0') {
		base = 8;
		str++;
		switch(*str) {
		case 'x':
			base = 16; str++; break;
		case 't':	
			base = 10; str++; break;
		case 'n':
			base = 2; str++; break;
		}
	}
	while((x = digit_value(*str)) >= 0 && (x < base)) {
		num = (num*base)+x;
		str++;
	}
	if(adr)
		*adr = (char*)str;
	return(num);
}


long strtol(const char *str, char **adr, int base)
{
	if(*str == '-')
		return(-(long)strtoul(&str[1], adr, base));
	else
		return((long)strtoul(str, adr, base));
}


int atoi(char *str)
{
	return((long)strtol(str,0,10));
}

long atol(char *str)
{
	return((long)strtol(str,0,10));
}

ulong atoul(char *str)
{
	return((ulong)strtoul(str,0,10));
}


uchar bcd_decode(uchar byte)
{
	return(((byte>>4)&0xf)*10 + (byte&0xf));
}

uchar bcd_encode(uchar byte)
{
	return(((byte/10)<<4) | (byte%10));
}

