/*  signal.h                                                    (CRC:ab321533)
 *      relative path: ./lib/posix
 *      creation date: 1992
 *      last modified: Sat Oct 26 15:09:24 1996
 *      header update: Sat Oct 26 15:09:25 1996
 *  
 *      Shag/OS: an Object-Oriented MicroKernel Operating System
 *       Libraries
 *        Posix-complient code
 *  
 *  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution.
 *  
 *  Shag/OS is free software for non-profit uses; it may be freely distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'COPYING'.  If this license was not received,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For bug reports, please read the file 'BUGS' first, then include the 
 *      product name and version, and the word 'bug' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

#include <posix/posix.h>
#include <posix/sys/types.h>


typedef unsigned int sigset_t;
typedef int	sig_atomic_t;
typedef void (*__sigfunc_t)(int);

#define SIG_DFL	((__sigfunc_t)0)
#define SIG_IGN	((__sigfunc_t)1)
#define SIG_ERR	((__sigfunc_t)-1)

#define SA_NOCLDSTOP	1

#define SIG_BLOCK          0
#define SIG_UNBLOCK        1
#define SIG_SETMASK        2

#define SIGHUP		 1
#define SIGINT		 2
#define SIGQUIT		 3
#define SIGILL		 4
#define SIGABRT		 6
#define SIGFPE		 8
#define SIGKILL		 9
#define SIGUSR1		10
#define SIGSEGV		11
#define SIGUSR2		12
#define SIGPIPE		13
#define SIGALRM		14
#define SIGTERM		15
#define SIGCHLD		17
#define SIGCONT		18
#define SIGSTOP		19
#define SIGTSTP		20
#define SIGTTIN		21
#define SIGTTOU		22



struct sigaction {
	__sigfunc_t sa_handler;
	sigset_t sa_mask;
	int sa_flags;
};

extern_C __sigfunc_t signal(int sig, __sigfunc_t func);
extern_C int raise(int sig);
extern_C int kill(pid_t pid, int sig);
extern_C int sigaction(int sig, const struct sigaction *act,
			struct sigaction *oact);
static __inline__ int sigaddset(sigset_t *set, int signo) {
	*set |= (1<<signo); return 0; }
static __inline__ int sigdelset(sigset_t *set, int signo) {
	*set &= ~(1<<signo); return 0; }
static __inline__ int sigemptyset(sigset_t *set) {
	*set = 0; return 0; }
static __inline__ int sigfillset(sigset_t *set) {
	*set = 0xffffffff; return 0; }
static __inline__ int sigismember(const sigset_t *set, int signo) {
	return((*set & (1<<signo)) > 0); }
extern_C int sigpending(sigset_t *set);
extern_C int sigprocmask(int how, const sigset_t *set, sigset_t *oset);
extern_C int sigsuspend(const sigset_t *sigmask);

