//  string.C                                                    (CRC:5e2685a4)
//      relative path: ./lib/posix
//      creation date: Dec 1994
//      last modified: Sat Oct 26 15:09:33 1996
//      header update: Sat Oct 26 15:09:33 1996
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       Libraries
//        Posix-complient code
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

// string manipulation functions


#include <types.h>
#include <posix/string.h>


#ifndef FUNC_strchr
#define FUNC_strchr
char *strchr(register const char *s, int c)
{
	while(*s && (*s != c))
		s++;
 	if(*s == c)
		return((char*)s);
	return(NULL);
}
#endif


#ifndef FUNC_strrchr
#define FUNC_strrchr
char *strrchr(register const char *s, int c)
{
char *cp;
	cp = NULL;
	while(*s) {
		if(*s == c)
			cp = (char*)s;
		s++;
	}
	return(cp);
}
#endif


#ifndef FUNC_strlen
#define FUNC_strlen
size_t strlen(register const char *s)
{
register int n = 0;
   while(*s++)
      n++;
   return(n);
}
#endif


#ifndef FUNC_strcpy
#define FUNC_strcpy
char *strcpy(char *s1, register const char *s2)
{
register char *sp1 = s1;
   while(*s2)
      *sp1++ = *s2++;
   *sp1 = '\0';
   return(s1);
}
#endif

#ifndef FUNC_strncpy
#define FUNC_strncpy
char *strncpy(char *s1, register const char *s2, size_t n)
{
register char *sp1 = s1;
   while(*s2 && n--)
      *sp1++ = *s2++;
   if (n)
      *sp1 = '\0';
   return(s1);
}
#endif

#ifndef FUNC_strcmp
#define FUNC_strcmp
int strcmp(register const char *s1, register const char *s2)
{
register int dif;
  while(*s1)
     if ((dif = (*s1++ - *s2++)))
         return(dif);
  return(*s1 - *s2);
}
#endif

#ifndef FUNC_strncmp
#define FUNC_strncmp
int strncmp(register const char *s1, register const char *s2, size_t n)
{
register int dif;
  while(*s1 && n-- > 1)
     if ((dif = (*s1++ - *s2++)))
         return(dif);
  return(*s1 - *s2);
}
#endif


#ifndef FUNC_strcat
#define FUNC_strcat
char *strcat(char *s1, register const char *s2)
{
register char *sp1 = s1;
  while(*sp1)
     sp1++;
  while(*s2)
     *sp1++ = *s2++;
  *sp1 = '\0';
  return(s1);
}
#endif


#ifndef FUNC_strncat
#define FUNC_strncat
char *strncat(char *s1, register const char *s2, size_t n)
{
register char *sp1 = s1;
  while(*sp1)
     sp1++;
  while(*s2 && n--)
     *sp1++ = *s2++;
  *sp1 = '\0';
  return(s1);
}
#endif


#ifndef FUNC_memcpy
#define FUNC_memcpy
void *memcpy(void *s1, const void *s2, size_t len)
{
register ulong *src, *dest;
register uchar *b1, *b2;
	b1 = (uchar*)s1;
	b2 = (uchar*)s2;
	while(len && ((vaddr)b1&3)) {
		*b1++ = *b2++;
		len--;
	}
	src = (ulong*)b2;
	dest = (ulong*)b1;
	while(len>16) {
		*dest++ = *src++;
		*dest++ = *src++;
		*dest++ = *src++;
		*dest++ = *src++;
		len -= 16;
	}
	b1 = (uchar*)dest;
	b2 = (uchar*)src;
	while(len) {
		*b1++ = *b2++;
		len--;
	}
	return(s1);
}
#endif

#ifndef FUNC_memset
#define FUNC_memset
void *memset(void *s, uchar val, size_t len)
{
register ulong *dest;
register ulong lfill;
register uchar *adr;
	adr = (uchar*)s;
	lfill = (val)|(val<<8)|(val<<16)|(val<<24);
	while(len && ((vaddr)adr&3)) {
		*adr++ = val;
		len--;
	}
	dest = (ulong*)adr;
	while(len>16) {
		*dest++ = lfill;
		*dest++ = lfill;
		*dest++ = lfill;
		*dest++ = lfill;
		len -= 16;
	}
	adr = (uchar*)dest;
	while(len--)
		*adr++ = val;
	return(s);
}
#endif

#ifndef FUNC_memset_w
#define FUNC_memset_w
void *memset_w(void *s, ushort val, size_t len)
{
register ushort *dest;
	dest = (ushort*)s;
	while(len-- > 0)
		*dest++ = val;
	return(s);
}
#endif

#ifndef FUNC_memset_l
#define FUNC_memset_l
void *memset_l(void *s, ulong val, size_t len)
{
register ulong *dest;
	dest = (ulong*)s;
	while(len-- > 0)
		*dest++ = val;
	return(s);
}
#endif
	

