/*  a_out.h                                                     (CRC:aab48b84)
 *      relative path: ./lib/sys
 *      creation date: 1992
 *      last modified: Sat Oct 26 15:09:35 1996
 *      header update: Sat Oct 26 15:09:36 1996
 *  
 *      Shag/OS: an Object-Oriented MicroKernel Operating System
 *       Libraries
 *        System-specific
 *  
 *  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution.
 *  
 *  Shag/OS is free software for non-profit uses; it may be freely distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'COPYING'.  If this license was not received,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For bug reports, please read the file 'BUGS' first, then include the 
 *      product name and version, and the word 'bug' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

/*	file format of a.out-style executable 
 */

/* XXX -- this file should eventually go away, to be replaced
 *	by the header file from an A_out class (just like class Elf32),
 *	except that, unless it is desperately needed, such a class
 *	might not get written for a while, since it will be hard
 *	to provide a lot of the necessary dynamic linking support,
 *	which Elf32 provides, and a.out executables are being phased out...
 *	Thus, this might all just go away completely?
 *	Farewell, a.out-- you served us well but your time is near an end...
 */
 

#define OMAGIC	0407	/* object file or impure executable */
#define NMAGIC	0410	/* pure executable */
#define ZMAGIC	0413	/* demand-paged executable */
#define QMAGIC	0314	/* demand-paged executable with header in text */
#define CMAGIC	0421	/* core file */
#define LMAGIC	0430	/* library file */
#define BSDMAGIC 01401	/* BSD executable */


#ifndef ASM

#include <types.h>

struct exec {
  ushort a_magic;	/* magic number */
  ushort a_machtype;	/* machine type */
  ulong a_text;		/* length of text, in bytes */
  ulong a_data;		/* length of data, in bytes */
  ulong a_bss;		/* length of uninitialized data area for file, in bytes */
  ulong a_syms;		/* length of symbol table data in file, in bytes */
  ulong a_entry;	/* start address */
  ulong a_trsize;	/* length of relocation info for text, in bytes */
  ulong a_drsize;	/* length of relocation info for data, in bytes */
};

struct nlist {
	ulong	n_strx;
	uchar	n_type;
	char 	n_other;
	ushort 	n_desc;
	ulong 	n_value;
};

#define N_UNDF	0
#define N_ABS	2
#define N_TEXT	4
#define N_DATA	6
#define N_BSS	8
#define N_COMM	0x12	/* common - ld internal */
#define N_LABEL	0x14	/* label - ld internal */
#define N_FN	0x1f	/* file name symbol */

#define N_EXT	1
#define N_TYPE	0x1e	/* type mask */

#define N_SHARE	0x20	/* global sym -- register in /lib */

#define N_STAB	0xe0	/* if any bits set, an SDB entry */
#define	N_INDR	0xa	/* indirect reference immediately follows */
#define N_SETA	0x14
#define N_SETT	0x16
#define N_SETD	0x18
#define N_SETB	0x1a
#define N_SETV	0x1c

struct relocation_info {
	ulong	r_address;
	unsigned r_symbolnum:24;
	unsigned r_pcrel:1;
	unsigned r_length:2;
	unsigned r_extern:1;
	unsigned r_pad:4;
};

#else

#define EXEC_MAGIC	0	/* uint16 */
#define EXEC_MACHINE	2	/* uint8  */
#define EXEC_FLAGS	3	/* uint8  */
#define EXEC_TEXT	4	/* uint32 */
#define EXEC_DATA	8	/* uint32 */
#define EXEC_BSS	12	/* uint32 */
#define EXEC_SYMS	16	/* uint32 */
#define EXEC_ENTRY	20	/* uint32 */
#define EXEC_TRSIZE	24	/* uint32 */
#define EXEC_DRSIZE	28	/* uint32 */


#define M_8086		4
#define M_386		100


#endif 


