//  ArgParse.C                                                  (CRC:74bf91e7)
//      relative path: ./sys/boot
//      creation date: Sun Oct 29 11:03:18 1995
//      last modified: Sat Oct 26 15:11:38 1996
//      header update: Sat Oct 26 15:11:38 1996
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       System files
//        Boot loader
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

// ArgParse

#pragma Make Includes_Banned += -nostdinc -nostdinc++

#include <string.h>

#include <stddef.h>
#include <strclass.h>
#include <iostream.h>

#include "ArgParse.h"

bool ArgParse::match(char *s, bool *set)
{
	if(_num < _argc) {
		if(!strcmp(s, _argv[_num])) {
			_num++;
			if(set)
				*set = true;
			return true;
		}
	} else
		missing();
	return false;
}


bool ArgParse::match(char *s, string *str)
{
	if(match(s)) 
		return get(str);
	return false;
}


bool ArgParse::prefix(char *s, bool *set)
{
	if(_num < _argc) {
		if(!strncmp(s, _argv[_num], strlen(s))) {
			_num++;
			if(set)
				*set = true;
			return true;
		}
	} else
		missing();
	return false;
}


bool ArgParse::prefix(char *s, string *str)
{
	if(prefix(s)) { 
		if(_argv[_num-1][strlen(s)-1]) {
			*str = &_argv[_num-1][strlen(s)-1];
		} else
			return get(str);
	}
	return false;
}


bool ArgParse::flag(char f, bool *)
{
	return false;
}


bool ArgParse::get(string *str)
{
	if(_num < _argc) {
		*str = _argv[_num++];
		return true;
	} else 
		missing();
	return false;
}


void ArgParse::badoption()
{
	cerr << "Bad option: " << _argv[_num-1] << endl;
	if(_quit)
		exit(1);
}

void ArgParse::badflag()
{
}


void ArgParse::missing()
{
	cerr << "Missing parameter after " << _argv[_num-1] << endl;
	if(_quit)
		exit(1);
}

