/*  $Id: boot.h,v 1.5 1996/10/26 19:11:07 shaggy Exp $
 *      Boot Info for SOLO/Shagboot
 *      
 *      relative path: ./sys/boot/m-pc
 *  
 *      Shag/OS: an Object-Oriented MicroKernel Operating System
 *       System files
 *        Boot loader
 *         (m-pc)
 *  
 *  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution.
 *  
 *  Shag/OS is free software for non-profit uses; it may be freely distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'COPYING'.  If this license was not received,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For bug reports, please read the file 'BUGS' first, then include the 
 *      product name and version, and the word 'bug' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

/*  $Log: boot.h,v $
 *  Revision 1.5  1996/10/26  19:11:07  shaggy
 *  header update
 *
 *  Revision 1.4  1996/10/04  16:41:39  shaggy
 *  changed partition 'end' to 'size'
 *
 *  Revision 1.3  1996/09/28  00:23:06  shaggy
 *  removed irrelevant and outdated code and info
 *
 *  Revision 1.2  1996/09/28  00:15:32  shaggy
 *  changed most mappings
 *  deleted "DELAY" definitions (inappropriate location)
 *
 *  Revision 1.1  1994/12/21  17:03:14  shaggy
 *  Initial revision
 *
 */


#include <types.h>
#include "chunk.h"



#define SECTOR_SIZE		0x200	// size of a disk sector 
#define SECTOR_2SIZE		9	// shift count for mul/div

#define PARTID_UNUSED	0x00
#define PARTID_DOS12	0x01
#define PARTID_EXTENDED	0x05

#define	BOOT_MAGIC		0xaa55
#define	BOOTMERGE_MAGIC		0xaa66


#ifdef ASM

#include "boot.ofs"

#if BootBlock_sizeof != 0x200
#error sizeof(BootBlock) != 0x200 (see boot.ofs)
#error Perhaps the compiler is failing to pack structures correctly?
#endif


// important addresses:
#define EXT_ADR		0x100000	// start of extended memory
#define INIT_ADR	0x8000		// start address for executables
#define BOOT_ADR	0x7c00		// where boot sector is located
#define BUF_ADR		0x6000		// BPMI call buffer area
#define BUF_SIZE	INIT_ADR-BUF_ADR	// 8K max (incl. bootsec)
#define MAP_ADR		0x5000		// where to load map file 
#define MAP_SIZE	BUF_ADR-MAP_ADR		// 4K max
#define MSG_ADR		0x4800		// area for kernel messages
#define MSG_SIZE	MAP_ADR-MSG_ADR	// 2K max
#define STACK_ADR	0x4700		// stack: 0x5f00 - 0x5fff
#define STACK_SIZE	MSG_ADR-STACK_ADR	// 256 max
#define LINE_ADR	0x4600		// input line buffer
#define	LINE_SIZE	STACK_ADR-LINE_ADR	// 256 max
#define FDT_ADR		0x4400 		// file descriptor table
#define FDT_SIZE	LINE_ADR-FDT_ADR	// 512 max (16 FD's)
#define TMP_ADR		0x4200		// temporary buffer area
#define TMP_SIZE	FDT_ADR-TMP_ADR		// 512 max (one sector)
#define PARAM_ADR	0x4100		// where to put BootParam block
#define PARAM_SIZE	TMP_ADR-PARAM_ADR	// 256 max
#define TSS_ADR		0x4080		// where to keep TSS info
#define TSS_SIZE	PARAM_ADR-TSS_ADR	// 128 max
#define RESV_ADR	0x4000		// reserved spare space
#define RESV_SIZE	TSS_ADR-RESV_ADR	// 128 max
#define LOAD_ADR	0x1000		// where to load SOLO 
#define LOAD_SIZE	RESV_ADR-LOAD_ADR	// 12K max
#define VAR_ADR		0x0c00		// where to store variables
#define	VAR_SIZE	LOAD_ADR-VAR_ADR	// 1K max
#define	PTAB_ADR	0x7be		// where to copy partition table

#define PROGRESS	0x7ffe		// used for keeping track of
					// progress in case of crash

#define STACK_PTR	STACK_ADR+STACK_SIZE

#define VARNAME_SIZE	9		// max len of var name + null
#define VARVAL_SIZE	23		// max len of var val + null
#define VARPAIR_SIZE	VARNAME_SIZE+VARVAL_SIZE
#define NVARS		VAR_SIZE/[VARPAIR_SIZE]

#define ARGV_ADR	PARAM_ADR+BootParam_size

#define BOOTCODE_ADR		0x7c40		// BOOT_ADR + BOOTBLOCK_CODE


// Note:
//	4K for the map file area means that there can
//	be up to slightly less than 1024 (depending on how
//	many file names are there) chunks.
//	each chunk can map anywhere from 1 to 64 sectors,
//	so the mapfile can map from 512K to 32Mb of startup files,
//	which should be sufficient for most cases.
//	To save mapfile space, (if you're running out)
//	make sure the disk being mapped isn't badly fragmented.
//	If necessary, the boundaries can be moved around, but
//	remember that any changes will cause incompatibilities
//	between ShagBoot and SOLO unless both are recompiled
//	and reinstalled.

//	Note that, in the worse case, with 512-byte blocks,
//	to load the full 4K of the mapfile will require,
//	at most, 8 chunks.  Therefore, only 8 chunks are reserved,
//	thus leaving plenty of room for future code enhancements
//	in the boot sector.


// Why does all this stuff have to fit in the lower 32K of memory?
// you may ask...
// Well, notice that it all stays resident while the kernel is running,
// to be available for BIOS calls, static file system access,
// and for debugging in case the kernel crashes.  Therefore, it
// is important to keep it as small as possible, and by keeping
// it in the lower 32K, the next 32K is available for loading
// the headers of executables, while keeping everything in the
// first 64K segment, so that no messing around with segment
// registers has to be done, thus keeping it even smaller and simpler.

#else

#include <packed>

// XXX - see the note at the bottom
//	all of this code had to be drastically rewritten
//	for GCC 2.7.2 since __attribute__((packed)) doesn't
//	seem to work anymore.


// packed_uint16 and packed_uint8 were created to overcome some
// of the problems, and bitfields had to be replaced with inline methods
// which means that code had to be modified to use the new inline methods
// instead of going directly to the bitfields.
// I suppose that's even more reason to do everything in the future
// through inline methods rather than accessing structures directly
// from higher level code.


volatile struct DiskParam {
	packed<uint16>	cylinders;
	uint8	heads;
	packed<uint16>	startcyl;
	packed<uint16>	writeprecomp;
	uint8 	maxecc;
	uint8 	control;
	uint8 	timeout;
	uint8	ftimeout;
	uint8 	ctimeout;
	packed<uint16> 	landing;
	uint8 	sectors;
	uint8 	unused;
};

volatile struct Partition {
	uint8 	boot;
	uint8	starthead;	// top 2 bits are bits 10-11 of startcylinder
	uint8	startsector;	// top 2 bits are bits 8-9 of startcylinder
	uint8	startcylinder;
	uint8 	id;
	uint8	endhead;	// top 2 bits are bits 10-11 of endcylinder
	uint8	endsector;	// top 2 bits are bits 8-9 of endcylinder
	uint8	endcylinder;
	packed<uint32>	start;	// linear sector address
	packed<uint32>	size;	// linear sector size

// XXX - well, I started to implement access methods for all of 
//	 what used to be bitfields, and then, upon searching for
//	 where they were used in the C++ code, discovered 
//	 that they aren't!
//	 So, when/if they do get used, the rest of these
//	 will have to be implemented, but for now, I have
//	 more important places to focus my time and effort...

//	Actually, this stuff might turn out not to be necessary,
//	since it seems that you can declare bitfields with
//	smaller alignments with (i.e.)  uchar bit:3;
//	(unfortunately enum types always align to int,
//	except with -fshort-enum)
//	However, access methods to take care of
//	the "scattered bits for backwards compatibility" problem
//	might still be a good idea, if this is ever needed.

/*
	uint start_sector() { return get_bits(startsector, 0, 6); }
	void start_sector(uint n) { set_bits(startsector, 0, 6, n); }
	uint start_cyl() { return startcylinder
				| (get_bits(startsector, 6, 2) << 6)
				| (get_bits(starthead, 6, 2) << 8); }
*/
};


volatile struct BootBlock {
	uint8		jmp[3];		// skip over header stuff
	char		sysname[8];	// system name for DOS
	uint8		bpb[0x2c];	// Bios Parameter Block
	uint8		resv[5];	// 16-byte alignment padding
	packed<uint32>	bootsec;	// dev/head/cyl/sec for boot sector
	uint8		code[0x14a]; 	// up to 330 bytes of code
	packed<uint16>	bootoffset;	// offset of boot program in map
	Chunk		chunk[8];	// up to 8 chunks 
	uint8		mapsize;	// # of map chunks
	uint8 		mapdev;		// device ID for map
	DiskParam	diskparam __attribute__ ((packed)) ;
	Partition	partition[4] __attribute__ ((packed)) ;
	packed<uint16>	magic;
};


#endif

