/*  chunk.h                                                     (CRC:9578cddc)
 *      definition of chunks in the mapfile
 *      
 *      relative path: ./sys/boot/m-pc
 *      creation date: December 1994
 *      last modified: Sat Oct 26 15:11:05 1996
 *      header update: Sat Oct 26 15:11:06 1996
 *  
 *      Shag/OS: an Object-Oriented MicroKernel Operating System
 *       System files
 *        Boot loader
 *         (m-pc)
 *  
 *  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution.
 *  
 *  Shag/OS is free software for non-profit uses; it may be freely distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'COPYING'.  If this license was not received,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For bug reports, please read the file 'BUGS' first, then include the 
 *      product name and version, and the word 'bug' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

#include <types.h>

#ifdef ASM

#define Chunk_sector		0	// except two high bits
#define Chunk_cyl		1	// low eight bits 
#define Chunk_head		2	// except two high bits
#define Chunk_count		3	// 7:partial, 6:small, 5-0: count
#define Chunk_sizeof		4	// size of 'Chunk'

#else

volatile struct Chunk {
	uint	sector: 6;	// CL:0-5  sector number
	uint	cyl2: 2;	// CL:6-7  cylinder (bits 8-9)
	uint	cyl1: 8;	// CH:0-7  cylinder (bits 0-7)
	uint	head: 6;	// DH:0-5  head (bits 0-5)
	uint	cyl3: 2;	// DH:6-7  cylinder (bits 10-11)
	uint	count: 6;	// AL:0-5  sector count
	uint	small: 1;	// AL:6  for small sectors
	uint	partial: 1;	// AL:7  for partial sectors
};

volatile struct ChunkLinear {
	uint	sector: 24;	// linear sector number 
	uint	count: 6;	// sector count
	uint	small: 1;	// for small sectors
	uint	partial: 1;	// for partial sectors
};

// if chunk.partial:
//	if chunk.small:
//		bits 0-4: start, in multiple of 16
//	else
//		bits 0-3: start, in multiple of 32
//		bits 4-5: select size: 32,64,128,256
// else
//	bits 0-5: number of consecutive sectors

// note:
//	'partial' chunks are for filesystems that can store multiple 
//	logical blocks in a sector (to save space for many small files)
//	only block sizes of 16(small),32,64,128 or 256 are supported
//	for partial sectors
//	They are currently not supported by the boot code, as
//	they are not yet needed, but such a mode may
//	be available in the shagfs file system, to save major
//	amounts of space in filesystems that consist of many small objects.

#endif

