/*  device.h                                                    (CRC:7df6842c)
 *      relative path: ./sys/boot/m-pc
 *      creation date: Sun Jul 28 11:22:44 1996
 *      last modified: Sat Oct 26 15:11:18 1996
 *      header update: Sat Oct 26 15:11:19 1996
 *  
 *      Shag/OS: an Object-Oriented MicroKernel Operating System
 *       System files
 *        Boot loader
 *         (m-pc)
 *  
 *  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution.
 *  
 *  Shag/OS is free software for non-profit uses; it may be freely distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'COPYING'.  If this license was not received,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For bug reports, please read the file 'BUGS' first, then include the 
 *      product name and version, and the word 'bug' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

#define DEV_NULL	0x00	 

#define DEV_MEM		0x01	// physical memory (real mode)
#define DEV_VMEM	0x02	// virtual memory (protected mode)

#define DEV_PORT	0x03	// I/O ports
// for unit number, 0=read separate ports,
//		   x=keep re-reading sequence of 'x' ports 
// for flags, port size = 2^flag
//	(i.e. 0=byte ports, 1=word ports, 2=long ports, etc)
// the effective blocksize = (2^flag)*unit   if unit>0
//			or   (2^flag)	     otherwise

#define DEV_CONSOLE	0x04	// console
// for flag bits:
#define  CONSOLE_CANON	0x01	// canonical (line editing)
#define  CONSOLE_ECHO	0x02	// echo input

#define DEV_DISK	0x05	// standard disk (floppy, hd)
// for unit:
#define	 DISK_FD0	0x00	// floppy disk 0 (A:)
#define	 DISK_FD1	0x01	// floppy disk 1 (B:)
#define  DISK_HD0	0x80	// hard drive 0 (primary/master)
#define  DISK_HD1	0x81	// hard drive 1 (secondary/slave)
// and for the flag bits,
// the high bit is set to 1 for linear disk I/O (block#)
// or 0 for nonlinear disk I/O (cylinder/sector/head),
#define  DISK_LINEAR 0x80
// and bits 6-0 select the partition number, where 0 is the whole disk,
// 1 through 4 are primary partitions, and 5 through 127
// are extended (logical) partitions...

#define DEV_MSG		0x06	// logging of kernel messages

// #define DEV_SERIAL	0x07	// serial ports
// #define DEV_PARALLEL	0x08	// parallel ports
// #define DEV_KEYBOARD	0x09	// keyboard
// #define DEV_VIDEO	0x0a	// video display
// #define DEV_SCSI	0x0b	// SCSI disks

#define NDEVICES	0x07	// number of devices

