//  $Id: fdisk.C,v 1.9 1996/10/26 19:11:20 shaggy Exp $
//      
//      Fixed Disk Partitioning Utility
//      
//      relative path: ./sys/boot/m-pc
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       System files
//        Boot loader
//         (m-pc)
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

  
//  $Log: fdisk.C,v $
//  Revision 1.9  1996/10/26  19:11:20  shaggy
//  header update
//
//  Revision 1.8  1996/10/10  02:00:53  shaggy
//  fixed overflow for printing MB sizes
//
//  Revision 1.7  1996/10/06  20:18:38  shaggy
//  fdisk 0.2.0 (October 06, 1996: 16:18:35)
//  	- prints extended partition information as it follows the chain
//
//  --------------------------------------------------------------------------
//
//  Revision 1.6  1996/10/06  20:07:46  shaggy
//  fdisk 0.1.3 (October 06, 1996: 16:07:44)
//  	- fixed warnings to take 1024 cylinder problem into account
//
//  Revision 1.5  1996/10/05  11:52:52  shaggy
//  fdisk 0.1.2 (October 05, 1996: 07:52:50)
//  	- fixed RCS version info
//
//  Revision 1.4  1996/10/05  11:32:23  shaggy
//  fdisk 0.1.1 (October 05, 1996: 07:32:20)
//  	fixed version info
//
//  Revision 1.3  1996/10/05  11:06:38  shaggy
//  better disk info
//
//  Revision 1.2  1996/09/28  02:34:36  shaggy
//  added Port code for drive info
//
//  Revision 1.1  1996/07/14  01:23:45  shaggy
//  Initial revision
//


#define fdisk_VER_MAJOR	0
#define fdisk_VER_MINOR	2
#define fdisk_VER_PATCH	0
#define fdisk_VER_STR	"0.2"
#define fdisk_VER_DATE	"October 06, 1996"
#define fdisk_VER_NDATE	"10/06/96"


#include <defines>
#include <posix/string.h>
#include <posix/stdlib.h>
#include <boot/libsolo.h>
#include <boot/fileinfo.h>
#include <boot/boot.h>
#include <hw/Port.h>


static char rcsid[] __attribute__((unused))
	= "$Id: fdisk.C,v 1.9 1996/10/26 19:11:20 shaggy Exp $"
	  "$Release: " fdisk_VER_STR " $";

//#define USE_DEV_PORT


struct PartitionType {
	uint8	id;
	const char *name;
};


// Hard-coded for now.... later to be read from a configurable file...

PartitionType ptype[] = {
	{0x00, "Empty"},
	{0x01, "DOS 12-bit FAT"},
	{0x02, "XENIX root"},
	{0x03, "XENIX usr"},
	{0x04, "DOS 16-bit <32M"},
	{0x05, "Extended"},
	{0x06, "DOS 16-bit >=32M"},
	{0x07, "OS/2 HPFS"},
	{0x08, "AIX"},
	{0x09, "AIX bootable"},
	{0x0a, "OS/2 Boot Manager"},
	{0x40, "Venix 80286"},
	{0x51, "Novell?"},
	{0x52, "Microport"},
	{0x63, "GNU HURD"},
	{0x64, "Novell"},
	{0x75, "PC/IX"},
	{0x80, "Old MINIX"},
	{0x81, "Linux/MINIX"},
	{0x82, "Linux swap"},
	{0x83, "Linux native"},
	{0x93, "Amoeba"},
	{0x94, "Amoeba BBT"},
	{0xa5, "BSD/386"},
	{0xae, "Shag/OS"},
	{0xaf, "Shag/OS swap"},
	{0xb7, "BSDI fs"},
	{0xb8, "BSDI swap"},
	{0xc7, "Syrinx"},
	{0xdb, "CP/M"},
	{0xe1, "DOS access"},
	{0xe3, "DOS R/O"},
	{0xf2, "DOS secondary"},
	{0xff, "BBT"}
};



// Partition table notes:
// Physical sector/head/cylinder are always absolute
// Logical start is absolute for primary partitions
// For the extended partition, the 'bootsector' contains another
// partition table.
//	The first entry is relative to the main
// 	extended partition, and is the first logical partition
// 	The second entry is another extended entry (these act
//	like a linked list)
//	from here on, each extended entry has its logical address
//	relative to the main extended partition,
//	and each individual partition entry has its logical address
//	relative to the extended partition in which it is found 
 

int cylinders, heads, sectors;



int getint(int default_val)
{
char buf[40];
	if(!gets(buf, sizeof(buf)) || buf[0] < '0' || buf[0] > '9')
		return default_val;
	return atoi(buf);
}


const char* ptype_name(uint8 id)
{
	for(size_t i=0; i<arraysize(ptype); i++)
		if(ptype[i].id == id)
			return ptype[i].name;
	return "unknown";
}




void chs_unpack(uint &cyl, uint &head, uint &sec)
{
// XXX - following the format used elsewhere, the top two bits
// of the head should be used as bits 10 and 11 of the cylinder
// However, from partition tables I've seen created by Linux fdisk,
// it seems that the head uses all 8 bits, and the cylinder gets only
// 10 total, meaning that cylinders > 1024 are not stored
// correctly.  For now, I'm using 12 bit cylinders, in the hopes
// that the real "standard" is such... but I've got the feeling that
// I'll be changing this to 10 bits soon...
#ifdef USE_10_BIT_CYLINDER
	uint newcyl = cyl+((sec&0xc0)<<2);
#else
	uint newcyl = cyl+((sec&0xc0)<<2)+((head&0xc0)<<4);
	head &= 0x3f;
#endif
	sec &= 0x3f;
	cyl = newcyl;
}


int chs_to_linear(uint cyl, uint head, uint sec)
{
	return ((cyl*heads)+head)*sectors+(sec-1);
}

int pchs_to_linear(uint cyl, uint head, uint sec)
{
	chs_unpack(cyl, head, sec);
	return chs_to_linear(cyl, head, sec);
}



int partition_start(Partition *part, int ofs = 0)
{
	int start = part->start + ofs;
	int start2 = pchs_to_linear(part->startcylinder,
			part->starthead, part->startsector);
	if(start != start2) {
		int count1024 = (start - start2)/(1024*heads*sectors);
		if(count1024 > 0) {
			start2 = pchs_to_linear(
				part->startcylinder+count1024*1024,
				part->starthead, part->startsector);
			printf("Warning: start cylinder invalid (cyl >= 1024)\n");
		}	
		if(start != start2)
			printf("Warning: inconsistent start = %d+%d = %d, chs -> %d\n",
				(int)part->start, ofs, start, start2);
	}
	return start;
}


int partition_size(Partition *part, int ofs = 0)
{
	int end = part->start+ofs+part->size-1;
	int end2 = pchs_to_linear(part->endcylinder,
			part->endhead, part->endsector);
 
	if(end != end2) {
		int count1024 = (end-end2)/(1024*heads*sectors);
		if(count1024 > 0) {
			end2 = pchs_to_linear(part->endcylinder+count1024*1024,
				part->endhead, part->endsector);
			printf("Warning: end cylinder invalid (cyl >= 1024)\n");
		}
		if(end != end2) 
			printf("Warning: inconsistent end = %d, chs -> %d\n",
				end, end2);
	}
	return part->size;
}



bool read_sector(fd_t devfd, void *adr, uint32 secnum)
{
	lseek(devfd, secnum*SECTOR_SIZE, SEEK_SET);
	return read(devfd, adr, SECTOR_SIZE) == SECTOR_SIZE;
}


bool read_sector(fd_t devfd, void *adr, int cyl, int head, int sec)
{
	return read_sector(devfd, adr, pchs_to_linear(cyl, head, sec));
}



/*  Table format:
Part AF   Cyl Hd Sec   Cyl Hd Sec    Start     Size     MB ID Type
 1/a 80 00000 00 000 00000 00 000 _______1 _______4 ___3.0 01 DOS
 2/b -- 00000 00 000 00000 00 000 _____234 ____4545 _455.2 83 Linux
 3/c -- 00000 00 000 00000 00 000 _____234 ____4545 _455.3 0a OS/2 Boot Manager
*/
 
void print_partition_entry(int index, Partition *part, uint offset = 0)
{
uint startcylinder = part->startcylinder;
uint starthead = part->starthead;
uint startsector = part->startsector;
uint endcylinder = part->endcylinder;
uint endhead = part->endhead;
uint endsector = part->endsector;
uint size_tenthMB;

	chs_unpack(startcylinder, starthead, startsector);
	chs_unpack(endcylinder, endhead, endsector);
	size_tenthMB = ((uint)part->size*10)/(_M_/SECTOR_SIZE);

	if(index)
		printf("%2d/%c ", index, index-1+'a');
	else
		printf("     ");
	if(part->boot)
		printf("%02x", part->boot);
	else
		printf("--");
	printf(" %05d %02d %03d %05d %02d %03d"
		"%9u%9u%5u.%1u %02x %s\n", 
		startcylinder, starthead, startsector,
		endcylinder, endhead, endsector,
		(uint)part->start+offset, (uint)part->size,
		size_tenthMB/10, size_tenthMB%10,
		part->id, ptype_name(part->id));
}

 
void print_partition_table(fd_t devfd)
{
BootBlock sec;
int ext, extofs;
int lognum;
int start;
int size;
	printf("Part AF   Cyl Hd Sec   Cyl Hd Sec    "
			"Start     Size     MB ID Type\n");
	if(!read_sector(devfd, &sec, 0)) {
		printf("error: could not read boot sector\n");
		return;
	}
	ext = 0;
	for(int i=0; i<4; i++) 
		if(sec.partition[i].id) {
			print_partition_entry(i+1, &sec.partition[i]);
			start = partition_start(&sec.partition[i]);
			size = partition_size(&sec.partition[i]);
			if(sec.partition[i].id == PARTID_EXTENDED)
				ext = start;
		}
	lognum = 4;
	extofs = ext;
	while(ext) {
		if(!read_sector(devfd, &sec, ext)) {
			printf("error: could not read sector %d\n", ext);
			return;
		}
		print_partition_entry(++lognum, &sec.partition[0], ext);
		start = partition_start(&sec.partition[0], ext);
		size = partition_size(&sec.partition[0], ext);
		ext = 0;
		if(sec.partition[1].id == PARTID_EXTENDED) {
			print_partition_entry(0, &sec.partition[1], extofs);
			ext = partition_start(&sec.partition[1], extofs);
		}
		// for debugging...
		// print_partition_entry(lognum, &sec.partition[1]);
	}
}


char * ide_info_strncpy(char *dest, const char *sp, int n)
{
	char *dp = dest;
	const char *ep = sp + n - 1;
	int bswap = 1;

	int space = 0;
	while(sp[bswap] && sp < ep) {
		char ch = (sp++)[bswap];
		bswap = -bswap;
		if(ch != ' ') {
			*dp++ = ch;
			space = 1;
		} else if(sp < ep && sp[bswap] != ' ') {
			*dp = ' ';
			dp += space;
		}
	}
	*dp = 0;
	return dest;
}


void get_ide_drive_info(int drive)
{
ushort id[256];
char buf[50];

	printf("Controller %d, Drive %d:\n", (drive&2)>>1, drive&1);

#ifdef USE_DEV_PORT  // outdated code...
fd_t port = open("/dev/port", O_RDWR);
fd_t rportw = open("/dev/rportw", O_RDONLY);
uchar cmd[2];
	if(port < 0) {
		printf("could not open /dev/port\n");
		return;
	}
	if(rportw < 0) {
		printf("could not open /dev/rportw\n");
		return;
	}
	cmd[0] = 0xa0+((drive&1) << 4);	// select drive 'drive'
	cmd[1] = 0xec;			// Indentify drive
	lseek(port, 0x1f6, SEEK_SET);
	write(port, cmd, 2);
	do {
		lseek(port, 0x1f7, SEEK_SET);
		read(port, cmd, 1);
	} while((cmd[0] & 0xc8) != 0x48);
	lseek(rportw, 0x1f0, SEEK_SET);
	read(rportw, id, sizeof(id));
	close(port);
	close(rportw);
#else
	int disk_iobase = (drive&2) ? 0x170 : 0x1f0;

	Port16 hd_data(disk_iobase+0);
	Port8 hd_select(disk_iobase+6);
	Port8 hd_cmd(disk_iobase+7);
	Port8 hd_status(disk_iobase+7);

	hd_select = 0xa0+((drive&1) << 4); // select drive 
	if((hd_status & 0xc0) != 0x40) {
		printf("Drive not responding\n");
		return;
	}

	hd_cmd = 0xec;		 // Get Drive Information
	while((hd_status & 0xc0) != 0x40)
		continue;

	if(hd_status & 1) {
		printf("GetDriveInformation not supported by this drive\n");
		return;
	}

	for(size_t i=0; i<arraysize(id); i++)
		id[i] = hd_data;
#endif


	printf("Model: %s, ", ide_info_strncpy(buf, (char*)&id[27], 41));
	printf("Revision: %s, ", ide_info_strncpy(buf, (char*)&id[23], 9));
	printf("SN: %s\n", ide_info_strncpy(buf, (char*)&id[10], 21));

	printf( "Buffer size: %dK, "
		"Configuration: %04x\n", id[21]/2, id[0]);

	printf("  ");
	if(id[0] & 0x40) printf("fixed drive, ");
	if(id[0] & 0x80) printf("removable media, ");
	if(id[0] & 0x02) printf("hard-sectored, ");
	if(id[0] & 0x04) printf("soft-sectored, ");
	if(id[0] & 0x08) printf("RLL/ARLL format\n");
		else printf("MFM encoded\n");
	if(id[0] & 0x10) printf("  head switch < 15ms, ");
		else printf("  head switch >= 15ms, ");
	if(id[0] & 0x20) printf("  power down supported\n");
	if(id[0] & 0x100) printf("  data rate < 5Mb/s\n");
	if(id[0] & 0x200) printf("  data rate between 5Mb/s and 10Mb/s\n");
	if(id[0] & 0x400) printf("  data rate > 10Mb/s\n");
	if(id[0] & 0x800) printf("  rotation deviation > 0.5%%\n");
	printf("  buffer type %04x: ", id[20]);
	switch(id[20]) {
	case 1: printf("single-sector\n"); break;
	case 2: printf("multi-sector, dual-ported\n"); break;
	case 3: printf("multi-sector, dual-ported, w/ read cache\n"); break;
	default: printf("unknown\n"); break;
	}
	if(id[48] & 1) printf("  32-bit I/O, ");
	else printf("  16-bit I/O, "); 

	printf("maxmult=%d, ", id[47]&0xff);
	printf("%s DMA, %s LBA, %s logical, %s EIDE\n",
		(id[49]&1) ? "" : "no",
		(id[49]&2) ? "" : "no",
		(id[53]&1) ? "" : "no",
		(id[53]&2) ? "" : "no");
	printf("  PIO cycle time: ");
	switch(id[51]>>8) {
	case 0: printf("600 ns"); break;
	case 1: printf("380 ns"); break;
	case 2: printf("240 ns"); break;
	case 3: printf("180 ns"); break;
	default: printf("unknown"); break;
	}
	printf(", DMA cycle time: ");
	switch(id[51]>>8) {
	case 0: printf("960 ns"); break;
	case 1: printf("380 ns"); break;
	case 2: printf("240 ns"); break;
	case 3: printf("150 ns"); break;
	default: printf("unknown"); break;
	}
	printf("\n");

	int secsize = id[5];
	if(secsize > 512 && secsize < 1024)
		secsize = 512;
	if(secsize > 1024 && secsize < 2048)
		secsize = 1024;
	printf("  unformatted bytes/sector: %d  bytes/track: %d\n",
		id[5], id[4]);
	printf("Physical geometry: c:%d h:%d s:%d ss:%d t:%dMb\n",	
		id[1], id[3], id[6], secsize, 
		(id[1]*id[3]*id[6])/(_M_/secsize));
	printf("Logical geometry: c:%d h:%d s:%d ts:%d t:%dMb\n",
		id[54], id[55], id[56], (id[58]<<16)|id[57],
		((id[58]<<16)|id[57])/(_M_/secsize));
	printf("LBA: ts:%d t:%d\n",
		(id[60]<<16)|id[61],
		((id[60]<<16)|id[61])/(_M_/secsize));
}


main(int argc, char **argv)
{
const char *devname = "/dev/hd0";
fd_t devfd;
FileInfo devinfo;
	/* This note isn't needed for now, since we can't write anyways...
	printf("NOTE: make sure you know what you're doing before using\n"
	       "this program.  This utility could DESTROY YOUR ENTIRE HARD DISK\n"
	       "if used improperly, or if there is a bug in it, so make sure\n"
	       "all your important data is backed up before proceeding!\n"
	       "Enter 'q' to simply quit if you're not sure.\n"
	       "\n");
	printf("Note2: This program is currently incomplete, and is missing\n"
	       "most of its necessary functionality.  Use a standard FDISK\n"
	       "program instead for now.\n"
	       "\n");
	*/
	for(int i=1; i<argc; i++) {
		if(!strcmp(argv[i], "-v")) {
			printf("fdisk " fdisk_VER_STR "\n");
			exit(0);
		} else
			devname = argv[i];
	}
	if((devfd = open(devname, O_RDWR)) < 0) {
		printf("could not open %s\n", devname);
		exit(1);
	}
	solo_fstat(devfd, &devinfo);
	cylinders = devinfo.cylinders;
	heads = devinfo.heads;
	sectors = devinfo.sectors;
	while(true) {
		printf("Command (m for help): ");
		char buf[80];
		while(!gets(buf, sizeof(buf))) continue;
		switch(buf[0]) {
		case 'I':
			get_ide_drive_info(RDEV_UNIT(devinfo.rdev) & 0xf);
			break;
		case 'p':
			printf("Disk %s: %d cylinders, %d heads, %d sectors\n\n",
				devname, cylinders, heads, sectors);
			print_partition_table(devfd);
			break;
		case 'c':
			printf("Number of cylinders (1-65535, currently %d): ",
					cylinders);
			cylinders = getint(cylinders);
			break;
		case 'h':
			printf("Number of heads (1-63, currently %d): ",
					heads);
			heads = getint(heads);
			break;
		case 's':
			printf("Number of sectors (1-255, currently %d): ",
					sectors);
			sectors = getint(sectors);
			break;
	        case 'w':
			printf("saving changes\n");
		case 'q':
			printf("exiting...\n");
			close(devfd);
			exit(0);
		default:
			printf("Command action\n"
			       "   m   print this menu\n"
			       "   I   get IDE drive info\n"
			       "   p   print partition table\n"
			       "   c   change number of cylinders\n"
			       "   h   change number of heads\n"
			       "   s   change number of sectors\n"
			       "   n   create new partition (unimplemented)\n"
			       "   q   quit without saving changes\n"
			       "   w   write changes and exit\n");
		}

	} 
}

