//  $Id: mksolo.C,v 1.6 1996/11/04 10:53:05 shaggy Exp shaggy $
//      
//      configure a disk to use SOLO
//      
//      relative path: ./sys/boot/m-pc
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       System files
//        Boot loader
//         (m-pc)
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

#define mksolo_VER_MAJOR	0
#define mksolo_VER_MINOR	24
#define mksolo_VER_PATCH	1
#define mksolo_VER_STR	"0.24.1"
#define mksolo_VER_DATE	"November 04, 1996"
#define mksolo_VER_NDATE	"11/04/96"
 
//  $Log: mksolo.C,v $
//  Revision 1.6  1996/11/04  10:53:05  shaggy
//  mksolo 0.24.1 (November 04, 1996: 05:53:01)
//  	- fixed: tags non-boot sectors with BOOTMERGE_MAGIC
//
//  Revision 1.5  1996/11/01  15:01:40  shaggy
//  mksolo 0.24.0 (November 01, 1996: 10:01:36)
//  	- working bootsector save/restore and creation of merge-able boot sector
//
//  --------------------------------------------------------------------------
//
//  Revision 1.4  1996/10/26  19:11:22  shaggy
//  header update
//
//  Revision 1.3  1996/10/05  14:28:30  shaggy
//  mksolo 0.23.1 (October 05, 1996: 10:28:28)
//  	fixed up some more version info
//
//  Revision 1.2  1996/10/05  00:40:21  shaggy
//  fixed to use new version info
//
//  Revision 1.1  1994/12/05  00:15:14  shaggy
//  Initial revision
//


#pragma Make Includes_Banned += -nostdinc -nostdinc++
#pragma Make Flags_Banned += -static

#define USE_STDLIB

#include <sys/types.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <linux/major.h>
#include <linux/fs.h>
#include <linux/fd.h>
#include <linux/hdreg.h>
#include <dirent.h>
#include <stdlib.h>

#include <types.h>
#include <stddef.h>
#include <iostream.h>
#include <fstream.h>
#include <strclass.h>
#include <time.h>

#include "ArgParse.h"
#include "boot.h"
#include <boot/mapfile.h>
#include "chunk.h"
#include "device.h"



bool debug = false;


static char rcsid[] __attribute__((unused))
	= "$Id: mksolo.C,v 1.6 1996/11/04 10:53:05 shaggy Exp shaggy $"
	  "$Release: " mksolo_VER_STR " $";


void find_device(string* devname, dev_t dev)
{
DIR* dir;
struct stat stbuf;
struct dirent* ent;
	dir = opendir("/dev");
	*devname = "";
	while((ent = readdir(dir))) {
		*devname = string("/dev/") + ent->d_name;
		if(stat(devname->chars(), &stbuf) < 0) {
			perror(ent->d_name);	
			exit(1);
		}
		if(stbuf.st_rdev == dev)
			break; 
		*devname = "";
	}
	closedir(dir);
	if(!devname->length()) {
		cerr << "Cannot find device " << hex << dev
			<< " in /dev" << endl; 
		exit(1);
	}
}


void find_device(string* devname, const char* fname)
{
struct stat stbuf;
	if(stat(fname, &stbuf) < 0) {
		perror(fname);
		exit(1);
	}
	find_device(devname, stbuf.st_dev);
}


struct Sector {
	int	head;
	int	sector;
	int	cyl;
};


class File_Geometry 
{
	int	sectors, heads, cylinders;
	int	start;	
	int	maxmult;
	int	block_size;
	int	sectors_per_block;
	int	sector_size;
	int	fd;
	int	size;
	string	devname;
	int	dev_id;
	bool	compact;
	int	nextblock;
	int	sectors_left;
	Sector	sec;
	uint32	mtime_hi;
	bool	linear;
	int	partition;
public:
	File_Geometry(const string fname, bool set_compact = false);
	~File_Geometry();
	bool	next_chunk(Chunk* c);
	Sector*	next_sector();
	Sector*	get_sector();
	int	get_size()		{ return size; }
	int	get_device()		{ return dev_id; }
	int	get_unit() { return linear ? dev_id|(partition<<8)|DISK_LINEAR
				: dev_id|(partition<<8); }
	time_t	get_mtime_hi()		{ return mtime_hi; }
};



File_Geometry::File_Geometry(string fname, bool set_compact)
{
int devfd;
struct stat stbuf;
	compact = set_compact;
	if((fd = open(fname.chars(), O_RDONLY)) < 0) {
		perror(fname.chars());
		exit(1);
	}
	fstat(fd, &stbuf);
	size = stbuf.st_size;
	// convert Unix time in seconds to Shag/OS time
	// in nanoseconds
	mtime_hi = ((int64)stbuf.st_mtime*(int64)1000000000) >> 32;
	if(stbuf.st_rdev)
		devfd = fd;
	else {
		find_device(&devname, stbuf.st_dev);
		if((devfd = open(devname.chars(), O_RDWR)) < 0) {
			perror(devname.chars());
			exit(1);
		}	
	}

	fstat(devfd, &stbuf);

	linear = false;	// use cylinder/head/sector
	partition = 0;	// whole disk

	switch(MAJOR(stbuf.st_rdev)) {
	case FLOPPY_MAJOR:
		struct floppy_struct fdgeo;
		if(ioctl(devfd, FDGETPRM, &fdgeo) < 0)
			perror("ioctl(FDGETPRM)");
		heads = fdgeo.head;
		cylinders = fdgeo.track;
		sectors = maxmult = fdgeo.sect;
		start = 0;
		dev_id = DISK_FD0 | (stbuf.st_rdev & 0x0f);
		break;
	case HD_MAJOR: case XT_DISK_MAJOR: case SCSI_DISK_MAJOR:
		struct hd_geometry hdgeo;
		if(ioctl(devfd, HDIO_GETGEO, &hdgeo) < 0)
		 	perror("ioctl(HDIO_GETGEO)");
		heads = hdgeo.heads;
		cylinders = hdgeo.cylinders;
		sectors = hdgeo.sectors;
		start = hdgeo.start;
		dev_id = DISK_HD0 | (MINOR(stbuf.st_rdev) >> 6); 
#if obsolete_code
		{
			struct hd_driveid hdid;
			string devname;
			int devfd;
			find_device(&devname, stbuf.st_rdev & 0xffc0);
			if((devfd = open(devname.chars(), O_RDWR)) < 0) {
				perror(devname.chars());
				exit(1);
			}
			hdid.max_multsect = 0;
			if(ioctl(devfd, HDIO_GET_IDENTITY, &hdid) < 0) {
				fprintf(stderr, "%s: ", devname.chars());
				perror("ioctl(HDIO_GET_IDENTITY)");
			}
			maxmult = hdid.max_multsect;
			if(!maxmult)
				maxmult = 4;	// should be safe...
		}
#else
		maxmult = sectors;
#endif
		break;
	default:
		cerr << "Unknown device type: "
			 << hex << stbuf.st_dev << endl;
	}
	sector_size = 512;
	if(devfd == fd) 
		size = cylinders*heads*sectors*sector_size;
	else
		close(devfd);

	sectors_left = 0;
	nextblock = 0;
	block_size = 0;
	ioctl(fd, FIGETBSZ, &block_size);
	if(!block_size)
		block_size = sector_size;
	sectors_per_block = block_size/sector_size;
	if(!compact && maxmult > sectors_per_block)
		maxmult = sectors_per_block;
	if(debug) {
		cout << "Disk params: " << hex << stbuf.st_rdev << '/'
			<< dev_id << " cylinders:" << dec << cylinders
			<< " heads:" << heads << " sectors:" << sectors
			<< " start:" << start 
			<< " maxmult:" << maxmult << endl;
		cout << "\tblocksize:" << block_size 
			<< " sectorsize:" << sector_size
			<< " size:" << size 
			<< " file: " << fname << endl;
	}
}



File_Geometry::~File_Geometry()
{
	close(fd);
}


Sector* File_Geometry::next_sector()
{
	if(--sectors_left > 0) {
		if(sec.sector && ++sec.sector > sectors) {
			sec.sector = 1;
			if(++sec.head >= heads) {
				sec.head = 0;
				sec.cyl++;
			}
		}
	} else {
		int block = nextblock++;
		if(block*block_size >= size) {
			sectors_left = 0;
			return (Sector*)0;
		}
		errno = 0;
		if(ioctl(fd, FIBMAP, &block) < 0 || block) {
			sec.sector = block*sectors_per_block+start;
			sec.cyl = sec.sector / (sectors*heads);
			sec.sector -= (sec.cyl*sectors*heads);
			sec.head = sec.sector / sectors;
			sec.sector -= (sec.head*sectors);
			sec.sector++;
		} else 
			sec.head = sec.cyl = sec.sector = 0;
		sectors_left = sectors_per_block;
	}
	return &sec;
}


Sector* File_Geometry::get_sector()
{
	if(sectors_left)
		return &sec;
	else
		return next_sector();
}


bool File_Geometry::next_chunk(Chunk *c)
{
Sector *sec;
int head;
int cyl;
int sector;
	if(!(sec = get_sector()))
		return false;
	cyl = sec->cyl;
	head = sec->head;
	sector = sec->sector;
	c->cyl1 = cyl&0xff;
	c->cyl2 = (cyl>>8)&3;
	c->cyl3 = (cyl>>10)&3;
	c->head = head;
	c->sector = sector;
	c->partial = false;
	c->small = false;
	c->count = 0;
	while((sec = get_sector()) && ((int)c->count < maxmult
			 || (!sector && c->count < 0x80))
			&& sec->cyl == cyl
			&& sec->head == head
			&& sec->sector == sector) {
		c->count++;
		if(sector)
			sector++;
		next_sector();
	}
	return true;
}




void save_boot_block(string dev, string save_fname)
{
struct BootBlock bootblock;
int fd;
	ifstream savefile_exists(save_fname.chars());
	if(savefile_exists.is_open()) {
		savefile_exists.close();
		return;
	}

	if((fd = open(dev.chars(), O_RDONLY)) < 0) {
		perror(dev.chars());
		exit(1);
	}
	read(fd, &bootblock, sizeof(bootblock));

	File_Geometry geo(dev.chars());
	Chunk chunk;
	if(!geo.next_chunk(&chunk)) {
		perror("next_chunk");
		exit(1);
	}
	if(bootblock.magic == BOOT_MAGIC) {
		bootblock.magic = BOOTMERGE_MAGIC;
		*(Chunk*)bootblock.sysname = chunk;
	}
	ofstream savefile(save_fname.chars(), ios::out | ios::noreplace);
	if(!savefile.is_open()) 
		return;
	cout << "saving bootsector from '" << dev << "' at "
		<< (chunk.cyl1 | (chunk.cyl2<<8) | (chunk.cyl3<<10)) << ","
		<< (int)chunk.head << "," << (int)chunk.sector << "  to '"
		<< save_fname << "'" << endl;
	savefile.write(&bootblock, sizeof(bootblock));
	close(fd);
	savefile.close();
}


void restore_boot_block(string dev, string save_fname)
{
struct BootBlock bootblock;
struct BootBlock oldblock;
int fd;
	ifstream savefile(save_fname.chars());
	if(!savefile.is_open()) {
		cout << "could not open saved bootsector '" << save_fname << "'" << endl;
		exit(1);
	}
	if((fd = open(dev.chars(), O_RDWR)) < 0) {
		perror(dev.chars());
		exit(1);
	}
	read(fd, &bootblock, sizeof(bootblock));
	savefile.read((char*)&oldblock, sizeof(oldblock));
	savefile.close();
	if(oldblock.magic == BOOTMERGE_MAGIC) {
		memcpy(bootblock.jmp, oldblock.jmp, sizeof(bootblock.jmp));
		memcpy(bootblock.code, oldblock.code, 
			sizeof(bootblock.code)+sizeof(bootblock.bootoffset)
			+sizeof(bootblock.chunk)
			+sizeof(bootblock.mapsize)+sizeof(bootblock.mapdev)
			+sizeof(bootblock.diskparam));
		bootblock.magic = BOOT_MAGIC;
	} else
		memcpy(&bootblock, &oldblock, sizeof(bootblock));

	lseek(fd, 0, SEEK_SET);
	write(fd, &bootblock, sizeof(bootblock));
	cout << "bootsector restored from '" << save_fname << "'" << endl;
	close(fd);
}


void write_boot_block(string dev, string bootblock_fname,
	 	File_Geometry& mapgeo, int bootoffset)
{
struct BootBlock bootblock;
int i;
int fd;
	if((fd = open(dev.chars(), O_RDWR)) < 0) {
		perror(dev.chars());
		exit(1);
	}
	read(fd, &bootblock, sizeof(bootblock));

	ifstream f(bootblock_fname.chars());
	if(!f.is_open()) {
		perror(bootblock_fname.chars());
		exit(1);
	}

	bootblock.jmp[0] = 0xe9;	// JMP
	bootblock.jmp[1] = (bootblock.code-bootblock.jmp)-3;
	bootblock.jmp[2] = 0x00;
	f.read((char*)&bootblock.code, sizeof(bootblock.code));
	f.close();
	i = 0;

	while(mapgeo.next_chunk(&bootblock.chunk[i]))
		i++;
	bootblock.mapsize = i;
	bootblock.magic = BOOT_MAGIC;
	bootblock.mapdev = mapgeo.get_device();
	bootblock.bootoffset = bootoffset;

	lseek(fd, 0, SEEK_SET);
	write(fd, &bootblock, sizeof(bootblock));
	close(fd);
	if(debug) {
		cout << "map size: " << (int)bootblock.mapsize << endl;
		for(i=0; i< bootblock.mapsize; i++) {
			Chunk &c = bootblock.chunk[i];
			cout << "head:" << c.head
			     << " cyl:"
			     << (c.cyl1 | (c.cyl2<<8) | (c.cyl3<<10))
			     << " sec:" << c.sector;
			if(c.partial) {
				if(c.small)
					cout << " offset:" 
					     << c.count*16
					     << " size:16" << endl;
				else
					cout << " offset:"
					     << (c.count&15)*32
					     << " size:"
					     << (32<<(c.count>>4)) << endl;
			} else 
				cout << " count:" << c.count << endl;
		}
	}
}


bool get_next_token(string &str, string &tok)
{
int i, p;
	for(i=0; str.chars()[i] && (str[i] <= ' ' || str[i] == '='); i++)
		continue; 
	if(!str.chars()[i] || str[i] == '#') {
		tok = "";
		return false;
	}

	p = i;
	for(; str.chars()[i] && str[i] > ' ' && str[i] != '='; i++)
		continue;
	tok = str(p, i-p);
	str = str(i, str.length()-i);
	return true;
}


int main(int argc, char **argv)
{
ArgParse arg(argc, argv);
ofstream mapf;
string rootdir = "/";
string bootdev;
string str;
string var, val, token;
string config_fname = "sys/solo.cnf";
string bootblock_fname, map_fname;
string fname;
struct MapFile mapfile;
bool compact = false;
bool uninstall = false;
int bootoffset = 0;
int mapoffset = 0;
bool boot_override = false;
	memset(&mapfile, 0, sizeof(mapfile));
	while(arg.more()) {
		if(arg.match("-h")) {
			cerr << "usage: " << argv[0]
			<< " [-d] [-r root dir] [-b boot device] "
			"<config_file>" << endl;
			exit(1);
		} else if(arg.match("-v")) {
			cout << "mksolo v" mksolo_VER_STR " (" 
				mksolo_VER_DATE ")" << endl;
			exit(1);
		} else if(!arg.match("-r", &rootdir) && 
		   !arg.match("-b", &bootdev) &&
		   !arg.match("-u", &uninstall) &&
		   !arg.match("-d", &debug)) {
			if(arg.prefix("-"))
				arg.badoption();
			else 
				arg.get(&config_fname);
		}
	}
	if(!bootdev.length()) 
		find_device(&bootdev, rootdir.chars());
	else
		boot_override = true;

	config_fname = rootdir + "/" + config_fname; 

	ifstream f(config_fname.chars());
	if(!f.is_open) {
		perror(config_fname.chars());
		exit(1);
	}
	
	while(readline(f, str)) {
		if(!get_next_token(str, var))
			continue;
		get_next_token(str, val);
		
		if(var == "compact")
			compact = true;
		else if(var == "bootblock")
			bootblock_fname = rootdir+"/"+val;
		else if(var == "boot") {
			if(boot_override)
				cout << "'boot' entry overridden" << endl;
			else
				bootdev = val;
		} else if(var == "saveboot") {
			if(uninstall)
				restore_boot_block(bootdev, rootdir+"/"+val);
			else
				save_boot_block(bootdev, rootdir+"/"+val);
		} else if(var == "map") {
			map_fname = rootdir+"/"+val;
			if(uninstall)
				continue;
			mapf.open(map_fname.chars());
			if(!mapf.is_open()) {
				perror(map_fname.chars());
				exit(1);
			}
		} else if(var == "image" || var == "loader") {
			if(val[0] == '/')
				val = val(1, val.length()-1);
			if(var == "loader") {
				bootoffset = mapoffset;
				if(debug)
					cout << "Loader: " << val << endl;
			}
			if(uninstall)
				continue;
			File_Geometry geo(rootdir+"/"+val, compact);
			mapfile.fsize = geo.get_size();
			mapfile.nchunks = 0;
			mapfile.device = DEV_DISK | DEVICE_MAPPED;
			mapfile.unit = geo.get_unit();
			mapfile.mtime_hi = geo.get_mtime_hi();
			mapfile.namelen = val.length()+1;
			// streampos top = mapf.seekoff(0, ios::cur);
			streampos top = mapf.tellp();
			mapf.write(&mapfile, sizeof(mapfile));
			mapoffset += sizeof(mapfile) + mapfile.namelen;
			mapf << val << '\0';

			Chunk c;
			while(geo.next_chunk(&c)) {
				mapf.write(&c, sizeof(c));
				mapoffset += sizeof(c);
				mapfile.nchunks++;
			}
			// streampos bot = mapf.seekoff(0, ios::cur);
			streampos bot = mapf.tellp();
			// mapf.seekpos(top);
			mapf.seekp(top);
			mapf.write(&mapfile, sizeof(mapfile));
			// mapf.seekpos(bot);
			mapf.seekp(bot);
		} else if(var == "device") {
			if(uninstall)
				continue;
			mapfile.nchunks = 0;
			get_next_token(str, token);
			int dev = strtol(token.chars(), 0, 0);
			mapfile.fsize = 0;
			mapfile.device = dev;
			get_next_token(str, token);
			mapfile.unit  = strtol(token.chars(), 0, 0);
			mapfile.fsize = 0;
			mapfile.mtime_hi =
				((int64)time(0)*(int64)1000000000) >> 32;
			mapfile.namelen = val.length()+1;
			mapf.write(&mapfile, sizeof(mapfile));
			mapoffset += sizeof(mapfile) + mapfile.namelen;
			mapf << val << '\0';
			if(debug) 
				cout << "Device: " << hex << (int)mapfile.device
					<< "/" << mapfile.unit
					<< " file: " << val << endl;
		} else {
			cerr << "unknown configuration variable: "
				<< var << endl;
			exit(1);
		}
	}
	if(!uninstall) {
		File_Geometry geo_map(map_fname, compact);
		mapfile.namelen = 0;
		mapf.write(&mapfile, sizeof(mapfile));
		f.close();
		mapf.close();
		write_boot_block(bootdev, bootblock_fname, geo_map, bootoffset);
	}
}

