//  $Id: readmap.C,v 1.3 1996/10/26 19:10:58 shaggy Exp $
//      
//      Read and display SOLO mapfiles	
//      
//      relative path: ./sys/boot/m-pc
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       System files
//        Boot loader
//         (m-pc)
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

  
//  $Log: readmap.C,v $
//  Revision 1.3  1996/10/26  19:10:58  shaggy
//  header update
//
//  Revision 1.2  1996/09/28  02:36:03  shaggy
//  updated for new headers
//
//  Revision 1.1  1994/12/18  01:27:14  shaggy
//  Initial revision
//

#pragma Make Includes_Banned = -nostdinc -nostdinc++
#pragma Make Flags_Banned = -static

#define USE_STDLIB

#include <stdio.h>

#include <types.h>
#include "boot.h"
#include <boot/mapfile.h>

static void readmap(FILE *f)
{
char str[256];
MapFile mapfile;
Chunk chunk;
int i;
int adr;
	while(fread(&mapfile, sizeof(mapfile), 1, f) && mapfile.namelen) {
		fread(str, 1, mapfile.namelen, f);
		printf("%s: nchunks=%d size=%d dev=0x%02X unit=0x%04X\n", str,
			mapfile.nchunks, mapfile.fsize, mapfile.device,
			mapfile.unit);
		adr = 0;
		for(i=0; i<mapfile.nchunks; i++) {
			fread(&chunk, sizeof(chunk), 1, f);
			if(chunk.partial)
				printf(
			 "  %d (%x):  cyl:%d  head:%d  sec:%d  %x:%x\n",
					i, adr, (chunk.cyl1)|(chunk.cyl2<<8)
					|(chunk.cyl3<<10), chunk.head, 
					chunk.sector, chunk.count, chunk.count);
			else
				printf(
			 "  %d (%x):  cyl:%d  head:%d  sec:%d  count:%d\n",
			   		i, adr, (chunk.cyl1)|(chunk.cyl2<<8)
					|(chunk.cyl3<<10), chunk.head,
					chunk.sector, chunk.count);
			adr += (SECTOR_SIZE*chunk.count);
		}
	}
}

int main(int argc, char **argv)
{
int i = 0;
FILE *f;
	if(argc == 1)
		readmap(stdin);
	while(++i < argc) {
		if(!strcmp(argv[i], "-"))
			readmap(stdin);
		else if(!(f = fopen(argv[i], "r"))) 
			perror(argv[i]);
		else {
			readmap(f);
			fclose(f);
		}
	}
	return 0;
}

