//  $Id: solotest.C,v 1.7 1996/10/26 19:11:32 shaggy Exp $
//      
//      Test SOLO BootParam interface
//      
//      relative path: ./sys/boot/m-pc
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       System files
//        Boot loader
//         (m-pc)
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

#define solotest_VER_MAJOR	0
#define solotest_VER_MINOR	95
#define solotest_VER_PATCH	3
#define solotest_VER_STR	"0.95.3"
#define solotest_VER_DATE	"October 06, 1996"
#define solotest_VER_NDATE	"10/06/96"
 
//  $Log: solotest.C,v $
//  Revision 1.7  1996/10/26  19:11:32  shaggy
//  header update
//
//  Revision 1.6  1996/10/06  12:27:01  shaggy
//  solotest 0.95.3 (October 06, 1996: 08:26:59)
//  	- added memory allocation testing
//
//  Revision 1.5  1996/10/05  11:56:12  shaggy
//  solotest 0.95.2 (October 05, 1996: 07:56:09)
//  	- prints correct SOLO version now
//
//  Revision 1.4  1996/10/05  11:53:07  shaggy
//  solotest 0.95.1 (October 05, 1996: 07:53:05)
//  	- fixed RCS version info
//
//  Revision 1.3  1996/10/05  11:09:35  shaggy
//  added mini debugger
//  added CPU reset test
//
//  Revision 1.2  1996/09/28  02:36:46  shaggy
//  added mini debugger stubs
//
//  Revision 1.1  1996/07/14  01:27:14  shaggy
//  Initial revision
//


#include <defines>
#include <boot/bootparam.h>
#include <boot/cpuinfo.h>
#include <boot/regblock.h>
#include <posix/string.h>
#include <posix/stdlib.h>
#include <boot/libsolo.h>
#include <hw/Port.h>

extern BootParam *bootparam;


static char rcsid[] __attribute__((unused))
	= "$Id: solotest.C,v 1.7 1996/10/26 19:11:32 shaggy Exp $"
	  "$Release: " solotest_VER_STR " $";



void show_file(const char *fname, int bufsize)
{
fd_t fd;
char buf[bufsize];
int n;
	printf("opening \"%s\"\n", fname);
	if((fd = open(fname, O_RDONLY)) < 0) {
		printf("no such file\n");
		return;
	}
	printf("fd=%x\n", fd);
	while((n = read(fd, buf, bufsize))) 
		write(STDOUT_FILENO, buf, n);
	printf("total bytes read: %ld\n", lseek(fd, 0, SEEK_CUR));
	close(fd);
}


void raw_con_test()
{
fd_t raw_con;
uchar ch;
	if((raw_con = open("/dev/rawcon", O_RDONLY)) < 0) {
		printf("cannot open /dev/rawcon\n");
		return;
	}
	printf("Press any key, or 'q' to quit\n");
	do {
		read(raw_con, &ch, 1);
		int n = ch;
		if(ch < 32) ch = '.';
		printf("you pressed key %d/0x%02x (%c)\n", n, n, ch);
	} while(ch != 'q');
	close(raw_con);
}



void mini_debug(char *cmdstr)
{
	char *cmd;
 
	cmd = strtok(cmdstr, " ");
	while(cmd) {
		bool port = false;
		if(*cmd == '%') {
			port = true;
			cmd++;
		}
	
		char *pos = strchr(cmdstr, '=');
		if(pos) {
			*pos++ = 0;
			uint32 adr = strtol(cmd, 0, 0);
			uint32 val = strtol(pos, 0, 0);
			if(port)
				(Port8(adr)) = val;
			else
				*(uint32*)adr = val;
		} else {
			uint32 adr = strtol(cmd, 0, 0);
			printf("0x%x\n", 
				port ?  Port8(adr) : *(uint32*)adr);
		}
		
		cmd = strtok(0, " ");
	}
}



main(int argc, char **argv)
{
int bufsize = 4096;
int x;
	printf("SOLO Test for SOLO v%d.%d.X\n", solotest_VER_MAJOR,
						solotest_VER_MINOR);  
	printf("This program was invoked with %d args\n", argc);
	for(int i=0; i<argc; i++)
		printf("  argv[%d] = \"%s\"\n", i, argv[i]);
	while(true) {
		printf("  1) Print BootParam info\n");
		printf("  2) Test console input\n");
		printf("  3) Test raw console input\n");
		printf("  4) Test file reading (bufsize=%d)\n", bufsize);
		printf("  5) Change buffer size\n");
		printf("  6) Test panic\n");
		printf("  7) Test exceptions/interrupts\n");
		printf("  8) Memory allocation tests\n");
		printf("  9) Test BIOS calls\n");
		printf("  d) Mini debugger command\n");
		printf("  q) Quit\n");

	get_command:
		printf("Enter option: ");
		char buf[80];
		if(!gets(buf, sizeof(buf)) || buf[0] == 'q')
			break;
		if(buf[0] == 'd') {
			mini_debug(&buf[1]);
			goto get_command;
		}

		printf("\n");
		switch(atoi(buf)) {
		case 1:
			printf("Bootparam = %p\n", bootparam);
			printf("Boot loader: %s v%d.%d (dev=0x%02x)\n",
				bootparam->bootname, 
				bootparam->bootver_major,
				bootparam->bootver_minor,
				bootparam->bootdev);
			printf("BIOS call buffer: %x-%x (%d bytes)\n",
				bootparam->buffer, 
				bootparam->buffer+bootparam->bufsize-1,
				bootparam->bufsize);
			printf("Entry=%x lomem=%x himem=%x\n",
				bootparam->entry, bootparam->lomem,
				bootparam->himem);
			printf("Base memory: %x-%x (%dK)\n",
				bootparam->base_addr, 
				bootparam->base_addr+bootparam->base_size-1,
				bootparam->base_size/1024);
			printf("base_lomem=%x base_himem=%x\n",
				bootparam->base_lomem, bootparam->base_himem);
			printf("Extended memory: %x-%x (%dK)\n",
				bootparam->ext_addr, 
				bootparam->ext_addr+bootparam->ext_size-1,
				bootparam->ext_size/1024);
			printf("Msgbuf=%x msgsize=%x\n",
				 bootparam->msgbuf, bootparam->msgsize);
			printf("Cpu: family=%d name=%s vendor=%s\n"
			       "     model=%d revision=%d type=%d flags=%08x\n",
				bootparam->cpuinfo->family,
				bootparam->cpuinfo->name,
				bootparam->cpuinfo->vendor,
				bootparam->cpuinfo->model,
				bootparam->cpuinfo->revision,
				bootparam->cpuinfo->type,
				bootparam->cpuinfo->flags);
			break;
		case 2:
			printf("Enter test string: ");
			if(gets(buf, sizeof(buf)))
				printf("got (%s)\n", buf);
			else
				printf("got EOF\n");
			break;
		case 3:
			raw_con_test();
			break;
		case 4:
			printf("Enter name of file to display: ");
			gets(buf, sizeof(buf));
			show_file(buf, bufsize);
			break;
		case 5:
			printf("Buffer size: ");
			gets(buf, sizeof(buf));
			bufsize = atoi(buf);
			break;
		case 6: {
			printf("Panic type (1=irq, 2=trap, 3=reset, 4=misc): ");
			gets(buf, sizeof(buf));
			int ptype = atoi(buf);
			printf("Panic number (irq/trap/misc #): ");
			gets(buf, sizeof(buf));
			int pnum = atoi(buf);
			printf("Extended error code: ");
			gets(buf, sizeof(buf));
			int pcode = atoi(buf);
			bootparam->panic(ptype, pnum, pcode, 0); 
			} break;
		case 7:
			printf("  0) Null pointer\n");
			printf("  1) Page fault\n");
			printf("  2) Division by 0\n");
			printf("  3) INT 3 (breakpoint)\n");
			printf("  4) IRQ\n");
			printf("  5) CPU reset\n");
			printf("Enter option: ");
			
			gets(buf, sizeof(buf));
			x = 0;
			switch(atoi(buf)) {
			case 0:
				printf("got byte: %d (?)\n", *(uint8*)0x0);
				break;
			case 1:
				printf("got byte: %d (?)\n", *(uint8*)0xdeadbeef);
				break;
			case 2:
				printf("answer=%d (?)\n", 123/x);
				break;
			case 3:
				asm("int $3");
				printf("returned from breakpoint (?)\n");
				break;
			case 4:
				printf("Turning on interrupts, enabling mask 0x0101,\n"
				       "and going into infinite loop...\n"
				       "press any key to cause an interrupt\n");
				asm("movb $1,%al; outb %al,$0x21; outb %al,$0xa1; sti");
				while(true) continue;
			case 5:
				asm("lidt 1f; int $3; 1: .long 0, 0");
				printf("(?)\n");
				break;
			}
			break;
		case 8:
			printf("The following commands are available:\n");
			printf("	m <size>	malloc(size)\n");
			printf("	r <adr> <size>	realloc(adr,size)\n");
			printf("	f <adr>		free(adr)\n");
			printf("	q		quit\n");
			while(true) {
				printf("cmd: ");
				if(!gets(buf, sizeof(buf)) || buf[0] == 'q')
					break;
				char *cp = &buf[1];
				uint32 n1 = strtoul(cp, &cp, 0);
				uint32 n2 = strtoul(cp, &cp, 0);
				switch(buf[0]) {
				case 'm':
					printf("%p\n", malloc(n1));
					break;
				case 'r':
					printf("%p\n", realloc((void*)n1, n2));
					break;
				case 'f':
					free((void*)n1);
					break;
				}
			}
			break;
		case 9:
			RegBlock reg;

			printf("  0) Change video mode (int 10)\n");
			printf("  1) Get KB character (int 16)\n");
			printf("Enter option: ");
			
			gets(buf, sizeof(buf));
			switch(atoi(buf)) {
			case 0:
				reg.eax = 0x0002;
				bootparam->callbios(0x10, &reg);
				break;
			case 1:
				reg.eax = 0x0000;
				bootparam->callbios(0x16, &reg);
				printf("key=%#x\n", reg.eax);
				break;
			default:
				printf("invalid option\n");
			}
			break;			
		default:
			printf("Invalid option\n");
		}
		printf("\n");
	} 
	printf("exiting\n");
}

