/*  vmem.h                                                      (CRC:40ab370d)
 *      segment and virtual memory definitions
 *      
 *      relative path: ./sys/boot/m-pc
 *      creation date: January 1995
 *      last modified: Sat Oct 26 15:10:55 1996
 *      header update: Sat Oct 26 15:10:56 1996
 *  
 *      Shag/OS: an Object-Oriented MicroKernel Operating System
 *       System files
 *        Boot loader
 *         (m-pc)
 *  
 *  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution.
 *  
 *  Shag/OS is free software for non-profit uses; it may be freely distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'COPYING'.  If this license was not received,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For bug reports, please read the file 'BUGS' first, then include the 
 *      product name and version, and the word 'bug' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

// Hardware-specific stuff:

#define PAGE_SIZE	4096		// 4K pages

#define MAXMEM_16BIT	0x10000		// 16-bit segment size: 64K
#define MAXMEM_BASE	0xa0000		// base memory: 640K 
#define MAXMEM_REAL	0x100000	// real-mode: 1 Megabyte
#define MAXMEM_DMA	0x1000000	// DMA: 16 Megabytes 


#define G_1BYTE 	0
#define G_4K		1
#define D_16BIT		0
#define D_32BIT		1
#define DPL_SYS		0
#define DPL_USER	3
#define S_SYSTEM	0
#define S_CODEDATA	1
#define P_PRESENT	1
#define P_NOTPRESENT	0
#define A_ACCESSED	1
#define A_NOTACCESSED	0
#define TYPE_DATA_RO	0x0
#define TYPE_DATA_RW	0x2
#define TYPE_DATA_RO_DN	0x4
#define TYPE_DATA_RW_DN	0x6
#define TYPE_EXEC_NR	0x8
#define TYPE_EXEC_R	0xa
#define TYPE_EXEC_NRC	0xc
#define TYPE_EXEC_RC	0xe
#define TYPE_TSS	0x9

#ifdef ASM
#define SEG_DESC(base,limit,g,d,dpl,type) \
	.word	limit&0xffff, base&0xffff, \
		[[base>>16]&0xff]|[type<<8]|[S_CODEDATA<<12] \
			|[dpl<<13]|[P_PRESENT<<15], \
		[[limit>>16]&0xf]|[d<<6]|[g<<7] \
			|[[[base>>24]&0xff]<<8]
#define TSS_DESC(base,limit,dpl)  \
		SEG_DESC(base,limit,G_1BYTE,D_16BIT,dpl,TYPE_TSS)
#endif


// initial segment layout for SOLO...

#define SEG_NULL	0x00	// the null descriptor
#define SEG_CS32	0x08	// CS segment to use in 32-bit mode
#define SEG_DS32	0x10	// DS segment to use in 32-bit mode
#define SEG_CS16	0x18	// CS segment to use in 16-bit mode 
#define SEG_DS16	0x20	// DS segment to use in 16-bit mode
#define SEG_TSS		0x24	// segment for task switches

// CR0 values:

#define CR0_PG		0x80000000
#define CR0_PE		0x00000001

