//  genofs.C                                                    (CRC:79dd7335)
//      relative path: ./util
//      creation date: Jun 1995
//      last modified: Sat Oct 26 15:12:47 1996
//      header update: Sat Oct 26 15:12:47 1996
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       Utilities for compilation, configuration, and development
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

//	generate offsets for data structures


// XXX - this should, of course, be modified to use CParse
//	(and maybe SymTab) to properly parse all 
//	varieties of code, instead of doing the cheats
//	that it does now...

//	Then, after it creates a real symbol table, it will
//	be simple to output the desired code.


#pragma Make Includes_Banned += -nostdinc -nostdinc++
#pragma Make Flags_Banned += -static

#include <String.h>
#include <Regex.h>
#include <iostream.h>
extern "C" {
#include <stdio.h>
}


bool getline(istream &is, String &s)
{
char buf[4096];
	if(!is.getline(buf, sizeof(buf)))
		return false;
	s = buf;
	return true;
}


Regex find_struct(String("\\(^\\|\\W+\\)struct\\W+"));
Regex find_class(String("\\(^\\|\\W+\\)class\\W+"));
Regex find_name(String("^\\w+\\(\\W\\|$\\)"));


void struct_ofs(String &result, String src, String sname, char **argv,
			 String prefix = "", String scope = "")
{
char buf[1024];
Regex ws(String("\\W*"));
Regex word(String("\\w+"));
Regex word_ws(String("\\w+\\W+"));
Regex var_decl(String("\\w+\\W*;"));
String fmt = "#define %s_%-32s 0x%04x\\n";
bool match = false;
	for(char **argp = argv; *argp; argp++)
		if(sname == *argp) {
			match = true;
			result += "\tprintf(\"\\n\");\n";
			break;
		}
	if(match) { 
		String struc = src;
		String vname;
		String vdecl;
		while(struc.contains(var_decl)) {
			vdecl = struc.through(var_decl);
			vdecl = vdecl.after(ws);
			struc = struc.after(var_decl);
			if(vdecl.contains(":")		// skip bitfields
			  || vdecl.contains("typedef") 	// type declarations
			  || vdecl.contains("{")	// or any code
			  || vdecl.contains("}")
			  || vdecl.contains("static")	// skip static vars
			  || vdecl.contains("extern")	// skip extern decls
			  || (vdecl.contains("const")
			      && !vdecl.contains("*")))	// skip constants
				continue;

			if(vdecl.contains("<"))
				while(vdecl.contains(">"))
					vdecl = vdecl.after(Regex(">\\W+"));
			else {
				do {
					vname = vdecl.through(word);
					vdecl = vdecl.after(word_ws);
				} while(vname == "const"
					|| vname == "static"
					|| vname == "volatile"
					|| vname == "mutable"
					|| vname == "auto");
			}
			do {
				vname = vdecl.through(word);
				vdecl = vdecl.after(word_ws);
			} while(vname == "int" ||  vname == "short"
			 || vname == "long" || vname == "char"
			 || vname == "float" || vname == "double"
			 || vname == "unsigned" || vname == "signed"
			 || vname == "wchar_t" || vname == "void"
			 || vname == "const" || vname == "volatile"
			 || vname == "static" || vname == "auto" );
	
			sprintf(buf, "\tprintf(\"%s\", "
			  "\"%s%s\", \"%s\", "
			  "(int)&((%s%s*)0)->%s);\n",
				fmt.chars(),
				prefix.chars(), sname.chars(), vname.chars(),
				scope.chars(), sname.chars(), vname.chars());
			result += buf;
		}
		sprintf(buf, "\tprintf(\"%s\", "
		  "\"%s%s\", \"sizeof\", "
		  "sizeof(%s%s));\n",
			fmt.chars(), prefix.chars(), sname.chars(),
				scope.chars(), sname.chars());
		result += buf;
		sprintf(buf, "\tif((1 << log2(sizeof(%s%s))) == sizeof(%s%s))\n",
			scope.chars(), sname.chars(),
			scope.chars(), sname.chars());
		result += buf;
		sprintf(buf, "\t\tprintf(\"%s\", "
		  "\"%s%s\", \"2sizeof\", "
		  "log2(sizeof(%s%s)));\n",
			fmt.chars(), prefix.chars(), sname.chars(),
				scope.chars(), sname.chars());
		result += buf;
	}
	while(true) {
		if(src.contains(find_struct))
			src = src.after(find_struct);
		else if(src.contains(find_class))
			src = src.after(find_class);
		else break;
		if(src.contains(find_name)) {
			String name = src.through(word);
			src = src.after(find_name);
			src = src.after("{");
			String struc = src.before("}");
			src = src.after("}");
			String str = struc;
			while(str.contains("{")) {
				str = str.after("{");
				str += src.before("}");
				struc += "}" + src.before("}");
				src = src.after("}");
			}
			String newprefix = prefix + sname + "_";
			String newscope = scope + sname + "::";
			struct_ofs(result, struc, name, argv,
			    newprefix, newscope);
			
		}	
	}	
}


void genofs(char **argv)
{
String line;
String result;
//	for(char **argp = argv; *argp; argp++) {
//		if(argp != argv)
//			line += "\\|";
//		line += String("\\(\\(^\\|\\W+\\)struct\\W+")
//			+ *argp + "\\(\\W\\|$\\)\\)";
//	}
//	Regex start(line);
	result += "int log2(int x)\n{\n\tint p=0;\n\n";
	result += "\twhile((x >>= 1) !=0) p++;\n";
	result += "\treturn p;\n}\n\n";

	result += "main()\n{\n";
	while(getline(cin, line)) {
		cout << line << endl;
		if(line.contains(find_struct)) 
			line = line.after(find_struct);
		else if (line.contains(find_class))
			line = line.after(find_class);
		else continue;
		String sname;
		if(line.contains(find_name)) {
			sname = line.through(Regex("\\w+"));
			line = line.after(find_name);
			goto found;
		}
		continue;
	found:
		while(!line.contains(Regex("{"))
			   && getline(cin, line))
			cout << line << endl;
		line = line.after("{");
		String struc;
		int level = 1;
		while(level > 0) {
			if(line.contains("}")) {
				String str = line.before("}");
				line = line.after("}");
				struc += str;
				while(str.contains("{")) {
					level++;
					str = str.after("{");
				}
				if(--level > 0) 
					struc += "}";
			} else {
				struc += line;
				while(line.contains("{")) {
					line = line.after("{");
					level++;
				}
				if(!getline(cin, line)) {
					cerr << "Error: missing }" << endl;
					exit(1);
				}
				cout << line << endl;
			}
		} 
		struct_ofs(result, struc, sname, argv);
	}
	result += "}\n";
	cout << "extern \"C\" int printf(const char *fmt, ...);" << endl;
	cout << result;
}


main(int, char **argv)
{
	genofs(&argv[1]);
}

