//  $Id: mkheader.C,v 1.4 1996/10/26 19:13:01 shaggy Exp $
//      make/change standard headers on source files
//      
//      relative path: ./util
//  
//      Shag/OS: an Object-Oriented MicroKernel Operating System
//       Utilities for compilation, configuration, and development
//  
//  Copyright (c) 1996 Frank E. Barrus.  All rights reserved.
//  
//  This file is a part of the Shag/OS operating system distribution.
//  
//  Shag/OS is free software for non-profit uses; it may be freely distributed
//  for non-profit purposes under the terms of the Shag/OS License, version 1.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
//  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
//  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
//  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
//  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
//  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
//  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
//  
//  You should have received a copy of the Shag/OS License along with this
//  program, in a file called 'COPYING'.  If this license was not received,
//  please contact the author and describe how this software was obtained.
//  
//  The author can be reached for questions, comments, and suggestions at:
//  
//      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
//      email:   shaggy@csh.rit.edu
//  
//      For bug reports, please read the file 'BUGS' first, then include the 
//      product name and version, and the word 'bug' in the subject.
//  
//  Complete, unaltered, original distributions of this and related code
//  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
//  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy

//  $Log: mkheader.C,v $
//  Revision 1.4  1996/10/26  19:13:01  shaggy
//  header update
//
//  Revision 1.3  1996/10/26  19:02:28  shaggy
//  fixed to remove redundant '.' directory purpose-printing
//
//  Revision 1.2  1996/09/20  12:32:10  shaggy
//  revised to support RCS headers
//
//  Revision 1.1  1995/10/29  10:07:55  shaggy
//  Initial revision
//


#pragma Make Includes_Banned += -nostdinc -nostdinc++
#pragma Make Flags_Banned += -static


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <limits.h>
#include <time.h>
#include <sys/stat.h>

#include <types.h>
#include <macro.h>
#include <ParsePath>


#define RELATIVE_PREFIX	"relative path: "
#define CREATION_PREFIX	"creation date: "
#define MODIFIED_PREFIX	"last modified: "
#define UPDATE_PREFIX	"header update: "


char* get_comment(char *s)
{
	if(s[0] == '#' && s[1] <= ' ') 
		s++;
	else if(((s[0] == '/' && s[1] == '/')
	    || ((s[0] == '/' || s[0] == ' ') && s[1] == '*')) && s[2] <= ' ')
		s += 2;
	else if(s[0] == ' ' && s[1] == '*' && s[2] == '/' && s[3] <= ' ')
		s += 3;
	else
		return 0;
	while(*s && *s != '\n' && *s <= ' ')
		s++;
	return s;
}


// XXX - should stat header to make sure files really need
// updating


bool make_header(const char *header, const char *fname, long htime)
{
ParsePath path(fname);
const char *top;
const char *left;
const char *bot;
FILE *f, *h, *fout;
char str[max(1024, PATH_MAX)];
char buf[max(1024, PATH_MAX)];
ulong cksum, old_cksum;
char tmpname[NAME_MAX];
char dir[NAME_MAX];
char lastmod[80];
char creation[80];
bool retval;
bool rcs = false;
char *s = 0;
struct stat st;
	printf("%s", fname);
	fflush(stdout);
	if(path.has_suffix(".C") || path.has_suffix(".H")) {
		top = "//  ";
		left = "//  ";
		bot = "";
	} else if(!path.get_ext(".")) {
		top = "#  ";
		left = "#  ";
		bot = "";
	} else {
		top = "/*  ";
		left = " *  ";
		bot = " */\n";
	}
	if(!(h = fopen(header?header:"/dev/null", "r"))) {
		perror(header);
		exit(1);
	}
	if(!(f = fopen(fname, "r"))) {
		perror(fname);
		exit(1);
	}
	old_cksum = 0;
	*creation = 0;
	if(fgets(str, sizeof(str), f)) {
		bool is_comment = get_comment(str);
		if(is_comment && strstr(str, "$Id"))
			rcs = true;
		else if(is_comment && (s = strstr(str, "(CRC:"))) {
			if(!(old_cksum = strtoul(&s[5], 0, 16))) 
				printf("***Warning: old_cksum == 0 (%s)\n",
					fname);
			while(fgets(str, sizeof(str), f)
			  && (s = get_comment(str))) {
				if(!strncmp(s, CREATION_PREFIX,
						strlen(CREATION_PREFIX))) 
					strcpy(creation, s);
				else if(!strncmp(s, MODIFIED_PREFIX,
						strlen(MODIFIED_PREFIX))) 
					strcpy(lastmod, s);
			}
			while(str[0] == '\n')
				fgets(str, sizeof(str), f);
		}
	} else {
		str[0] = 0;
		cksum = 0;
	}

	if(!rcs) {
		FILE *p = popen("cksum > _header.crc", "w");
		fprintf(p, "%s;%s", fname, path.get_dir());
		do {
			if(str[0])
				fputs(str, p);
		} while(fgets(str, sizeof(str), f));
		pclose(p);
		FILE *fcrc = fopen("_header.crc", "r");
		fscanf(fcrc, "%lu", &cksum);
		fclose(fcrc);
	} else
		cksum = 0;
	
	if(stat(fname, &st) < 0) {
		perror("stat fname");
		exit(1);
	}
	
	if(cksum == old_cksum && (htime < st.st_ctime
				 || htime < st.st_mtime)) {
		memset(buf, 32, strlen(fname));
		buf[strlen(fname)] = 0;
		printf("\r%s\r", buf);
		return false;
	}

	sprintf(tmpname, "%s.tmp", fname);
	if(!(fout = fopen(tmpname, "w"))) {
		perror(tmpname);
		exit(1);
	}

	rewind(f);
	if(!rcs)
		fprintf(fout, "%s%-60s(CRC:%8lx)\n", top,
			path.get_base(), cksum);
	if(old_cksum || rcs) {
		bool echo = true;
		fgets(str, sizeof(str), f);	// skip CRC line
		if(rcs) 
			fprintf(fout, "%s%s", top, get_comment(str));
		while(fgets(str, sizeof(str), f) && (s = get_comment(str))) {
			if(!strncmp(s, RELATIVE_PREFIX,
					strlen(RELATIVE_PREFIX)))
				echo = false;
			if(echo)
				fprintf(fout, "%s    %s", left, s);
		}
	} else
		str[0] = '\n';
	while(str[0] == '\n' && fgets(str, sizeof(str), f))
		continue;

	fprintf(fout, "%s    %s%s\n", left, RELATIVE_PREFIX, path.get_dir());
	if(!rcs) {
		if(*creation)
			fprintf(fout, "%s    %s", left, creation);
		else
			fprintf(fout, "%s    %s%s", left, CREATION_PREFIX,
					ctime(st.st_ctime < st.st_mtime
					 ? &st.st_ctime : &st.st_mtime));
		if(old_cksum != cksum) {
			sprintf(lastmod, "%s%s", MODIFIED_PREFIX, 
					ctime(st.st_ctime > st.st_mtime
					  ? &st.st_ctime : &st.st_mtime));
			retval = true;
		} else
			retval = false;
		fprintf(fout, "%s    %s", left, lastmod);

		if(stat(tmpname, &st) < 0) {
			perror("stat tmpname");
			exit(1);
		}

		fprintf(fout, "%s    %s%s", left, UPDATE_PREFIX,
				ctime(&st.st_ctime));
	} else
		retval = false;

	fprintf(fout, "%s\n", left);
	char indent[80];
	strcpy(indent, "    ");
	while(path.get_next_dir(dir)) {
		path.get_dir_path(buf);
		strcat(buf, "/.purpose");
		FILE *fp;
		if((fp = fopen(buf, "r"))) {
			while(fgets(buf, sizeof(buf), fp))
				fprintf(fout, "%s%s%s", left, indent, buf);
			fclose(fp);
		} else {
			if(strcmp(dir, "."))
				fprintf(fout, "%s%s(%s)\n", left, indent, dir);
		}
		strcat(indent, " ");
	}
	if(header)
		fprintf(fout, "%s\n", left);
	while(fgets(buf, sizeof(buf), h)) {
		fprintf(fout, "%s%s", left, buf);
	}
	fprintf(fout, "%s\n", bot);
	fclose(h);
	do {
		fputs(str, fout);
	} while(fgets(str, sizeof(str), f));
	fclose(f);
	fclose(fout);
	if(!(fout = fopen(fname, "w"))) {
		perror(fname);
		exit(1);
	}
	if(!(f = fopen(tmpname, "r"))) {
		perror(tmpname);
		exit(1);
	}
	while(fgets(str, sizeof(str), f)) 
		fputs(str, fout);
	fclose(f);
	fclose(fout);
	if(unlink(tmpname) < 0) {
		sprintf(buf, "unlink(\"%s\")", tmpname);
		perror(buf);
		exit(1);
	}
	if(retval) 
		printf("\r%s: %s", fname, lastmod);
	else
		printf("\r%s: updated header\n", fname);
	return retval;
}



main(int argc, char **argv)
{
char *header = 0;
char *source = 0;
unsigned long htime = 0;
	if(argc == 1) {
		fprintf(stderr,
		"usage: %s [-h headerfile] [-u] [-f pathfile] [file...]\n",
				argv[0]);
	}
	for(int i=1; i<argc; i++) {
		if(!strcmp(argv[i], "-h")) {
			header = argv[++i];
			struct stat st;
			if(stat(header, &st) < 0) {
				perror("stat");
				exit(1);
			}
			htime = st.st_ctime > st.st_mtime
				 ? st.st_ctime : st.st_mtime;
		} else if(!strcmp(argv[i], "-f"))
			source = argv[++i];
		else if (!strcmp(argv[i], "-u"))
			htime = LONG_MAX;
		else
			make_header(header, argv[i], htime);
	}
	if(source) {
		FILE *f;
		char str[PATH_MAX];
		char path[PATH_MAX];
		if(!strcmp(source, "-"))
			f = stdin;
		else
			if(!(f = fopen(source, "r"))) {
				perror(source);
				exit(1);
			}
		while(fgets(str, sizeof(str), f)) {
			sscanf(str, "%s", path);
			make_header(header, path, htime);
		}
		if(f != stdin)
			fclose(f);
	}
}

